; *************************************************************************
;                         Free Flight Computer
;
;                              DATA MODULE
;
; *************************************************************************
;
; Development Tool:     MPLAB v4.12.12
; File Creation:        20/01/00                RNH
; Last Update:          09/08/04                RNH
; Code Issue:           A                       RNH
;
; Global access RAM structure:
;
;       Bank 0    020 - 07F       96 bytes
;       Bank 1    0A0 - 0EF       80 bytes
;       Bank 2    110 - 16F       96 bytes
;       Bank 3    190 - 1EF       96 bytes
;
;
;       Bank 0 70 - 7F is mapped to all banks so they can be used anywhere.
;       If need to be addressed indirectly for some reason the appropriate
;       address needs to be loaded into INDF.  The uppermost 9th bit comes
;       from uppermost bank select.
;
;       Mappings to:    Bank 1    0F0 - 0FF
;                       Bank 2    170 - 1FF
;                       Bank 3    1F0 - 1FF


; Bank 0        (20 - 6F)       80 bytes
; ------        ---------
DP0     UDATA

FSR_COPY        res     1       ; Interrupt routine copy of fsr

QFE_OFF         res     2       ; 16 bit 2s complement - add to alt for QFE
QNH_OFF         res     2       ; 16 bit 2s complement - add to alt for QFE
MAXALTITUDE     res     2       ; signed 16 bit integer
MAXTAS          res     2       ; unsigned 16 bit integer

RX_BUFFER       res     d'16'   ; Receive buffer
RX_PTR          res     1       ;
RX_TMP          res     1       ; Buffer pointers
RX_END          res     1       ;
TX_BUFFER       res     d'16'   ; Transmit buffer
TX_PTR          res     1       ;
TX_END          res     1       ; Buffer pointers

ALTITUDE        res     2       ; Signed 2s complement QNE in feet.  Add QFE_OFF / QNH_OFF before use!
VARIO           res     1       ; Signed 2s complement.  5 = 100 fpm
TAS             res     1       ; True airspeed in mph
TEMPERATURE     res     1       ; 0 = -10 degrees C, 5 = -9 degrees C, 50 = 0 degrees C ...
IMPACTPRESSURE  res     1       ; ASI impact pressure reading from internal adc

ALT_M           res     3       ; 24 bit float for pres = m * volt + c
ALT_C           res     3       ;

ASI_M           res     3       ; 24 bit float for impactpressure /mbar = m * adc_reading + c
ASI_C           res     3       ;

BEEP_ON_PERIOD  res     1       ; units of 10ms to have sounder on for
BEEP_OFF_PERIOD res     1       ; units of 10ms to have sounder off for
BEEP_DUTY_COUNT res     1       ; current count; msb set if in on cycle

R11             res     1       ; general purpose bank 0 register
R12             res     1       ; general purpose bank 0 register
R13             res     1       ; general purpose bank 0 register
R14             res     1       ; general purpose bank 0 register
R15             res     1       ; general purpose bank 0 register
R16             res     1       ; general purpose bank 0 register

DGV_MSB         res     1       ; M.S.Byte of previous alti reading used
DGV_LSB         res     1       ; L.S.Byte of previous alti reading used
DGV_INIT        res     1       ; Countdown of adc reads till initialise dgv

spare_bank0     res     d'4'


; Shared Bank                   16 bytes
; -----------

DPX     UDATA

R0              res     1       ; general purpose inter-routine register
R1              res     1       ; general purpose inter-routine register
R2              res     1       ; general purpose inter-routine register
R3              res     1       ; general purpose inter-routine register
R4              res     1       ; general purpose inter-routine register
R5              res     1       ; general purpose inter-routine register
R6              res     1       ; general purpose inter-routine register
R7              res     1       ; general purpose inter-routine register
R8              res     1       ; general purpose inter-routine register
R9              res     1       ; general purpose inter-routine register
R10             res     1       ; general purpose inter-routine register
INT_WORK        res     1       ; Temporary work for interrupt routines
PCLATH_COPY     res     1       ; Used by interrupt routine
STATUS_COPY     res     1       ; Used by interrupt routine
WREG_COPY       res     1       ; Used by interrupt routine
FLAGS           res     1       ; 0 - quarter second tick incremented
                                ; 1 - time to increment barograph
                                ; 2 - timer is running
                                ; 3 - full update display please
                                ; 4 - update qfe (on screen 2)
                                ; 5 - update qnh (on screen 2)
                                ; 6 - barograph time remaining update required


; Bank 1        (A0 - EF)       80 bytes
; ------        ---------
DP1     UDATA

TEN_MS_TICK     res     1       ; 10ms tick, wraps at 25
QUARTER_TICK    res     1       ; quarter-seconds counter, wraps at 4
SECOND_TICK     res     1       ; seconds counter
TIMER_SEC       res     1       ; airtime seconds
TIMER_MIN       res     1       ; airtime minutes
TIMER_HOUR      res     1       ; airtime hours

AUTO_TICK       res     1       ; countdown reset when vario bleeps have happened ...
ACTIVE_TICK     res     1       ; ... when this count reaches a set number

KEY             res     1       ; key pressed
KEY_TICKS       res     1       ; arbitary tick - how many times into routine
KEY_DIV         res     1       ; rapid fire key divide

MMIFLAGS        res     1       ; 1,0 - vario type:
                                ;       00      no beeps
                                ;       01      vario beeps on up only
                                ;       02      vario beeps on down only
                                ;       03      vario beeps on up and down
                                ; 2 - start/stop timer automatically
                                ; 3 - selects which barograph store to use
                                ; 4 - master sound on/off; if set beeps on, else beeps off
                                ; 6,5 - qfe 00
                                ;       qnh 01
                                ;       qne 02
                                ; 7 - eeprom update required

; NOTE - currently leave baro store set to 0 (store 1); and copy to store 2
;       when requested by user.  Code in place in main.asm though to select
;       appropriate one.

BARO_TICK       res     1       ; countdown of seconds till next update
BARO_RATE       res     1       ; seconds between barograph update (min 2)
BARO_LEN        res     1       ; bytes in baro buffer to write to eeprom
                                ; (reset to ff when eeprom clear again)
BAROP_H         res     1       ; Barograph pointer (high byte)
BAROP_L         res     1       ; Barograph pointer (low byte)
BARO_END_H      res     1       ; Barograph end (high byte)
BARO_END_L      res     1       ; Barograph end (low byte)

HEADWIND        res     1       ; signed 2s complement, 4 x lsb = 1mph
DEST_HEIGHT     res     1       ; 0 = 0feet, 64 = 1000feet, 128 = 2000feet ...

PRESSURE        res     3       ; Latest pressure reading /mbar in floating

SCREEN          res     1       ; current screen
SELECT          res     1       ; general use for mmi
CURSORPOS       res     1       ; current cursor position on screen

GLIDE           res     1       ; Current glide angle; eg 10.5:1 would be stored as 105
GLIDE_DISTANCE  res     1       ; Distance till reach height in km; eg. 231 would be 23.1km
AVERAGE_VARIO   res     1       ; Signed 2s complement.  5 = 100 fpm
PEAK_VARIO      res     1       ; Signed 2s complement.  5 = 100 fpm
PEAK_VARIO_TICK res     1       ; reset peak vario reading when this second tick = 0

R17             res     1       ; general purpose bank 1 register
R18             res     1       ; general purpose bank 1 register (used by math.asm only)
R19             res     1       ; general purpose bank 1 register (used by math.asm only)
R20             res     1       ; general purpose bank 1 register (used by math.asm only)
R21             res     1       ; general purpose bank 1 register (used by math.asm only)
R22             res     1       ; general purpose bank 1 register (used by math.asm only)
R23             res     1       ; general purpose bank 1 register (used by math.asm only)
R24             res     1       ; general purpose bank 1 register (used by math.asm only)
R25             res     1       ; general purpose bank 1 register (used by math.asm only)
R26             res     1       ; general purpose bank 1 register (used by math.asm only)
R27             res     1       ; general purpose bank 1 register (used by math.asm only)
R28             res     1       ; general purpose bank 1 register (used by math.asm only)
R29             res     1       ; general purpose bank 1 register (used by math.asm only)
R30             res     1       ; general purpose bank 1 register (used by math.asm only)

VARIOBFR        res     d'8'    ; 8 seconds worth of VARIO readings
VARIOPTR        res     1       ; Pointer to where to store latest

spare_bank1     res     d'24'


; Bank 2        110 - 16F
; ------        ---------
DP2     UDATA

ADC0BFR         res     d'32'   ; most recent 16 adc0 (alt) words
ADCPTR          res     d'1'    ; where to write next adc word
ADCFRAME        res     d'1'    ; count-up of measure frame from 0 to 9 then wrap
ADCTICK         res     d'1'    ; from 0 to 24


; Bank 3        190 - 1EF
; ------        ---------
DP3     UDATA


; Export these labels


        ; Bank 0

        global  FSR_COPY
        global  QFE_OFF, QNH_OFF, MAXALTITUDE, MAXTAS
        global  RX_BUFFER, RX_PTR, RX_TMP, RX_END
        global  TX_BUFFER, TX_PTR, TX_END
        global  ALTITUDE, VARIO, TAS, TEMPERATURE, IMPACTPRESSURE
        global  BEEP_ON_PERIOD, BEEP_OFF_PERIOD, BEEP_DUTY_COUNT
        global  R11, R12, R13, R14, R15, R16

        global  DGV_MSB, DGV_LSB, DGV_INIT

        ; Bank 0 special

        global  R0, R1, R2, R3, R4, R5, R6, R7, R8, R9, R10
        global  INT_WORK, PCLATH_COPY, STATUS_COPY, WREG_COPY
        global  FLAGS


        ; Bank 1

        global  TEN_MS_TICK, QUARTER_TICK, SECOND_TICK, AUTO_TICK
        global  TIMER_SEC, TIMER_MIN, TIMER_HOUR, ACTIVE_TICK
        global  KEY, KEY_TICKS, KEY_DIV, MMIFLAGS
        global  BARO_TICK, BARO_RATE, BARO_LEN, BAROP_H, BAROP_L
        global  BARO_END_H, BARO_END_L
        global  HEADWIND, DEST_HEIGHT, PRESSURE

        global  SCREEN, SELECT, CURSORPOS, GLIDE, GLIDE_DISTANCE
        global  AVERAGE_VARIO, PEAK_VARIO, PEAK_VARIO_TICK, VARIOBFR, VARIOPTR
        global  ALT_M, ALT_C, ASI_M, ASI_C

        global  R17, R18, R19, R20, R21, R22, R23, R24
        global  R25, R26, R27, R28, R29, R30


        ; Bank 2

        global  ADCFRAME, ADCTICK, ADC0BFR, ADCPTR


        ; Bank 3


        end
