; *************************************************************************
;                         Free Flight Computer
;
;                            EEPROM MODULE
;
; *************************************************************************
;
; Development Tool:     MPLAB v4.12.12
; File Creation:        30/01/00                RNH
; Last Update:          22/04/04                RNH
; Code Issue:           A                       RNH
;
        list    P=16F76                 ; PIC 16F76
        include "P16C76r.INC"           ; r is my lower case version
        include "eealloc.inc"           ; eeprom allocations
        include "macro.inc"             ; my macros

        constant        SDA = 7         ; Port B.7
        constant        SCL = 0         ; Port C.0

        extern  R0, R1, R2, R3, R4, R5, R6, R7, R8, R9, R10, R17
        extern  SECOND_TICK, BAROP_H, BAROP_L, BARO_LEN, BARO_END_H, BARO_END_L
        extern  ALTITUDE, AVERAGE_VARIO, TAS, TEMPERATURE, QFE_OFF, FLAGS
        extern  TEN_MS_TICK, ADCTICK, BARO_RATE

_eeprom code


; ==========================================================================
;                                BAROGRAPH
; ==========================================================================
;
; Called every adctick for scheduling purposes.  Check if have something
; to do and if so then start writing when appropriate and according to ticks.

barograph

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****

        movf    BARO_LEN,w      ; barograph write in progress?
        btfss   status,z        ; skip if no
        goto    write_barograph

        btfss   FLAGS,1         ; time to start a new write?
        retlw   0               ; return if no


; Check if have any room left; ie is pointer at or beyond end marker?

        movf    BARO_END_H,w
        subwf   BAROP_H,w
        btfss   status,c        ; carry set if BAROP_H >= BARO_END_H (unsigned )
        goto    spaceavail      ; is clear, so must be less and thus space available

        movf    BARO_END_L,w
        addlw   d'256'-5        ; subtract 5 as need room for FF FF FF FF nn at end
        subwf   BAROP_L,w       
        btfss   status,c        ; carry set if BAROP_L >= BARO_END_L, ie no room.
        goto    spaceavail

        bcf     FLAGS,0         ; No room at the inn, can't do, clear pending
        goto	terminate_baro	; Write FF FF FF FF nn if haven't already done so


spaceavail

        bcf     status,rp0
        bsf     status,rp1      ; ... to bank 2   <--- **** NOTE ****
        movf    ADCTICK,w       ; only start off write on adctick of zero
        btfss   status,z
        retlw   0

        bsf     status,rp0
        bcf     status,rp1      ; ... to bank 1   <--- **** NOTE ****
        bcf     FLAGS,1         ; clear for next time

        movf    BAROP_H,w       ; check first to see if reached
        subwf   BARO_END_H,w    ; end of barograph
        btfss   status,z
        goto    initiate_entry_write
        movf    BAROP_L,w
        subwf   BARO_END_L,w
        btfss   status,z
        goto    initiate_entry_write
        retlw   0               ; reached end, no action

initiate_entry_write
        movlw   5               ;
        movwf   BARO_LEN        ; 5 bytes to write please
        retlw   0


; Store according to tick

write_barograph
        movf    BAROP_H,w
        movwf   R1
        movf    BAROP_L,w
        movwf   R0              ; base address to write to in R1R0
        bcf     status,rp0
        bsf     status,rp1      ; ... to bank 2   <--- **** NOTE ****

store_qfemsb
        movf    ADCTICK,w
        sublw   d'21'
        btfss   status,z
        goto    store_qfelsb
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        movf    ALTITUDE+1,w
        addwf   QFE_OFF+1,w     ; add l.s.bytes to affect carry if should
        movf    ALTITUDE,w      ; get m.s.byte
        btfsc   status,c
        incf    ALTITUDE,w      ; if carry set, get increment version instead
        addwf   QFE_OFF,w       ; add offset m.s.byte, now in wreg
        movwf   R8              ; store in R8 ready for writing
        movlw   d'0'            ; barograph byte 0
        goto    store_byte

store_qfelsb
        movf    ADCTICK,w
        sublw   d'22'
        btfss   status,z
        goto    store_vario
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        movf    ALTITUDE+1,w
        addwf   QFE_OFF+1,w     ; lsb in wreg
        movwf   R8              ; store in R8 ready for writing
        movlw   d'1'            ; barograph byte 1
        goto    store_byte

store_vario
        movf    ADCTICK,w
        sublw   d'10'
        btfss   status,z
        goto    store_temperature
        bcf     status,rp1      ;
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    AVERAGE_VARIO,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R8              ; store in R8 ready for writing
        movlw   d'2'            ; barograph byte 2
        goto    store_byte

store_temperature
        movf    ADCTICK,w
        sublw   d'9'
        btfss   status,z
        goto    store_tas
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        movf    TEMPERATURE,w
        movwf   R8              ; store in R8 ready for writing
        movlw   d'3'            ; barograph byte 3
        goto    store_byte

store_tas
        movf    ADCTICK,w
        sublw   d'8'
        btfss   status,z
        retlw   0               ; return if not a tick to store on
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        movf    TAS,w
        movwf   R8              ; store in R8 ready for writing
        movlw   d'4'            ; barograph byte 4

store_byte
        addwf   R0,f
        btfsc   status,c
        incf    R1,f            ; amend address to write to in R1R0
        call    write_eeprom    ; write byte in R8

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  BARO_LEN,f
        retlw   0               ; Just return if more to do else ...

        movf    BAROP_L,w       ; ... increment to next barograph location
        addlw   d'5'
        movwf   BAROP_L
        btfsc   status,c
        incf    BAROP_H,f
        bsf     FLAGS,6         ; update time remaining

; Check if now at end of barograph, if so terminate right here right now

        movf    BARO_END_H,w
        subwf   BAROP_H,w
        btfss   status,z
        retlw   0                 ; not equal, so not at end
        movf    BARO_END_L,w
        subwf   BAROP_L,w
        btfss   status,z
        retlw   0                 ; not equal, so not at end
                                  ; else run on to terminate routine below ...


; ==========================================================================
;                           TERMINATE BAROGRAPH
; ==========================================================================
;
; Write FF to mark end of this trace, and clear any update flag.
; Can store several traces in one bank.
; Miss some adc ticks this way but don't care; nobody will notice!
;
; Called in any bank
; Returns in undefined bank


terminate_baro

; ----- Cease any write in progress and copy current address to R1R0

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        clrf    BARO_LEN
        bcf     FLAGS,1         ; clear update flag

        movf    BARO_END_H,w    ; carry set if BAROP_H >= BARO_END_H
        subwf   BAROP_H,w       ; if so then possibly at end of baro
        btfss   status,c
        goto    write_marker
        movf    BARO_END_L,w    ; if low byte of pointer also >= then
        subwf   BAROP_L,w       ; at end
        btfss   status,c
        goto    write_marker
        bsf     FLAGS,6         ; update time remaining
        retlw   0               ; reached end, no action

write_marker
        movf    BAROP_H,w
        movwf   R1
        movf    BAROP_L,w
        movwf   R0

        call    wait_10ms       ; wait for any previous write to conclude

; ----- Now blat it (or next location it would have written to) with 0xff

        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        movlw   4               ; End marker is 0xFF FF FF FF (barorate)
        movwf   R6

write_end
        movlw   0xff
        movwf   R8
        clrwdt
        call    write_eeprom
        call    wait_10ms
        incf    R0,f
        btfsc   status,z
        incf    R1,f
        decfsz  R6,f
        goto    write_end

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    BARO_RATE,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R8
        clrwdt
        call    write_eeprom
        call    wait_10ms


; ----- Finally, advance R1R0 pointer past marker to where next
;       trace will start and store back in BARO_P in memory and eeprom.

        incf    R0,f            ;
        btfsc   status,z        ; move past last byte of marker
        incf    R1,f            ; (where stored BARO_RATE)

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R1,w
        movwf   BAROP_H
        movwf   R8
        movf    R0,w
        movwf   BAROP_L
        movwf   R9              ; in R8R9 for write_eeprom_16 routine
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        movlw   EE_BAROP_H
        movwf   R1
        movlw   EE_BAROP_L
        movwf   R0

        call    write_eeprom_16


; ----- All done!

        bsf     FLAGS,6         ; update time remaining
        retlw   0



; ==========================================================================
;                               READ EEPROM
; ==========================================================================
;
; Reads the data at the 11 bit address supplied in R1 R0, returns byte in R82
; Routine takes 0.523 ms to execute.
;
; Entry conditions:     Bank 0          (also exits in this bank)
;
; Alters on exit:       R2, R3, R4, R5, R17, Wreg, Carry (set if successful)
;

read_eeprom
        movlw   1
        movwf   R2                      ; only 1 byte to read


read_eeprom_start

; First, ensure SDA (left as input) output sits at low as would have been
; mucked about with through previous bit operations to this port.

        bcf     portb,SDA


read_init_timeout

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    TEN_MS_TICK,w
        addlw   3
        sublw   d'24'           ; w = 24 - incremented_tick
        btfsc   status,c        ; carry set if incremented_tick >= 25
        addlw   d'25'
        sublw   d'24'           ; w = 24 - (24 - incremented_tick + 25)
        movwf   R17             ; R17 = incremented_tick, -25 if applicable to wrap
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****


read_send_start
        call    send_start              ; send start sequence


read_send_device                        ; prepare device address word and r/nW
        movlw   b'10100000'             ; load device address, lsb write
        movwf   R5                      ; copy address into R5
        call    send_8bits              ; send it
        call    rx_ack                  ; check for acknowledge
        btfsc   status,c                ; check if successful
        goto    read_start_success      ; yup

        bsf     status,rp0              ; ... to bank 1   <--- **** NOTE ****
        movf    R17,w
        subwf   TEN_MS_TICK,w
        bcf     status,rp0              ; ... to bank 0   <--- **** NOTE ****
        btfsc   status,z
        retlw   0                       ; aborted (due to timeout)
        clrwdt
        goto    read_send_start


read_start_success


; Send high byte of address

        movf    R1,w
        movwf   R5
        call    send_8bits
        call    rx_ack
        btfss   status,c                ; check if successful
        retlw   0                       ; nope


; Send low byte of address

        movf    R0,w
        movwf   R5
        call    send_8bits
        call    rx_ack
        btfss   status,c                ; check if successful
        retlw   0                       ; nope


; Send start condition again

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bsf     trisb,SDA       ; SDA line high
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        call    wait_5us                ; wait 5us
        call    send_start


; Send device address again, this time for read

        movlw   b'10100001'             ; load device address, lsb read
        movwf   R5                      ; copy address into R5
        call    send_8bits              ; send it
        call    rx_ack                  ; check for acknowledge
        btfss   status,c                ; check if successful
        retlw   0                       ; nope


; Now can read byte(s)

read_first_byte
        call    read_8bits
        movf    R5,w
        movwf   R8                      ; store
        decfsz  R2,f
        goto    read_second_byte
        goto    conclude_read

read_second_byte
        call    tx_ack
        call    read_8bits
        movf    R5,w
        movwf   R9                      ; store
        decfsz  R2,f
        goto    read_third_byte
        goto    conclude_read

read_third_byte
        call    tx_ack
        call    read_8bits
        movf    R5,w
        movwf   R10                     ; store


; Conclude read - send nack then stop bit

conclude_read
        call    tx_nack
        call    send_stop


; Return data value

        bsf     status,c                ; success
        retlw   0



; ==========================================================================
;                               READ EEPROM 16
; ==========================================================================
;
; Reads the data at the 11 bit address supplied in R1 R0
; Returns bytes in R8 (msb) and R9
; Routine takes 0.523 ms to execute.
;
; Entry conditions:     Bank 0          (also exits in this bank)
;
; Alters on exit:       R2, R3, R4, R5, R17, Wreg, Carry (set if successful)


read_eeprom_16
        movlw   2
        movwf   R2
        goto    read_eeprom_start



; ==========================================================================
;                               READ EEPROM 24
; ==========================================================================
;
; Reads the data at the 11 bit address supplied in R1 R0
; Returns bytes in R8 (msb), R9, R10
; Routine takes 0.523 ms to execute.
;
; Entry conditions:     Bank 0          (also exits in this bank)
;
; Alters on exit:       R2, R3, R4, R5, Wreg, Carry (set if successful)


read_eeprom_24
        movlw   3
        movwf   R2
        goto    read_eeprom_start



; ==========================================================================
;                               WRITE EEPROM
; ==========================================================================
;
; Writes the data supplied in R8 at the address in R1 R0.
; Routine takes approx 0.55 ms to execute.
;
; Polls device to see if ready for 20-30ms so could take a while to return
; sometimes.  (write takes approx 10ms but routine returns before then)
;
; Entry conditions:     Bank 0.
;
; Alters on exit:       R2, R3, R4, R5, R17, Wreg, Carry (set if successful)


write_eeprom

; First, ensure SDA (left as input) output sits at low as would have been
; mucked about with through previous bit operations to this port.

        bcf     portb,SDA


write_init_timeout

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    TEN_MS_TICK,w
        addlw   3
        sublw   d'24'           ; w = 24 - incremented_tick
        btfsc   status,c        ; carry set if incremented_tick >= 25
        addlw   d'25'
        sublw   d'24'           ; w = 24 - (24 - incremented_tick + 25)
        movwf   R17             ; R17 = incremented_tick, -25 if applicable to wrap
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****


write_send_start
        call    send_start              ; send start sequence


write_send_device                       ; prepare device address word and r/nW
        movlw   b'10100000'             ; load device address, lsb write
        movwf   R5                      ; copy address into R5
        call    send_8bits              ; send it
        call    rx_ack                  ; check for acknowledge
        btfsc   status,c                ; check if successful
        goto    write_start_success     ; yup
        bsf     status,rp0              ; ... to bank 1   <--- **** NOTE ****
        movf    R17,w
        subwf   TEN_MS_TICK,w
        bcf     status,rp0              ; ... to bank 0   <--- **** NOTE ****
        btfsc   status,z
        retlw   0                       ; aborted (due to timeout)
        clrwdt
        goto    write_send_start


write_start_success


; Send high byte of address

        movf    R1,w
        movwf   R5
        call    send_8bits
        call    rx_ack
        btfss   status,c                ; check if successful
        retlw   0                       ; nop


; Send low byte of address

        movf    R0,w
        movwf   R5
        call    send_8bits
        call    rx_ack
        btfss   status,c                ; check if successful
        retlw   0                       ; nop


; Send first byte

write_first_byte
        movf    R8,w
        movwf   R5                      ; copy address into R5
        call    send_8bits              ; send it
        call    rx_ack                  ; check for acknowledge
        btfss   status,c                ; check if successful
        retlw   0                       ; nop


; Send stop bit

conclude_write
        call    send_stop
        bsf     status,c                ; success
        retlw   0



; ==========================================================================
;                               WRITE EEPROM 16
; ==========================================================================
;
; Writes the data supplied in R8 and R9 at the address in R1 R0.
; Routine takes approx 0.55 ms to execute, but must not be called again
; within 10ms and takes 10ms.  Corrupts R8.
;
; Use random access writes so don't have to worry about eeprom 32byte pages
;
; Entry conditions:     Bank 0.
;
; Alters on exit:       R0, R1, R2, R3, R4, R5, Wreg, Carry (set if successful)

write_eeprom_16
        call    write_eeprom
        btfss   status,c
        retlw   0               ; return if not successful
        call    wait_10ms

        movf    R9,w
        movwf   R8
        incf    R0,f
        movf    R0,w
        btfsc   status,z
        incf    R1,f
        call    write_eeprom

        retlw   0


; ==========================================================================
;                               WRITE EEPROM 24
; ==========================================================================
;
; Writes the data supplied in R8 and R9 at the address in R1 R0.
; Routine takes approx 0.55 ms to execute, but must not be called again
; within 10ms and takes 20ms.  Corrupts R8.
;
; Use random access writes so don't have to worry about eeprom 32byte pages
;
; Entry conditions:     Bank 0.
;
; Alters on exit:       R0, R1, R2, R3, R4, R5, Wreg, Carry (set if successful)

write_eeprom_24
        call    write_eeprom
        btfss   status,c
        retlw   0               ; return if not successful
        call    wait_10ms

        movf    R9,w
        movwf   R8
        incf    R0,f
        movf    R0,w
        btfsc   status,z
        incf    R1,f
        call    write_eeprom
        btfss   status,c
        retlw   0               ; return if not successful
        call    wait_10ms

        movf    R10,w
        movwf   R8
        incf    R0,f
        movf    R0,w
        btfsc   status,z
        incf    R1,f
        call    write_eeprom

        retlw   0


; ==========================================================================
;                            EEPROM SUB-ROUTINES!
; ==========================================================================


; *************
; * WAIT 10ms *
; *************
;
; 10ms is the minimum time after a write to an EEPROM access.
;
; Altered on exit:      Wreg, R3, R4
;
; 4MHz clock gives 1us per instruction cycle.


wait_10ms
        movlw   d'13'
        movwf   R4
wait_1ms                        ; 2 instruction cycles calling this routine
        movlw   d'191'          ; 1
        movwf   R3              ; 1
wait_1ms_loop
        nop                     ; 3 instructions except on last one
        decfsz  R3,f            ; when is 2
        goto    wait_1ms_loop   ;

        decfsz  R4,f
        goto    wait_1ms
        retlw   0



; ************
; * WAIT 5uS *
; ************
;
; 5us is the minimum adequate time for all timing operations on the EEPROM.
; The total time to call this routine and return is actually 6us.
;
; Altered on exit:      Wreg, R3
;
; 4MHz clock gives 1us per instruction cycle.

wait_5us                        ; 2 instruction cycles calling this routine
        nop                     ; 1 instruction cycle here
        retlw   0               ; 2 instruction cycles




; **************
; * SEND START *
; **************
;
; Transmits start sequence.
;
;       SDA initial = high
;       SCL initial = low
;
;       SDA exit = low
;       SCL exit = low, ready for normal operation
;
; Process:      Sets SCL
;               Waits 5us
;               Clears SDA  (ie. now output)
;               Waits 5us
;               Clears SCL
;               Waits 5us

send_start
        bsf     portc,SCL
        call    wait_5us

        bsf     status,rp0              ; ... to bank 1 <--- **** NOTE ****
        bcf     trisb,SDA               ; SDA line low (output zero)
        bcf     status,rp0              ; ... to bank 0 <--- **** NOTE ****

        call    wait_5us

        bcf     portc,SCL
        call    wait_5us                ; min 4us hold time!

        retlw   0



; ***************
; * SEND 8 BITS *
; ***************
;
; Transmits the byte in EE_WORK2 to the device
;
;       SDA initial = X
;       SCL initial = low
;
;       SDA exit = X
;       SCL exit = low, just gone
;
; Altered on exit:      Wreg
;

send_8bits
        movlw   8
        movwf   R4
sending

; ----- Setup SDA line

        rlf     R5,f            ; rotate through carry
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bcf     trisb,SDA       ; set low initially
        btfsc   status,c        ; skip if carry clear
        bsf     trisb,SDA       ; bit set, so set SDA high (by making input)
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        call    wait_5us        ; 5us


; ----- Clock it out

        bsf     portc,SCL
        call    wait_5us                ; 5us

        bcf     portc,SCL

        decfsz  R4,f
        goto    sending         ; send all 8 bits
        retlw   0



; ***************
; * READ 8 BITS *
; ***************
;
; Read byte into R5, need to send ack/nack after


read_8bits

; Read data
; SDA and SCL have just gone low after RX_ACK

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bsf     trisb,SDA       ; SDA line controlled by slave, now input
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        clrf    R5
        movlw   8
        movwf   R4
read_byte
        call    wait_5us                ; wait for data to stabilise
        bsf     portc,SCL

        call    wait_5us                ; wait for min high time

        bcf     status,c
        btfsc   portb,SDA               ; if SDA clear skip else
        bsf     status,c                ; set carry
        rlf     R5,f                    ; rotate through carry - ie
                                        ; collect SDA bit!

        bcf     portc,SCL

        decfsz  R4,f
        goto    read_byte               ; read in all 8 bits to R5

        retlw   0



; ***********
; * TX_NACK *
; ***********
;

tx_nack

; ----- Clock out a failed acknowledge to terminate read properly
;       SDA line still high as configured as input, so just clock out

        call    wait_5us
        bsf     portc,SCL
        call    wait_5us
        bcf     portc,SCL

        retlw   0



; **********
; * TX_ACK *
; **********
;

tx_ack

; ----- Clock out an acknowledge

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bcf     trisb,SDA       ; SDA line now output, and low
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        call    wait_5us
        bsf     portc,SCL
        call    wait_5us
        bcf     portc,SCL

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bsf     trisb,SDA       ; SDA line now input, and high
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        retlw   0



; **********
; * RX_ACK *
; **********
;
;       SDA initial = X
;       SCL initial = low
;
;       SDA exit = high
;       SCL exit = low, has been for 5us to allow its SDA to return hi Z
;
; Process:      Set SDA as input
;               Wait 5us
;               Set SCL
;               Wait 5us
;               Read SDA
;
;               if low- Clear SCL
;                       Wait 5us
;                       Set carry
;                       Return
;
;               else-   Clear SCL
;                       Wait 5us
;                       Send STOP sequence
;                       Clear carry
;                       Return
;
; Altered on exit:      Wreg
;

; Receive acknowledge

rx_ack
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bsf     trisb,SDA       ; SDA line now input
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        call    wait_5us

        bsf     portc,SCL
        call    wait_5us

        btfsc   portb,SDA       ; SDA should be low
        goto    eeprom_error    ; error if not

        bcf     portc,SCL

        call    wait_5us
        bsf     status,c        ; set carry - success
        retlw   0

eeprom_error
        bcf     portc,SCL

        call    wait_5us
        call    send_stop       ; send stop sequence
        bcf     status,c        ; clear carry - failure
        retlw   0




; *************
; * SEND STOP *
; *************
;
; Transmits stop sequence.  Must be called to terminate any EEPROM
; operation, not only for NM24C16 but because sets SDA back high (as input)
; again.
;
;       SDA initial = X
;       SCL initial = low
;
;       SDA exit = high
;       SCL exit = low
;
; Process:      Clears SDA
;               Waits 5us
;
;               Sets SCL
;               Waits 5us
;
;               Sets SDA
;               Waits 5us
;
;               Clears SCL
;               Waits 5us (in case immediate read next)

send_stop
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bcf     trisb,SDA       ; SDA line low
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        call    wait_5us

        bsf     portc,SCL
        call    wait_5us

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bsf     trisb,SDA       ; SDA line high (input)
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        call    wait_5us

        bcf     portc,SCL
        call    wait_5us

        retlw   0

; -------------------------------------------------------------------------


        global  barograph, terminate_baro
        global  read_eeprom, read_eeprom_16, read_eeprom_24
        global  write_eeprom, write_eeprom_16, write_eeprom_24
        global  wait_10ms

        end

