; *************************************************************************
;                         Free Flight Computer
;
;                              LCD MODULE
;
; *************************************************************************
;
; Development Tool:     MPLAB v4.12.12
; File Creation:        20/01/00                RNH
; Last Update:          09/08/04                RNH
; Code Issue:           A                       RNH
;
        list    P=16F76                         ; PIC 16F76
        include "P16C76r.INC"                   ; r is my lower case version
        include "macro.inc"                     ; my macros
        include "gpio.inc"                      ; hardware/configuration

        extern  R0, R1, R2, R3, R4, R5, R6, R7, FLAGS
        extern  R8, R9, R10
        extern  INT_WORK, PCLATH_COPY, STATUS_COPY, WREG_COPY
        extern  SCREEN, CURSORPOS

        extern  ALTITUDE, QNH_OFF, QFE_OFF, VARIO, TAS
        extern  PEAK_VARIO, AVERAGE_VARIO, GLIDE, GLIDE_DISTANCE
        extern  TEMPERATURE, TIMER_SEC, TIMER_MIN, TIMER_HOUR
        extern  PRESSURE, MAXALTITUDE, MAXTAS, MMIFLAGS, BARO_RATE
        extern  BAROP_H, BAROP_L, BARO_END_H, BARO_END_L

        extern  conv2int_unsigned


; ==========================================================================


_lcd    code


; ==========================================================================
;                               RESET LCD
; ==========================================================================
;
; Resets LCD after power up and put into 4 bit mode
; Called in bank 0.
; Alters R0.
;
;   RB0..RB3 = data4..7
;   RB4 = E
;   RB5 = RS (0 = cmd, 1 = data)
;   RB6 = RW (0 = write, 1 = read)
;   RB7 = bidirectional data for eeprom (so write 0 here)
;

reset_lcd


; Wait 15ms after voltage rail passed 4.5V ...

        call    wait_5ms
        call    wait_5ms
        call    wait_5ms


; Make lower nibble outputs ...

        clrf    portb
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   b'10000000'
        movwf   trisb
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****


; Issue first reset word ...

        movlw   b'00000011'     ; need to send this three times
        movwf   portb
        call    latch_nibble    ; clock it out


; Wait 5ms ...

        call    wait_5ms


; Now clock out remaining two init words, 100us in between

        call    latch_nibble    ; clock it out

        call    wait_50us
        call    wait_50us

        call    latch_nibble    ; clock it out

        call    wait_50us
        call    wait_50us


; Reset complete, now initialise into 4 bit mode ...

        movlw   b'00000010'     ; set 4 bit mode
        movwf   portb
        call    latch_nibble    ; clock it out
        call    wait_50us       ; min 40us


; Now in 4 bit mode, so have to clock out both upper and lower nibbles
; Haven't quite concluded init sequence yet, need to do following:


; 0010NFxx
; N = 1, F = 0 (two lines, 7 x 5)

        movlw   b'00000010'
        movwf   portb
        call    latch_nibble    ; clock it out
        movlw   b'00001000'
        movwf   portb
        call    latch_nibble    ; clock it out
        call    wait_50us       ; min 40us


; 00001000
; Display off

        movlw   b'00000000'
        movwf   portb
        call    latch_nibble    ; clock it out
        movlw   b'00001000'
        movwf   portb
        call    latch_nibble    ; clock it out
        call    wait_50us       ; min 40us


; 00000001
; Clear display

        movlw   b'00000000'
        movwf   portb
        call    latch_nibble    ; clock it out
        movlw   b'00000001'
        movwf   portb
        call    latch_nibble    ; clock it out
        call    wait_1ms
        call    wait_1ms        ; min 1.64ms


; 000001(I/D)S
; Entry mode set
; Increment but no display shift

        movlw   b'00000000'
        movwf   portb
        call    latch_nibble    ; clock it out
        movlw   b'00000110'
        movwf   portb
        call    latch_nibble    ; clock it out
        call    wait_50us       ; min 40us


; Program up user defined characters ...

        call    program_udgs


; Init now complete.  So lets turn the display on and party!
; Can (and should) now use routines which incorporate busy check ...

        movlw   b'00001100'     ; Display on, no cursor underline or blink
        call    send_4bit_cmd   ; send a 4 bit command

        retlw   0               ; all done!


; ==========================================================================
;                              Message 0
; ==========================================================================
;
; Clears display and writes "FFC"

message0
        movlw   b'00000001'     ; Clear display, cursor home
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'F'
        call    send_4bit_data
        call    send_4bit_data  ; another 'F'
        movlw   'C'
        call    send_4bit_data
        retlw   0


; ==========================================================================
;                              Message 1
; ==========================================================================
;
; Writes "Calibrating ..." to third line

message1
        movlw   LINE3           ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'C'
        call    send_4bit_data
        movlw   'a'
        call    send_4bit_data
        movlw   'l'
        call    send_4bit_data
        movlw   'i'
        call    send_4bit_data
        movlw   'b'
        call    send_4bit_data
        movlw   'r'
        call    send_4bit_data
        movlw   'a'
        call    send_4bit_data
        movlw   't'
        call    send_4bit_data
        movlw   'i'
        call    send_4bit_data
        movlw   'n'
        call    send_4bit_data
        movlw   'g'
        call    send_4bit_data
        movlw   ' '
        call    send_4bit_data
        movlw   '.'
        call    send_4bit_data
        call    send_4bit_data  ; '.'
        call    send_4bit_data  ; '.'
        retlw   0


; ==========================================================================
;                              Message 2
; ==========================================================================
;
; Writes "Reading data ..." to fourth line

message2
        movlw   LINE4           ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'R'
        call    send_4bit_data
        movlw   'e'
        call    send_4bit_data
        movlw   'a'
        call    send_4bit_data
        movlw   'd'
        call    send_4bit_data
        movlw   'i'
        call    send_4bit_data
        movlw   'n'
        call    send_4bit_data
        movlw   'g'
        call    send_4bit_data
        movlw   ' '
        call    send_4bit_data
        movlw   'D'
        call    send_4bit_data
        movlw   'a'
        call    send_4bit_data
        movlw   't'
        call    send_4bit_data
        movlw   'a'
        call    send_4bit_data
        movlw   ' '
        call    send_4bit_data
        movlw   '.'
        call    send_4bit_data
        call    send_4bit_data  ; '.'
        call    send_4bit_data  ; '.'
        retlw   0


; ==========================================================================
;                               WAIT 50us
; ==========================================================================

wait_50us
        movlw   d'13'           ; x 4us
        movwf   R0
wait_50us_loop
        nop
        decfsz  R0,f
        goto    wait_50us_loop
        retlw   0


; ==========================================================================
;                               WAIT 1ms
; ==========================================================================

wait_1ms
        movlw   d'250'
        movwf   R0
wait_1ms_inner
        nop
        decfsz  R0,f
        goto    wait_1ms_inner
        retlw   0


; ==========================================================================
;                               WAIT 5ms
; ==========================================================================

wait_5ms
        movlw   d'5'            ; x 1ms
        movwf   R1
wait_5ms_outer
        movlw   d'250'
        movwf   R0
wait_5ms_inner
        nop
        decfsz  R0,f
        goto    wait_5ms_inner
        decfsz  R1,f
        goto    wait_5ms_outer
        retlw   0


; ==========================================================================
;                               PROGRAM UDGS
; ==========================================================================

program_udgs
        movlw   b'01000000'     ; Move to CGRAM at address 0
        call    send_4bit_cmd   ; send a 4 bit command

        movlw   b'00111'        ; Char 00
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data

        movlw   b'11111'        ; Char 01
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        movlw   b'00000'
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data

        movlw   b'11111'        ; Char 02
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        movlw   b'00111'
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data

        movlw   b'00111'        ; Char 03
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        movlw   b'11111'
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data

        movlw   b'11111'        ; Char 04
        call    send_4bit_data
        call    send_4bit_data
        movlw   b'00000'
        call    send_4bit_data
        call    send_4bit_data
        movlw   b'11111'
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data

        movlw   b'11111'        ; Char 05
        call    send_4bit_data
        call    send_4bit_data
        movlw   b'00111'
        call    send_4bit_data
        call    send_4bit_data
        movlw   b'11111'
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data

        movlw   b'00111'        ; Char 06
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        movlw   b'00000'
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data

        movlw   b'00000'        ; Char 07
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        movlw   b'00111'
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data
        call    send_4bit_data

; Big numbers are constructed as follows:
;
; 0 = 02  1 = 0-  2 = 62  3 = 62  4 = 00
;     03      0-      04      75      -2
;
; 5 = 01  6 = 01  7 = 62  8 = 05  9 = 05
;     75      05      -0      03      -0
;
; (see table at end)
;
; Where - is a space (blank)
; Use 7- for decimal point

        movlw   b'10000000'     ; Move to Display RAM at address 0
        call    send_4bit_cmd   ; send a 4 bit command

        retlw   0



; ==========================================================================
;                               DRAW SCREEN
; ==========================================================================
;
; Clears the display and draws the screen.
; Exits via update_display which then fills in the information.

draw_screen
        movlw   b'00001100'     ; Display on, no cursor underline or blink
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   b'00000001'     ; Clear display, cursor home
        call    send_4bit_cmd   ; send a 4 bit command

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   0xff
        movwf   CURSORPOS       ; switch off cursor
        movf    SCREEN,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        btfsc   status,z
        goto    draw_screen0
        addlw   d'256'-1
        btfsc   status,z
        goto    draw_screen1
        addlw   d'256'-1
        btfsc   status,z
        goto    draw_screen2
        goto    draw_screen3


; ----- Screen 0:  Main

draw_screen0
        movlw   LINE4+d'9'      ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'm'
        call    send_4bit_data
        movlw   'p'
        call    send_4bit_data
        movlw   'h'
        call    send_4bit_data

        movlw   LINE3+d'13'     ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'V'
        call    send_4bit_data
        movlw   'R'
        call    send_4bit_data

        movlw   LINE4+d'13'     ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'P'
        call    send_4bit_data
        movlw   'K'
        call    send_4bit_data

        movlw   LINE2+d'17'     ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   7               ;
        call    send_4bit_data  ; Use dot defined above

        goto    full_update


; ----- Screen 1:  Second

draw_screen1
        movlw   'T'
        call    send_4bit_data
        movlw   'M'
        call    send_4bit_data
        movlw   'P'
        call    send_4bit_data

        movlw   LINE2           ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'Q'
        call    send_4bit_data
        movlw   'N'
        call    send_4bit_data
        movlw   'E'
        call    send_4bit_data

        movlw   LINE2+d'11'     ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'P'
        call    send_4bit_data
        movlw   'R'
        call    send_4bit_data
        movlw   'S'
        call    send_4bit_data

        movlw   LINE2+d'19'     ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'm'
        call    send_4bit_data

        movlw   LINE1+d'11'     ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'T'
        call    send_4bit_data

        movlw   LINE4           ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'M'
        call    send_4bit_data
        movlw   'a'
        call    send_4bit_data
        movlw   'x'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data
        movlw   'Q'
        call    send_4bit_data
        movlw   'F'
        call    send_4bit_data
        movlw   'E'
        call    send_4bit_data

        movlw   LINE4+d'13'     ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'T'
        call    send_4bit_data
        movlw   'A'
        call    send_4bit_data
        movlw   'S'
        call    send_4bit_data

        goto    full_update


; ----- Screen 2:  Config

draw_screen2
        movlw   'Q'
        call    send_4bit_data
        movlw   'F'
        call    send_4bit_data
        movlw   'E'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data

        movlw   LINE2           ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'Q'
        call    send_4bit_data
        movlw   'N'
        call    send_4bit_data
        movlw   'H'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data

        movlw   LINE1+d'11'     ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'S'
        call    send_4bit_data
        movlw   'N'
        call    send_4bit_data
        movlw   'D'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data

        movlw   LINE2+d'11'     ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'T'
        call    send_4bit_data
        movlw   'M'
        call    send_4bit_data
        movlw   'R'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data

        goto    full_update


; ----- Screen 3:  Baro

draw_screen3
        movlw   'R'
        call    send_4bit_data
        movlw   'a'
        call    send_4bit_data
        movlw   't'
        call    send_4bit_data
        movlw   'e'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data

        movlw   LINE1+d'9'      ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   's'
        call    send_4bit_data

        movlw   LINE1+d'13'
        call    send_4bit_cmd
        movlw   'B'
        call    send_4bit_data
        movlw   'a'
        call    send_4bit_data
        movlw   'r'
        call    send_4bit_data
        movlw   'o'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data


        movlw   LINE3
        call    send_4bit_cmd
        movlw   'T'
        call    send_4bit_data
        movlw   'i'
        call    send_4bit_data
        movlw   'm'
        call    send_4bit_data
        movlw   'e'
        call    send_4bit_data
        movlw   ' '
        call    send_4bit_data
        movlw   'l'
        call    send_4bit_data
        movlw   'e'
        call    send_4bit_data
        movlw   'f'
        call    send_4bit_data
        movlw   't'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data


        movlw   LINE3           ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'R'
        call    send_4bit_data
        movlw   'e'
        call    send_4bit_data
        movlw   's'
        call    send_4bit_data
        movlw   'e'
        call    send_4bit_data
        movlw   't'
        call    send_4bit_data
        movlw   ' '
        call    send_4bit_data
        movlw   't'
        call    send_4bit_data
        movlw   'o'
        call    send_4bit_data
        movlw   ' '
        call    send_4bit_data
        movlw   'o'
        call    send_4bit_data
        movlw   't'
        call    send_4bit_data
        movlw   'h'
        call    send_4bit_data
        movlw   'e'
        call    send_4bit_data
        movlw   'r'
        call    send_4bit_data
        movlw   ' '
        call    send_4bit_data
        movlw   'b'
        call    send_4bit_data
        movlw   'a'
        call    send_4bit_data
        movlw   'r'
        call    send_4bit_data
        movlw   'o'
        call    send_4bit_data
        movlw   '?'
        call    send_4bit_data

        movlw   LINE4           ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   ':'
        call    send_4bit_data
        movlw   'Y'
        call    send_4bit_data
        movlw   'e'
        call    send_4bit_data
        movlw   's'
        call    send_4bit_data

        movlw   LINE4+d'10'     ; Move
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   ':'
        call    send_4bit_data
        movlw   'N'
        call    send_4bit_data
        movlw   'o'
        call    send_4bit_data

        goto    full_update


; ==========================================================================
;                              UPDATE DISPLAY
; ==========================================================================
;
; Update screen as required.


update_display

        btfsc   FLAGS,3         ; full display update requested?
        goto    full_update

        btfsc   FLAGS,4         ; update qfe? (should only be set on screen 2)
        goto    qfe_update

        btfsc   FLAGS,5         ; update qnh? (should only be set on screen 2)
        goto    qnh_update

        retlw   0               ; nothing required so return



full_update
        bcf     FLAGS,3         ; clear request to service
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    SCREEN,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        btfsc   status,z
        goto    update_screen0
        addlw   d'256'-1
        btfsc   status,z
        goto    update_screen1
        addlw   d'256'-1
        btfsc   status,z
        goto    update_screen2
        goto    update_screen3



; ----- Update screen 0:  Main
;
; altitude                  9 (big font)
; up/down/nothing          10 (big font)
; rate             xx x    12 (big font, little dot drawn above)
; + more

update_screen0

        movlw   LINE3+d'8'      ; Move ready to display QNH / QFE
        call    send_4bit_cmd   ; send a 4 bit command
        movlw   'Q'
        call    send_4bit_data

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        btfss   MMIFLAGS,6
        goto    display_qfeqnh

display_qne
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   'N'
        call    send_4bit_data
        movlw   'E'
        call    send_4bit_data
        clrf    R1
        clrf    R0
        goto    display_altitude

display_qfeqnh
        btfsc   MMIFLAGS,5
        goto    display_qnh

display_qfe
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   'F'
        call    send_4bit_data
        movlw   'E'
        call    send_4bit_data
        movf    QFE_OFF,w
        movwf   R1
        movf    QFE_OFF+1,w
        movwf   R0
        goto    display_altitude

display_qnh
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   'N'
        call    send_4bit_data
        movlw   'H'
        call    send_4bit_data
        movf    QNH_OFF,w
        movwf   R1
        movf    QNH_OFF+1,w
        movwf   R0

display_altitude
        movf    ALTITUDE+1,w    ; add qfe/qnh offset (R1R0) to altitude
        addwf   R0,f
        btfsc   status,c
        incf    R1,f
        movf    ALTITUDE,w
        addwf   R1,f            ; display altitude now in R1R0, signed

        movlw   ' '
        movwf   R2              ; assume +ve altitude for now, blank for sign

        btfss   R1,7            ; if we are at -ve altititude then need
        goto    do_alt_sign     ; to do more, otherwise ready to display

        movlw   0x01            ; display a minus for sign
        movwf   R2
        comf    R1,f
        comf    R0,f
        incf    R0,f
        btfsc   status,z        ; 2s complement on R1R0 to
        incf    R1,f            ; make altitude positive

do_alt_sign

; Display altitude sign ...

        movf    R0,w
        movwf   R5
        movf    R1,w
        movwf   R6              ; copy R1R0 into R6R5, as corrupted below

        movlw   LINE2
        call    send_4bit_cmd
        movf    R2,w
        call    send_4bit_data

        movf    R5,w
        movwf   R0
        movf    R6,w
        movwf   R1              ; restore R1R0


; Now display altitude itself ...

        call    digitise        ; break down unsigned number in R1R0 into ...


        movlw   1               ; display at location 1
        movwf   R7
        movf    R6,w            ; ten thousands
        movwf   R8              ;
        btfsc   status,z        ;
        movlw   d'12'           ; blank if need to skip leading zero
        movwf   R6
        call    bignum          ; display ten thousands

        movlw   3               ; display thousands at location 3
        movwf   R7
        movf    R5,w            ;
        iorwf   R8,w            ; check if have had any non-zero yet
        movwf   R8              ;
        btfss   status,z
        goto    bigalt_thou     ; display proper number if yes, else ...
        movlw   d'12'
        movwf   R5              ; display a blank.
bigalt_thou
        movf    R5,w
        movwf   R6
        call    bignum

        movlw   5               ; display hundreds at location 5
        movwf   R7
        movf    R4,w            ;
        iorwf   R8,w            ; check if have had any non-zero yet
        movwf   R8              ;
        btfss   status,z
        goto    bigalt_hun      ; display proper number if yes, else ...
        movlw   d'12'
        movwf   R4              ; display a blank.
bigalt_hun
        movf    R4,w
        movwf   R6
        call    bignum

        movlw   7               ; display tens at location 7
        movwf   R7
        movf    R3,w            ;
        iorwf   R8,w            ; check if have had any non-zero yet
        movwf   R8              ;
        btfss   status,z
        goto    bigalt_ten      ; display proper number if yes, else ...
        movlw   d'12'
        movwf   R3              ; display a blank.
bigalt_ten
        movf    R3,w
        movwf   R6
        call    bignum

        movlw   9
        movwf   R7
        movf    R2,w
        movwf   R6
        call    bignum          ; display units


display_big_vario
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        btfsc   AVERAGE_VARIO,7 ; bit set if sinking
        goto    negvario
posvario
        movlw   ' '
        movwf   R2
        goto    disp_vario_sign
negvario
        movlw   0x01
        movwf   R2


disp_vario_sign
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

; Display vario sign ...

        movlw   LINE2+d'12'
        call    send_4bit_cmd
        movf    R2,w
        call    send_4bit_data

        clrf    R1
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    AVERAGE_VARIO,w ; 5 = 100up/down ...
        movwf   R0
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        btfss   R0,7            ; make positive if negative
        goto    disp_vario_rate
        comf    R0,f
        incf    R0,f


disp_vario_rate

        rlf     R0,f
        btfsc   status,c
        incf    R1,f            ; double so have 10 = 100fpm etc.
        call    digitise        ; break down unsigned number in R1R0 into ...

        movf    R4,w
        btfsc   status,z
        movlw   d'12'           ; turn any leading zero into a blank
        movwf   R6
        movlw   d'13'
        movwf   R7
        call    bignum          ; display hundreds

        movf    R3,w
        movwf   R6
        movlw   d'15'
        movwf   R7
        call    bignum          ; display tens

        movf    R2,w
        movwf   R6
        movlw   d'18'
        movwf   R7
        call    bignum          ; display units


; Now do small numbers

        movlw   LINE4+d'7'      ; display address
        call    send_4bit_cmd   ; Move to display air speed
        movf    TAS,w
        movwf   R0
        clrf    R1              ; clear upper byte
        call    digitise        ; get 100s, 10s and 1s in R4 R3 R2
        movf    R3,w
        btfsc   status,z
        movlw   ' '-'0'                 ; if zero, write a space instead
        addlw   '0'
        call    send_4bit_data
        movf    R2,w
        addlw   '0'
        call    send_4bit_data

        movlw   LINE3
        call    send_4bit_cmd   ; Move to display glide angle

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    GLIDE,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        movwf   R0
        clrf    R1              ; clear upper byte
        call    digitise        ; get 100s, 10s and 1s in R4 R3 R2
        movf    R4,w
        btfsc   status,z
        movlw   ' '-'0'                 ; if zero, write a space instead
        addlw   '0'
        call    send_4bit_data
        movf    R3,w
        addlw   '0'
        call    send_4bit_data
        movlw   '.'
        call    send_4bit_data
        movf    R2,w
        addlw   '0'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data
        movlw   '1'
        call    send_4bit_data

        movlw   LINE4
        call    send_4bit_cmd   ; Move to display glide distance
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    GLIDE_DISTANCE,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        movwf   R0
        clrf    R1              ; clear upper byte
        call    digitise        ; get 100s, 10s and 1s in R4 R3 R2
        movf    R4,w
        btfsc   status,z
        movlw   ' '-'0'                 ; if zero, write a space instead
        addlw   '0'
        call    send_4bit_data
        movf    R3,w
        addlw   '0'
        call    send_4bit_data
        movlw   '.'
        call    send_4bit_data
        movf    R2,w
        addlw   '0'
        call    send_4bit_data
        movlw   'k'
        call    send_4bit_data
        movlw   'm'
        call    send_4bit_data

        movf    VARIO,w         ; most recent full reading
        movwf   R0
        movlw   LINE3+d'15'     ; display address
        movwf   R7              ;
        call    display_vario

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    PEAK_VARIO,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R0
        movlw   LINE4+d'15'     ; display address
        movwf   R7              ;
        call    display_vario

        retlw   0



; ----- Update screen 1:  Second
;
; 4  TEMP     -xx.x     0 = =10C, 50 = 0C etc.
; 13 Timer    x:xx:xx
; 24 QNE     -xxxxx     don't display 10,000s if zero  (space at end instead)
; 35 PRS      xxxx
; 67 max qfe -xxxxx     don't display 10,000s if zero  (space at end instead)
; 77 max tas -xxx       don't display 100s if zero (space at end instead)


update_screen1

        movlw   LINE1+d'4'
        call    send_4bit_cmd           ; Move for +ve / -ve sign

        movf    TEMPERATURE,w
        addlw   d'256'-d'50'            ; carry set if >= 50 (ie. positive)
        movwf   R2                      ;
        btfsc   status,c
        goto    positive_temp

negative_temp
        movlw   '-'
        call    send_4bit_data
        movf    R2,w
        movwf   R0                      ; display sign, retrieve number
        comf    R0,f                    ;
        incf    R0,f                    ; invert so get number to display
        goto    display_temp

positive_temp
        movlw   ' '
        call    send_4bit_data
        movf    R2,w
        movwf   R0                      ; display sign, retrieve number

display_temp
        clrf    R1                      ; clear upper byte for word -> num calc
        bcf     status,c
        rlf     R0,f
        rlf     R1,f                    ; double R1R0 so is now temperature x 10

        call    digitise                ; get 100s, 10s and 1s in R4 R3 R2

        movf    R4,w
        btfsc   status,z
        movlw   ' '-'0'                 ; if zero, write a space instead
        addlw   '0'
        call    send_4bit_data
        movf    R3,w
        addlw   '0'
        call    send_4bit_data
        movlw   '.'
        call    send_4bit_data
        movf    R2,w
        addlw   '0'
        call    send_4bit_data


display_timer
        movlw   LINE1+d'13'
        call    send_4bit_cmd
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    TIMER_HOUR,w
        bcf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R0
        call    digitise_byte           ; get hundreds, tens and units into R4 R3 R2
        movf    R2,w
        addlw   '0'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    TIMER_MIN,w
        bcf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R0
        call    digitise_byte           ; get hundreds, tens and units into R4 R3 R2
        movf    R3,w
        addlw   '0'
        call    send_4bit_data
        movf    R2,w
        addlw   '0'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    TIMER_SEC,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R0
        call    digitise_byte           ; get hundreds, tens and units into R4 R3 R2
        movf    R3,w
        addlw   '0'
        call    send_4bit_data
        movf    R2,w
        addlw   '0'
        call    send_4bit_data

do_pressure
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    PRESSURE,w
        movwf   R8
        movf    PRESSURE+1,w
        movwf   R9
        movf    PRESSURE+2,w
        movwf   R10
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        setpage conv2int_unsigned
        call    conv2int_unsigned       ; returns in R9R10
        setpage display_pressure

display_pressure

        movf    R9,w
        movwf   R1
        movf    R10,w
        movwf   R0
        call    digitise

        movlw   LINE2+d'15'
        call    send_4bit_cmd   ; Move to position

        movf    R5,w            ; thousands
        addlw   '0'
        call    send_4bit_data

        movf    R4,w            ; hundreds
        addlw   '0'
        call    send_4bit_data

        movf    R3,w            ; tens
        addlw   '0'
        call    send_4bit_data

        movf    R2,w            ; units
        addlw   '0'
        call    send_4bit_data


do_qne
        movf    ALTITUDE,w
        movwf   R1
        movf    ALTITUDE+1,w
        movwf   R0
        movlw   LINE2+d'4'
        movwf   R7
        call    display_signed_word

do_maxqfe
        movf    QFE_OFF,w
        movwf   R1
        movf    QFE_OFF+1,w
        movwf   R0

        movf    MAXALTITUDE+1,w         ; add qfe offset (R1R0) to max altitude
        addwf   R0,f
        btfsc   status,c
        incf    R1,f
        movf    MAXALTITUDE,w
        addwf   R1,f                    ; display altitude now in R1R0, signed

        movlw   LINE4+d'7'
        movwf   R7
        call    display_signed_word

do_maxtas
        movlw   LINE4+d'17'
        movwf   R7
        movf    MAXTAS,w
        movwf   R0
        call    display_unsign_byte

        retlw   0



; ----- Update screen 2:  Config
;
; qfe             4
; qnh             24
; beep (up/down)  16
; tmr (auto ...)  36


update_screen2

        call    qfe_update      ; display qfe

        call    qnh_update      ; display qnh

do_beep_type
        movlw   LINE1+d'16'
        call    send_4bit_cmd   ; Move to position
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    MMIFLAGS,w
        andlw   b'00000011'
        btfss   status,z
        goto    beeps_updown
beeps_off
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   'O'
        call    send_4bit_data
        movlw   'f'
        call    send_4bit_data
        movlw   'f'
        call    send_4bit_data
        movlw   ' '
        call    send_4bit_data
        goto    done_beep_disp

beeps_updown
        btfss   MMIFLAGS,0
        goto    beep_downonly
        btfss   MMIFLAGS,1
        goto    beep_uponly
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   'U'
        call    send_4bit_data
        movlw   'p'
        call    send_4bit_data
        movlw   'D'
        call    send_4bit_data
        movlw   'n'
        call    send_4bit_data
        goto    done_beep_disp

beep_uponly
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   'U'
        call    send_4bit_data
        movlw   'p'
        call    send_4bit_data
        movlw   ' '
        call    send_4bit_data
        call    send_4bit_data  ; ' '
        goto    done_beep_disp

beep_downonly
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   'D'
        call    send_4bit_data
        movlw   'o'
        call    send_4bit_data
        movlw   'w'
        call    send_4bit_data
        movlw   'n'
        call    send_4bit_data
done_beep_disp

        movlw   LINE2+d'16'
        call    send_4bit_cmd   ; Move to position
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        btfsc   MMIFLAGS,2
        goto    auto_timer
manual_timer
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   'M'
        call    send_4bit_data
        movlw   'a'
        call    send_4bit_data
        movlw   'n'
        call    send_4bit_data
        movlw   ' '
        call    send_4bit_data
        goto    done_timer
auto_timer
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   'A'
        call    send_4bit_data
        movlw   'u'
        call    send_4bit_data
        movlw   't'
        call    send_4bit_data
        movlw   'o'
        call    send_4bit_data
done_timer

        retlw   0


; This code is kept separate as also used for partial updates
; to the screen.

qfe_update
        movf    QFE_OFF,w
        movwf   R1
        movf    QFE_OFF+1,w
        movwf   R0
        movf    ALTITUDE+1,w    ; add offset (R1R0) to altitude
        addwf   R0,f
        btfsc   status,c
        incf    R1,f
        movf    ALTITUDE,w
        addwf   R1,f            ; display altitude now in R1R0, signed
        movlw   LINE1+d'4'
        movwf   R7
        call    display_signed_word
        bcf     FLAGS,4
        retlw   0

qnh_update
        movf    QNH_OFF,w
        movwf   R1
        movf    QNH_OFF+1,w
        movwf   R0
        movf    ALTITUDE+1,w    ; add offset (R1R0) to altitude
        addwf   R0,f
        btfsc   status,c
        incf    R1,f
        movf    ALTITUDE,w
        addwf   R1,f            ; display altitude now in R1R0, signed
        movlw   LINE2+d'4'
        movwf   R7
        call    display_signed_word
        bcf     FLAGS,5
        retlw   0


; ----- Update screen 3:  Baro
;
; update rate (barorate)       6                   eg. 10


update_screen3

; First, update barograph rate shown on screen.

        movlw   LINE1+d'6'
        movwf   R7
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    BARO_RATE,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R0
        call    display_unsign_byte

; Now show which store we're using

        movlw   LINE1+d'19'
        call    send_4bit_cmd   ; Move to position
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   1               ; MMIFLAGS.3 = 0 is barograph store 1
        btfsc   MMIFLAGS,3
        movlw   2               ; MMIFLAGS.3 = 1 is barograph store 2
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        addlw   '0'
        call    send_4bit_data

; Now display barograph store remaining.  Only do this if need to as
; calculation is quite intensive:

        btfss   FLAGS,6         ; bit 6 set if need to display
        goto    done_baro
        bcf     FLAGS,6

; First, calculate number of bytes remaining

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****

        comf    BAROP_H,w
        movwf   R1
        comf    BAROP_L,w
        movwf   R0
        incf    R0,f
        btfsc   status,z
        incf    R1,f            ; R1R0 = -BARO_P

        movf    BARO_END_L,w
        addwf   R0,f
        btfsc   status,c
        incf    R1,f
        movf    BARO_END_H,w
        addwf   R1,f            ; R1R0 = BARO_END-BARO_P

; Divide by five to get number entries
; Put note in EEALLOC.INC to make sure IS multiple of five!!!

        clrwdt
        clrf    R3
        clrf    R2              ; R3R2 = 0
divide_bytes_by_five
        movf    R1,w
        iorwf   R0,w
        btfsc   status,z        ; Have we finished dividing out?
        goto    done_divbytes   ; (OK, assume will end up on even boundary!)
        movlw   d'256'-d'5'
        addwf   R0,f                    ; subtract 5
        btfss   status,c        ; if carry set then was >= 5
        decf    R1,f            ; if not then decrement msb
        incf    R2,f
        btfsc   status,z
        incf    R3,f
        goto    divide_bytes_by_five
done_divbytes


; Multiply by BARO_RATE for seconds

        clrwdt
        clrf    R0
        clrf    R1              ; R1R0 = 0
barorate_multiply
        movf    R2,w
        iorwf   R3,w
        btfsc   status,z
        goto    done_barorate_multiply
        movf    R2,w
        btfsc   status,z        ; if lsb is zero, will need to decrement msb as well
        decf    R3,f
        decf    R2,f
        movf    BARO_RATE,w
        addwf   R0,f
        btfsc   status,c
        incf    R1,f            ; R1R0 = R1R0 +  BARORATE
        goto    barorate_multiply
done_barorate_multiply


; Divide R1R0 into hours (R7), minutes (R8) and seconds (R9)
; 1 hour = 3600 seconds = 0E10

        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

divide_seconds_to_hours
        clrf    R7
divide_hours
        movlw   d'256'-0e       ; 0e10h = 3600
        addwf   R1,f
        btfss   status,c        ; if carry then still positive
                goto    done_hour_div1
        movlw   d'256'-010
        addwf   R0,f
        btfsc   status,c        ; if carry then still positive
        goto    inc_hours
        movf    R1,f            ; test R1 for zero
        btfsc   status,z
        goto    done_hour_div2  ; finished divide if msb already zero
        decf    R1,f
inc_hours
        incf    R7,f
        goto    divide_hours
done_hour_div2
        movlw   010
        addwf   R0,f            ; restore R0
done_hour_div1
        movlw   0e
        addwf   R1,f            ; restore R1


; R1R0 is seconds remaining after divided out hours (now in R7)
; 1 minute = 60 seconds

divide_seconds_to_mins
        clrf    R8
        movlw   d'256'-d'60'
divide_mins
        addwf   R0,f
        btfsc   status,c
        goto    inc_mins        ; carry set if lsb >= 60
        movf    R1,f            ; test R1 for zero
        btfsc   status,z
        goto    done_mins       ; if msb already zero, then done
        decf    R1,f
inc_mins
        incf    R8,f
        goto    divide_mins
done_mins
        movlw   d'60'
        addwf   R0,w            ; restore seconds into wreg
        movwf   R9              ; and store in R9 (seconds)


; Now have R7 as hours, R8 as mins and R9 as seconds remaining.  So display!
; as h:mm:ss


        movlw   LINE2+d'13'
        call    send_4bit_cmd

        movf    R7,w
        movwf   R0                      ; break down R0 into R4 R3 R2 digits
        call    digitise_byte           ; alters R1
        movf    R2,w
        addlw   '0'
        call    send_4bit_data          ; alters R0,R1
        movlw   ':'
        call    send_4bit_data

        movf    R8,w
        movwf   R0
        call    digitise_byte
        movf    R3,w
        addlw   '0'
        call    send_4bit_data
        movf    R2,w
        addlw   '0'
        call    send_4bit_data
        movlw   ':'
        call    send_4bit_data

        movf    R9,w
        movwf   R0
        call    digitise_byte
        movf    R3,w
        addlw   '0'
        call    send_4bit_data
        movf    R2,w
        addlw   '0'
        call    send_4bit_data

done_baro

        retlw   0



; ==========================================================================
;                              DIGITISE
; ==========================================================================
;
; Called with number in R1R0
; Outputs 10,000s, 1,000s, 100s, 10s and units in R6 R5 R4 R3 R2

digitise
        clrf    R2
        clrf    R3
        clrf    R4
        clrf    R5
        clrf    R6

find_tenthou                    ; 10000 = 39 * 256 + 16
        movlw   d'256'-d'39'
        addwf   R1,f            ;
        btfss   status,c        ; carry clear if R1 < 39
        goto    got_tenthou_a
        movlw   d'256'-d'16'
        addwf   R0,f
        btfss   status,c
        goto    dec_tenthou_upper
tenthou_inc
        incf    R6,f
        goto    find_tenthou
dec_tenthou_upper
        movf    R1,w
        btfsc   status,z
        goto    got_tenthou_b   ; upper nibble zero
        decf    R1,f
        goto    tenthou_inc     ; else decrement and go back for more
got_tenthou_b
        movlw   d'16'
        addwf   R0,f            ; restore check
got_tenthou_a
        movlw   d'39'
        addwf   R1,f            ; restore check

find_thou                       ; 1000 = 3 * 256 + 232
        movlw   d'256'-d'3'
        addwf   R1,f
        btfss   status,c
        goto    got_thou_a
        movlw   d'256'-d'232'
        addwf   R0,f
        btfss   status,c
        goto    dec_thou_upper
thou_inc
        incf    R5,f
        goto    find_thou
dec_thou_upper
        movf    R1,w
        btfsc   status,z
        goto    got_thou_b      ; upper nibble zero
        decf    R1,f
        goto    thou_inc        ; else decrement and go back for more
got_thou_b
        movlw   d'232'
        addwf   R0,f            ; restore check
got_thou_a
        movlw   d'3'
        addwf   R1,f            ; restore check

byte_entry
        movlw   d'256'-d'100'
find_hundreds
        addwf   R0,f
        btfss   status,c
        goto    hundreds_carry
        incf    R4,f
        goto    find_hundreds
hundreds_carry
        movf    R1,w
        btfsc   status,z
        goto    got_hundreds
        decf    R1,f
        incf    R4,f
        goto    byte_entry      ; as need to restore wreg value
got_hundreds
        movlw   d'100'
        addwf   R0,f            ; restore check

        movlw   d'256'-d'10'
find_tens
        addwf   R0,f
        btfss   status,c
        goto    got_tens
        incf    R3,f            ; increment tens
        goto    find_tens
got_tens
        movlw   d'10'
        addwf   R0,w            ; restore check into wreg
        movwf   R2              ; units are left over!

        retlw   0



; ==========================================================================
;                              DIGITISE_BYTE
; ==========================================================================
;
; Called with number in R0, alters R1.
; Outputs 100s, 10s and units in R4 R3 R2

digitise_byte
        clrf    R1                      ; so can use code above
        clrf    R2
        clrf    R3
        clrf    R4
        goto    byte_entry



; ==========================================================================
;                             BIG NUMBER!
; ==========================================================================
;
; Displays 2 x 2 digit R6 at position R7
;
; Note will only work on a pair of lines; eg. lines 1 & 2 and lines 3 & 4
; because just uses simple calculation for offset to next line.

bignum
        movf    R6,w                    ; range check digit first
        addlw   d'256'-d'13'            ; number should be less than 13
        btfss   status,c                ; carry set (overflows) if isn't
        goto    bignum_ok
        movlw   d'13'                   ; clip to 13
        movwf   R6                      ; display solid block so know o/range

bignum_ok
        bsf     R7,7                    ; make into display move command
        bcf     status,c
        rlf     R6,f                    ; multiply entry by four so
        rlf     R6,f                    ; points to start of char block

        setpage bignum_table            ; need explicit so loads all of pclath

        movf    R7,w
        call    send_4bit_cmd           ; move
        movf    R6,w
        call    bignum_table            ; get top-left block
        call    send_4bit_data

        incf    R7,w
        call    send_4bit_cmd           ; move
        incf    R6,f
        movf    R6,w
        call    bignum_table            ; get top-right block
        call    send_4bit_data

        movlw   (LINE2-LINE1)           ; Offset for next line
        addwf   R7,w
        movwf   R7                      ; save for next block
        call    send_4bit_cmd           ; move
        incf    R6,f
        movf    R6,w
        call    bignum_table            ; get bottom-left block
        call    send_4bit_data

        incf    R7,w
        call    send_4bit_cmd           ; move
        incf    R6,w
        call    bignum_table            ; get bottom-right block
        call    send_4bit_data

        retlw   0


; ==========================================================================
;                             DISPLAY SIGNED WORD
; ==========================================================================
;
; Displays signed word in R1R0 with -ve / space leftmost, left flushed
; R7 contains the move command address (eg. LINE1+8)

display_signed_word

        movf    R0,w
        movwf   R2                      ; save in R0
        movf    R1,w
        movwf   R3                      ; save in R3

        bsf     R7,7                    ; safety, in case not already done
        movf    R7,w
        call    send_4bit_cmd           ; move to position


; Display the sign

        movlw   ' '                     ; assume positive for now
        btfss   R3,7                    ; make positive if negative
        goto    display_sw_sign
        comf    R3,f
        comf    R2,f                    ; 1s complement
        incf    R2,f
        btfsc   status,z
        incf    R3,f                    ; 2s complement!
        movlw   '-'                     ; new sign
display_sw_sign
        call    send_4bit_data

        movf    R2,w
        movwf   R0
        movf    R3,w
        movwf   R1                      ; restore R1R0


; Now display the number ...

        call    digitise                ; get in R6 R5 R4 R3 R2
        clrf    R8                      ; non-zero if have had a zero
        clrf    R9                      ; number of spaces to blank at end

        incf    R9,f
        movf    R6,w
        iorwf   R8,f
        movf    R8,w
        btfsc   status,z
        goto    done_tenthou
        decf    R9,f
        movf    R6,w
        addlw   '0'
        call    send_4bit_data
done_tenthou

        incf    R9,f
        movf    R5,w
        iorwf   R8,f
        movf    R8,w
        btfsc   status,z
        goto    done_thou
        decf    R9,f
        movf    R5,w
        addlw   '0'
        call    send_4bit_data
done_thou

                                        ; hijack code below to
        goto    display_ub              ; display hundreds onwards



; ==========================================================================
;                             DISPLAY UNSIGNED BYTE
; ==========================================================================
;
; Displays unsigned byte in R0 left-flushed
; R7 contains the move command address (eg. LINE1+8)

display_unsign_byte

        call    digitise_byte           ; get hundreds, tens and units into R4 R3 R2

        bsf     R7,7                    ; safety, in case not already done
        movf    R7,w
        call    send_4bit_cmd           ; move to position

        clrf    R8                      ; non-zero if have had a zero
        clrf    R9                      ; number of spaces to blank at end


display_ub

        incf    R9,f
        movf    R4,w
        iorwf   R8,f
        movf    R8,w
        btfsc   status,z
        goto    done_hund
        decf    R9,f
        movf    R4,w
        addlw   '0'
        call    send_4bit_data
done_hund

        incf    R9,f
        movf    R3,w
        iorwf   R8,f
        movf    R8,w
        btfsc   status,z
        goto    done_ten
        decf    R9,f
        movf    R3,w
        addlw   '0'
        call    send_4bit_data
done_ten

        movf    R2,w
        addlw   '0'
        call    send_4bit_data

do_blank
        movf    R9,w
        btfsc   status,z
        goto    done_num
        decf    R9,f
        movlw   ' '
        call    send_4bit_data
        goto    do_blank

done_num
        retlw   0



; ==========================================================================
;                             DISPLAY VARIO
; ==========================================================================
;
; Similar as display number but with sign and a decimal place between tens
; and units.
; If tens is zero, writes a space instead
; R7 contains the move command address (eg. LINE1+8)


display_vario

        movf    R0,w
        movwf   R2              ; save in R2

        bsf     R7,7            ; safety, in case not already set
        movf    R7,w
        call    send_4bit_cmd   ; move to position

        movlw   ' '             ; assume positive for now
        btfss   R2,7            ; make positive if negative
        goto    display_vario_sign
        comf    R2,f
        incf    R2,f
        movlw   '-'
display_vario_sign
        call    send_4bit_data

        clrf    R1
        bcf     status,c
        rlf     R2,w
        movwf   R0
        btfsc   status,c
        incf    R1,f            ; double so get in instant displayable format

        call    digitise

        movf    R4,w
        btfsc   status,z        ;
        movlw   ' '-'0'         ; if zero, display a space instead
        addlw   '0'
        call    send_4bit_data

        movf    R3,w
        addlw   '0'
        call    send_4bit_data

        movlw   '.'
        call    send_4bit_data

        movf    R2,w
        addlw   '0'
        call    send_4bit_data

        retlw   0



; ==========================================================================
;                            SEND 4 BIT CMD
; ==========================================================================
;
; Called and returns in bank 0
; Sends out a 4 bit command
; Alters R0, R1.

;   RB0..RB3 = data4..7
;   RB4 = E
;   RB5 = RS (0 = cmd, 1 = data)
;   RB6 = RW (0 = write, 1 = read)
;   RB7 = bidirectional data for eeprom (so write 0 here)


send_4bit_cmd

        movwf   R1              ; store byte to send

        call    wait_nbusy

        rrf     R1,f
        rrf     R1,f
        rrf     R1,f            ; swap upper and lower nibbles so
        rrf     R1,f            ; send out upper nibble first

; Reduce noise by setting portb gradually ...

        movf    R1,w
        andlw   b'00000011'     ; RB7..RB2 zero
        movwf   portb
        movf    R1,w
        andlw   b'00001111'     ; RB7..RB4 zero
        movwf   portb

        call    latch_nibble    ; clock it out

        rrf     R1,f
        rrf     R1,f
        rrf     R1,f
        rrf     R1,f            ; get lower nibbles back
        rrf     R1,f            ; need 'extra' shift as go through carry

        movf    R1,w
        andlw   b'00000011'     ; RB7..RB2 zero
        movwf   portb
        movf    R1,w
        andlw   b'00001111'     ; RB7..RB4 zero
        movwf   portb

        call    latch_nibble    ; clock it out

        movf    R1,w            ; restore wreg so can call routine again
                                ; with same byte

        return                                  ; use return so don't destroy wreg



; ==========================================================================
;                            SEND 4 BIT DATA
; ==========================================================================
;
; Called and returns in bank 0
; Sends out 4 bit data
; Alters R0, R1

;   RB0..RB3 = data4..7
;   RB4 = E
;   RB5 = RS (0 = cmd, 1 = data)
;   RB6 = RW (0 = write, 1 = read)
;   RB7 = bidirectional data for eeprom (so write 0 here)

send_4bit_data

        movwf   R1

        call    wait_nbusy

        rrf     R1,f
        rrf     R1,f
        rrf     R1,f            ; swap upper and lower nibbles so ...
        rrf     R1,f            ; send out upper nibble first

; Reduce noise by setting portb gradually ...

        movf    R1,w
        andlw   b'00000011'     ; RB7..RB2 zero
        iorlw   b'00100000'     ; we are data
        movwf   portb
        movf    R1,w
        andlw   b'00001111'     ; RB7..RB4 zero
        iorlw   b'00100000'     ; we are data
        movwf   portb

        call    latch_nibble    ; clock it out

        rrf     R1,f
        rrf     R1,f
        rrf     R1,f
        rrf     R1,f            ; get lower nibbles back
        rrf     R1,f            ; need 'extra' shift as go through carry

        movf    R1,w
        andlw   b'00000011'     ; RB7..RB2 zero
        iorlw   b'00100000'     ; we are data
        movwf   portb
        movf    R1,w
        andlw   b'00001111'     ; RB7..RB4 zero
        iorlw   b'00100000'     ; we are data
        movwf   portb

        call    latch_nibble    ; clock it out

        movf    R1,w            ; restore wreg so can call routine again
                                ; with same byte

        return                                  ; use return so don't destroy wreg



; ==========================================================================
;                            WAIT FOR NOT-BUSY
; ==========================================================================
;
;
; Called and returns in bank 0
; Corrupts R0
;
;   RB0..RB3 = data4..7
;   RB4 = E
;   RB5 = RS (0 = cmd, 1 = data)
;   RB6 = RW (0 = write, 1 = read)
;   RB7 = bidirectional data for eeprom (so write 0 here)

wait_nbusy


; First, gradually set output bits to zero to remove charge

        bcf     portb,0
        nop
        bcf     portb,1
        nop
        bcf     portb,2
        nop
        bcf     portb,3


; Now set nibble as inputs

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   b'10001111'
        movwf   trisb
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****


; Poll and wait for not-busy (4 bit interface remember)

        movlw   b'01000000'     ; leave rb7 zero as used for eeprom
        movwf   portb           ; cmd, read, enable low, data low (so no
wait_nbusy_1                    ;   surprise when switch back to outputs)
        bsf     portb,4         ; enable
        nop                     ; pipeline thing
        movf    portb,w
        movwf   R0              ; read upper nibble into R0
        bcf     portb,4
        call    latch_nibble    ; clock in lower nibble but don't read
        btfsc   R0,3            ; wait until DB7 goes low
        goto    wait_nbusy_1


; Make lower nibble outputs again ...

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   b'10000000'
        movwf   trisb
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        retlw   0



; ==========================================================================
;                            LATCH NIBBLE
; ==========================================================================
;
; Simple routine to save code space, as used lots

latch_nibble
        bsf     portb,4
        nop                     ; pipeline
        bcf     portb,4         ; clock it out second time
        retlw   0


; ==========================================================================

_big_num        code

; Following table returns big number characters

bignum_table
        addwf   pcl,f

        retlw   0               ; '0'
        retlw   2
        retlw   0
        retlw   3

        retlw   0               ; '1'
        retlw   ' '
        retlw   0
        retlw   ' '

        retlw   6               ; '2'
        retlw   2
        retlw   0
        retlw   4

        retlw   6               ; '3'
        retlw   2
        retlw   7
        retlw   5

        retlw   0               ; '4'
        retlw   0
        retlw   ' '
        retlw   2

        retlw   0               ; '5'
        retlw   1
        retlw   7
        retlw   5

        retlw   0               ; '6'
        retlw   1
        retlw   0
        retlw   5

        retlw   6               ; '7'
        retlw   2
        retlw   ' '
        retlw   0

        retlw   0               ; '8'
        retlw   5
        retlw   0
        retlw   3

        retlw   0               ; '9'
        retlw   5
        retlw   ' '
        retlw   0

        retlw   1               ; 'up'
        retlw   1
        retlw   ' '
        retlw   ' '

        retlw   ' '             ; 'down'
        retlw   ' '
        retlw   0xFF
        retlw   0xFF

        retlw   ' '             ; blank
        retlw   ' '
        retlw   ' '
        retlw   ' '

        retlw   0xFF            ; 13 = invalid char - out of range!
        retlw   0xFF
        retlw   0xFF
        retlw   0xFF


; ==========================================================================

        global  reset_lcd, draw_screen, update_display, digitise
        global  message0, message1, message2
        global  send_4bit_cmd, send_4bit_data

        END



