; *************************************************************************
;                         Free Flight Computer
;
;                              MAIN MODULE
;
; *************************************************************************
;
; Development Tool:     MPLAB v4.12.12
; File Creation:        20/01/00                RNH
; Last Update:          23/11/04                RNH
; Code Issue:           A                       RNH
;
        list    P=16F76                         ; PIC 16F76
        include "P16C76r.INC"                   ; r is my lower case version
        include "eealloc.inc"                   ; eeprom allocations
        include "macro.inc"                     ; my macros
        include "gpio.inc"                      ; hardware/configuration

        extern  R0, R1, R2, R3, R4, R5, R6, R7, R8, R9, R10, FLAGS
        extern  INT_WORK, PCLATH_COPY, STATUS_COPY, WREG_COPY

        extern  FSR_COPY, QFE_OFF, QNH_OFF, MAXALTITUDE, MAXTAS
        extern  ALT_M, ALT_C, ASI_M, ASI_C
        extern  RX_BUFFER, RX_PTR, RX_TMP, RX_END, TX_BUFFER, TX_PTR, TX_END
        extern  ALTITUDE, VARIO, AVERAGE_VARIO, PEAK_VARIO, PEAK_VARIO_TICK
        extern  VARIOBFR, VARIOPTR, TAS, GLIDE, GLIDE_DISTANCE, TEMPERATURE

        extern  QUARTER_TICK, SECOND_TICK, TIMER_SEC, TIMER_MIN, TIMER_HOUR
        extern  ADCFRAME, ADCTICK, ADCPTR, AUTO_TICK, ACTIVE_TICK
        extern  KEY, KEY_TICKS, MMIFLAGS, KEY_TICKS, TEN_MS_TICK, QUARTER_TICK, SECOND_TICK
        extern  BARO_TICK, BARO_RATE, BARO_LEN, BAROP_H, BAROP_L, BARO_END_H, BARO_END_L
        extern  HEADWIND, DEST_HEIGHT
        extern  BEEP_ON_PERIOD, BEEP_OFF_PERIOD, BEEP_DUTY_COUNT

        extern  DGV_MSB, DGV_LSB, DGV_INIT, SCREEN

        extern  reset_lcd, draw_screen, message0, message1, message2
        extern  measure, update_display, barograph
        extern  scan_keypad, do_mmi, service_uart
        extern  reset_ext_adc, initiate
        extern  read_eeprom, write_eeprom
        extern  read_eeprom_16, write_eeprom_16
        extern  read_eeprom_24, write_eeprom_24
        extern  terminate_baro


; -------------------------------------------------------------------------

; ----- Set configuration bits

       __config        b'11111110110101'       ; code protection off
                                               ; disable int. brown out
                                               ; enable powerup timer (72ms)
                                               ; enable watchdog
                                               ; XT oscillator

;        __config        b'11111110110111'       ; code protection off
;                                                ; disable int. brown out
;                                                ; enable powerup timer (72ms)
;                                                ; enable watchdog
;                                                ; RC oscillator

; ----- Alti / vario update rate

        constant        TIMER_HI = h'd8'        ; overflows every 10ms
        constant        TIMER_LO = h'f0'-8      ; at 4MHz, x1 prescaler
                                                ; subtract 8 for addition time in interrupt
;#define SOUND_DEBUG

; ----- Start of code

_vectors        code

        goto    init            ; reset vector
        nop                     ;
        nop                     ;
        nop                     ;
        movwf   WREG_COPY       ; save wreg     continues below ...


_main           code

        movf    status,w        ;               SEE ROUTINE FOR
        movwf   STATUS_COPY     ; save status   DETAILS OF THIS BIT
        movf    pclath,w        ;
        movwf   PCLATH_COPY     ; save pclath (available from any bank)
        setpage interrupt       ; select this page
        goto    interrupt       ; goto interrupt


; ==========================================================================
;                            MAIN INITIALISATION
; ==========================================================================


init
        bcf     intcon,gie      ; disable interrupts
        clrwdt                  ; kick dog, will set timeout to 576ms later

        bcf     status, rp0     ;
        bcf     status, rp1     ; select bank 0 for direct file addressing
        bcf     status, irp     ; select bank 0/1 for indirect addressing


; ----- initialise ports
;
; Port A (mainly Analogue)
;       0       headwind        analogue
;       1       dest_height     analogue
;       2       nMeasure        output  (switches on temperature sensor)
;       3       temp            analogue
;       4       nadcready       i
;       5       adcdata         open collector io       default kept as i
;
; Port B (Display, Keyboard, Eeprom)
;       0       data0           io                      default kept as i
;       1       data1           io                      default kept as i
;       2       data2           io                      default kept as i
;       3       data3           io                      default kept as i
;       4       e               o
;       5       rs / keycol0    o
;       6       rw / keycol1    o
;       7       eedata          open collector io       default kept as i
;
; Port C (UART, Sounder, Serial clock)
;       0       sclk            o
;       1       pwm             o
;       2       keyrow0         i
;       3       keyrow1         i
;       4       keyrow2         i
;       5       nadcs           o
;       6       serialout       o
;       7       serialin        i

        movlw   b'00000100'     ; keep bit 5 set zero so can use as o/c.  See measure.asm.
        movwf   porta
        movlw   b'00001111'
        movwf   portb           ; ee.asm just ensure bit low when has to use
        movlw   b'11100000'
        movwf   portc

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****

        movlw   b'11111011'
        movwf   trisa
        movlw   b'10001111'
        movwf   trisb
        movlw   b'11011100'     ; note UART will override RC6
        movwf   trisc           ; to be an output (tx) pin


; ----- initialise / conclude port a initialisation
;               ADCON0 setup as -
;                       just over 1.6us conversion on 4MHz processor
;                       select channel 0 for now
;                       clear Go/nDone  (no A-D yet please)
;                       shut off A-D module for now  (bit 0)
;               ADCON1 setup as -
;                       RA0, RA1, RA3 as analogue inputs; RA2, RA5 digital
;                       Vref as VDD
;

        movlw   b'00000100'     ; setup ADCON1
        movwf   adcon1
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   b'01000000'     ; setup ADCON0
        movwf   adcon0


; ----- setup UART to 19200 8N1 async

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   d'12'           ; 19200 baud with BRGH = 1 and 4MHz

        movwf   spbrg
        movlw   b'10000110'     ; async, 8 bits, high speed
        movwf   txsta           ; asynchronous, 8 bits, high speed
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   b'10010000'     ;
        movwf   rcsta           ; asynchronous, 8 bits, continuous rx


; ----- setup interrupts etc.

;       first want to assign prescaler to wdt so have to run this code ...

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   b'11000101'     ; prescaler other than 1:1.  Port B pull-ups disabled.
        movwf   option_reg      ; and assign it to tmr0
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        clrf    tmr0            ; clear tmr0 and prescaler
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   b'11001101'     ; assign same prescaler to wdt.  Port B pull-ups disabled.
        movwf   option_reg      ;

;       OK, now prescaler given to wdt can initialise rest of option reg.

        clrwdt
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****

        movlw   b'00100001'     ; only want timer 1 overflow
        movwf   pie1            ; and rx uart interrupts
        movlw   0               ; disable CCP2 interrupt
        movwf   pie2            ;

        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        movlw   b'00000101'     ; enable timer 1, 1:1 prescaler, 16 bit
        movwf   t1con
        movlw   TIMER_HI
        movwf   tmr1h
        movlw   TIMER_LO
        movwf   tmr1l           ; initialise so overflows at update rate
        clrf    ccp1con         ; clear pwm / capture register (not used)

        movlw   b'00000001'     ; disable timer 2, 1:4 prescaler
        movwf   t2con           ; enable by setting T2CON.2, period in PR2 (bank 1)
        clrf    ccp2con         ; clear pwm / capture register (vario beep)


; ----- Reset LCD

        setpage reset_lcd
        call    reset_lcd       ; reset LCD
        clrwdt
        call    message0        ; "FFC"
        call    message1        ; "Calibrating ..."
        clrwdt


; ----- initialise external adc

        setpage reset_ext_adc
        call    reset_ext_adc
        setpage initialise_vars

initialise_vars

; ----- initialise altimeter variables

        clrf    TAS             ; zero true airspeed
        clrf    TEMPERATURE     ; zero temperature (so display is tidy)
        clrf    ALTITUDE
        clrf    ALTITUDE+1      ; set altitude (16 bit unsigned) to zero
        clrf    VARIO           ; zero vario
        clrf    BEEP_DUTY_COUNT ; zero beep count

        movlw   d'2'
        movwf   DGV_INIT        ; digital vario countdown till ready

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        clrf    GLIDE           ; zero glide angle
        clrf    GLIDE_DISTANCE  ; zero glide distance
        clrf    AVERAGE_VARIO
        clrf    PEAK_VARIO
        clrf    PEAK_VARIO_TICK
        clrf    VARIOBFR        ; zero vario averaging buffer & reset pointer
        clrf    VARIOBFR+1
        clrf    VARIOBFR+2
        clrf    VARIOBFR+3
        clrf    VARIOBFR+4
        clrf    VARIOBFR+5
        clrf    VARIOBFR+6
        clrf    VARIOBFR+7
        movlw   VARIOBFR
        movwf   VARIOPTR


; ----- initialise variables

        clrf    TEN_MS_TICK
        clrf    QUARTER_TICK
        clrf    SECOND_TICK
        clrf    TIMER_SEC
        clrf    TIMER_MIN
        clrf    TIMER_HOUR
        clrf    AUTO_TICK
        clrf    ACTIVE_TICK
        clrf    FLAGS
        clrf    KEY
        clrf    KEY_TICKS

        clrf    HEADWIND
        clrf    DEST_HEIGHT

        clrf    SCREEN

        bcf     status,rp0      ;
        bsf     status,rp1      ; ... to bank 2   <--- **** NOTE ****
        clrf    ADCPTR
        clrf    ADCTICK
        movlw   d'9'            ; initialise frame to
        movwf   ADCFRAME        ; special temperature read one


; ----- initialise UART variables

        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        movlw   RX_BUFFER
        movwf   RX_PTR
        movwf   RX_TMP
        movwf   RX_END
        movlw   TX_BUFFER
        movwf   TX_PTR
        movwf   TX_END


; ----- say now reading data

        setpage message2
        call    message2        ; "Reading data ..."
        setpage read0


; ----- read in EEPROM info

read0                           ; alt sensor M constant
        movlw   EE_ALTM_H
        movwf   R1
        movlw   EE_ALTM_L
        movwf   R0
        setpage read_eeprom_24  ; read 24 bit number
        call    read_eeprom_24  ; returns in R8, R9, R10 (consecutive)
        setpage read0_done
read0_done
        btfss   status,c        ; skip default if successful
        goto    read0_default   ; eeprom read failed - do default
        movf    R8,w
        movwf   ALT_M
        movf    R9,w
        movwf   ALT_M+1
        movf    R10,w
        movwf   ALT_M+2
        goto    read0_end       ; success!
read0_default
        movlw   078             ; m = 782023 = 0.00977403
        movwf   ALT_M           ; pressure/mbar = m * adc + c
        movlw   020
        movwf   ALT_M+1
        movlw   023             ; failed, so use default values
        movwf   ALT_M+2
read0_end


read1                           ; alt sensor C constant
        movlw   EE_ALTC_H
        movwf   R1
        movlw   EE_ALTC_L
        movwf   R0
        setpage read_eeprom_24  ; read 24 bit number
        call    read_eeprom_24  ; returns in R8, R9, R10 (consecutive)
        setpage read1_done
read1_done
        btfss   status,c        ; skip default if successful
        goto    read1_default   ; eeprom read failed - do default
        movf    R8,w
        movwf   ALT_C
        movf    R9,w
        movwf   ALT_C+1
        movf    R10,w
        movwf   ALT_C+2
        goto    read1_end       ; success!
read1_default
        movlw   087
        movwf   ALT_C           ; failed, so use default values
        movlw   049
        movwf   ALT_C+1         ; c = 8749af = 403.4mbar
        movlw   0af             ; pressure/mbar = m * adc + c
        movwf   ALT_C+2
read1_end



read10                          ; asi sensor M constant
        movlw   EE_ASIM_H
        movwf   R1
        movlw   EE_ASIM_L
        movwf   R0
        setpage read_eeprom_24  ; read 24 bit number
        call    read_eeprom_24  ; returns in R8, R9, R10 (consecutive)
        setpage read10_done
read10_done
        btfss   status,c        ; skip default if successful
        goto    read10_default  ; eeprom read failed - do default
        movf    R8,w
        movwf   ASI_M
        movf    R9,w
        movwf   ASI_M+1
        movf    R10,w
        movwf   ASI_M+2
        goto    read10_end      ; success!
read10_default
        movlw   079             ; m = 79400F = 0.023445
        movwf   ASI_M
        movlw   040
        movwf   ASI_M+1
        movlw   00f             ; failed, so use default values
        movwf   ASI_M+2
read10_end


read11                          ; asi sensor C constant
        movlw   EE_ASIC_H
        movwf   R1
        movlw   EE_ASIC_L
        movwf   R0
        setpage read_eeprom_24  ; read 24 bit number
        call    read_eeprom_24  ; returns in R8, R9, R10 (consecutive)
        setpage read11_done
read11_done
        btfss   status,c        ; skip default if successful
        goto    read11_default  ; eeprom read failed - do default
        movf    R8,w
        movwf   ASI_C
        movf    R9,w
        movwf   ASI_C+1
        movf    R10,w
        movwf   ASI_C+2
        goto    read11_end      ; success!
read11_default
        movlw   057             ; c = 570CBC = 0.0000000000001 = almost zero!
        movwf   ASI_C
        movlw   00C
        movwf   ASI_C+1
        movlw   0BC             ; failed, so use default values
        movwf   ASI_C+2
read11_end

read2                           ; QFE offset
        movlw   EE_QFEOFF_H
        movwf   R1
        movlw   EE_QFEOFF_L
        movwf   R0
        setpage read_eeprom_16  ; read 16 bit number
        call    read_eeprom_16  ; returns in R8, R9 (consecutive)
        setpage read2_done
read2_done
        clrf    QFE_OFF         ; zero, in case read fails
        clrf    QFE_OFF+1
        btfss   status,c        ; skip and store if successful
        goto    read3           ; eeprom read failed - stick with default
        movf    R8,w
        movwf   QFE_OFF
        movf    R9,w
        movwf   QFE_OFF+1


read3                           ; QNH offset
        movlw   EE_QNHOFF_H
        movwf   R1
        movlw   EE_QNHOFF_L
        movwf   R0
        setpage read_eeprom_16  ; read 16 bit number
        call    read_eeprom_16  ; returns in R8, R9 (consecutive)
        setpage read3_done
read3_done
        clrf    QNH_OFF         ; zero, in case read fails
        clrf    QNH_OFF+1
        btfss   status,c        ; skip and store if successful
        goto    read4           ; eeprom read failed - stick with default
        movf    R8,w
        movwf   QNH_OFF
        movf    R9,w
        movwf   QNH_OFF+1


read4                           ; MMI Flags
        movlw   EE_MMIFLAGS_H
        movwf   R1
        movlw   EE_MMIFLAGS_L
        movwf   R0
        setpage read_eeprom     ; read 8 bit number
        call    read_eeprom     ; returns in R8
        setpage read4_done
read4_done
        movlw   d'00000111'     ; use no-vb, baro store 1, auto timer
        btfss   status,c        ; beep up and down
        movwf   R8              ; copy default to R8 if eeprom problem
        movf    R8,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   MMIFLAGS
        bcf     MMIFLAGS,7      ; don't need to update, just read!
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

read5                           ; Maximum QFE recorded
        movlw   EE_MAXALTITUDE_H
        movwf   R1
        movlw   EE_MAXALTITUDE_L
        movwf   R0
        setpage read_eeprom_16  ; read 16 bit number
        call    read_eeprom_16  ; returns in R8, R9 (R8 being msb)
        setpage read5_done
read5_done
        clrf    MAXALTITUDE
        clrf    MAXALTITUDE+1   ; zero in case read failed
        btfss   status,c        ; skip and store if successful
        goto    read6           ; eeprom read failed - stick with default
        movf    R8,w
        movwf   MAXALTITUDE
        movf    R9,w
        movwf   MAXALTITUDE+1


read6                           ; Maximum TAS recorded
        movlw   EE_MAXTAS_H
        movwf   R1
        movlw   EE_MAXTAS_L
        movwf   R0
        setpage read_eeprom     ; read 8 bit number
        call  read_eeprom       ; returns in R8
        setpage read6_done
read6_done
        clrf    MAXTAS          ; zero in case read failed
        btfss   status,c        ; skip and store if successful
        goto    read8           ; eeprom read failed - stick with default
        movf    R8,w
        movwf   MAXTAS

read8                           ; Barograph rate
        movlw   EE_BARORATE_H
        movwf   R1
        movlw   EE_BARORATE_L
        movwf   R0
        setpage read_eeprom     ; read 8 bit number
        call    read_eeprom     ; returns in R2
        setpage read8_done
read8_done
        movlw   d'5'            ; default of 5 seconds
        btfss   status,c
        movwf   R8              ; copy default to R8 if eeprom problem
        movf    R8,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   BARO_RATE


read9                      ; Barograph pointer
        movlw   EE_BAROP_H
        movwf   R1
        movlw   EE_BAROP_L
        movwf   R0
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        setpage read_eeprom_16  ; read 16 bit number
        call    read_eeprom_16  ; returns in R8, R9 (R8 being msb)
        setpage read9_done
read9_done
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R8,w
        sublw   0xff
        btfss   status,z
        goto    conclude_baroptr_init
        movf    R9,w
        sublw   0xff
        btfss   status,z
        goto    conclude_baroptr_init

; Have read 0xffff from eeprom for barograph pointer, assume uninitialised
; eeprom therefore and setup with something sensible instead ...

        btfsc   MMIFLAGS,3      ; which barograph store are we using?
        goto    initforbaro2
        movlw   EE_BARO1_H      ; initialise for barograph store 1
        movwf   R8
        movlw   EE_BARO1_L
        movwf   R9
        goto    conclude_baroptr_init
initforbaro2                ; initialise for barograph store 2
        movlw   EE_BARO2_H
        movwf   R8
        movlw   EE_BARO2_L
        movwf   R9


; ----- initialise barograph

conclude_baroptr_init
        movf    R8,w
        movwf   BAROP_H
        movf    R9,w
        movwf   BAROP_L

; Setup end of barograph according to which bank we're in ...

        comf    R8,f
        comf    R9,f
        incf    R9,f
        btfsc   status,z
        incf    R8,f            ; R8R9 = -BAROP

        movlw   EE_BARO1_ENDL
        addwf   R9,f
        btfsc   status,c
        incf    R8,f
        movlw   EE_BARO1_ENDH
        addwf   R8,f            ; R8R9 = -BARO_P+EE_BARO1_END

        btfss   status,c
        goto    inbaro2         ; if carry then in barograph 1

inbaro1
        movlw   EE_BARO1_ENDH
        movwf   BARO_END_H
        movlw   EE_BARO1_ENDL
        movwf   BARO_END_L
        bcf     MMIFLAGS,3      ; set barograph store just to be sure!
        goto    finish_off_baro_init
inbaro2
        movlw   EE_BARO2_ENDH
        movwf   BARO_END_H
        movlw   EE_BARO2_ENDL
        movwf   BARO_END_L
        bsf     MMIFLAGS,3      ; set barograph store just to be sure!

finish_off_baro_init
        clrf    BARO_TICK       ; clear barograph timer tick
        clrf    BARO_LEN        ; and data left to write

        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****


; ----- reset timers & interrupt flags, enable interrupts

        bcf     rcsta,cren      ; clear uart rx before enable interrupts ...
        bsf     rcsta,cren      ;   reset oerr by disabling then enabling rx
        movf    rcreg,w  ;   reset frame error by reading rcref

        clrf    pir1            ; reset peripheral interrupt status bit
        clrf    pir2            ; reset CCP2 interrupt bit

        movlw   b'11000000'     ; disable all interrupts except peripheral
        movwf   intcon          ; and enable global interrupts - go go go!



; ==========================================================================
;                              MAIN SCHEDULER
; ==========================================================================
;
;       if update interrupt has occured:
;               acquire alti, vario, asi, temp, keys
;               set vario tone
;               update info on display
;               if barographing, put info in buzzer and start off write process
;       scan keypad
;       if key pressed, go to mmi
;       write any info to eeprom required
;       go to engineering handler if turned on
;


start

        clrwdt
        setpage draw_screen
        call    draw_screen     ; draw first screen (screen 0)

        bcf     status,rp0      ; ... to bank 0          <--- **** NOTE ****
        setpage scheduler

scheduler

        btfsc   porta,nADCREADY ; goes low when external adc has update
        goto    scheduler1      ; use this as tick for scheduling measure
                                ; activity.  returns high once adc read.

; Perform measurements / update according to adctick
        clrwdt
        setpage measure
        call    measure
        bcf     status,rp0
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****

; Update barograph if applicable, also according to adctick
        clrwdt
        setpage barograph
        call    barograph       ; check / update barograph if work to do
                                ; could return in any bank
; Now increment adctick ...
        bcf     status,rp0
        bsf     status,rp1      ; ... to bank 2   <--- **** NOTE ****
        incf    ADCTICK,f
        movf    ADCTICK,w
        sublw   d'25'
        btfss   status,z        ; zero flag set when tick equals 25
        goto    adctick_done    ; if not then not finished this frame yet
        clrf    ADCTICK         ; wrap when ADCTICK = 25
        incf    ADCFRAME,f
        movf    ADCFRAME,w
        sublw   d'10'
        btfsc   status,z
        clrf    ADCFRAME        ; when reach frame 10, wrap
adctick_done
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****


scheduler1

        clrwdt
        setpage scan_keypad
        call    scan_keypad
        bcf     status,rp0      ; ... to bank 0          <--- **** NOTE ****


scheduler2

        clrwdt
        setpage do_mmi
        call    do_mmi
        bcf     status,rp0      ; ... to bank 0          <--- **** NOTE ****


scheduler3

        clrwdt
        setpage update_display
        call    update_display


scheduler4

        bsf     status,rp0      ; ... to bank 1          <--- **** NOTE ****
        bcf     pie1,5          ; disable rx interrupts
        bcf     status,rp0      ; ... to bank 0          <--- **** NOTE ****
        clrwdt
        setpage service_uart
        call    service_uart
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bsf     pie1,5          ; re-enable rx interrupts

scheduler5

        bcf     status,rp0      ; ... to bank 0          <--- **** NOTE ****
        setpage scheduler
        goto    scheduler


; ==========================================================================
;                           VARIO UPDATE
; ==========================================================================

; Do negative / positive peak detect, depending whether rising or sinking
; Called from measure.asm as insufficient room in that bank.

update_vario
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    PEAK_VARIO,w
        btfsc   status,z             ; do we have a zero peak?
        goto    load_new_peak_vario  ; just loadup if yes
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        movf    VARIO,w         ; copy into wreg for processing below
        btfss   VARIO,7         ; negative vario?
        goto    pos_vario
neg_vario
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        btfss   PEAK_VARIO,7
        goto    check_vario_timeout  ; branch if peak currently positive
        subwf   PEAK_VARIO,w    ; wreg = (-ve)PEAK_VARIO - (-ve)VARIO
        btfss   status,c        ; carry set if peak
        goto    check_vario_timeout
        goto    load_new_peak_vario
pos_vario
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        btfsc   PEAK_VARIO,7
        goto    check_vario_timeout ; on negative peak so skip
        subwf   PEAK_VARIO,w    ; wreg = (-ve)PEAK_VARIO - VARIO
        btfsc   status,c        ; carry clear if peak
        goto    check_vario_timeout
load_new_peak_vario
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    VARIO,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   PEAK_VARIO
        movlw   PEAK_TIMEOUT
        movwf   PEAK_VARIO_TICK ; reset timeout tick
        goto    update_vario_average
check_vario_timeout
        movf    PEAK_VARIO_TICK,w
        btfsc   status,z
        goto    load_new_peak_vario     ; if peak timed out, start a new peak detect


; now update average, arrive here in bank 1 ...
; first, store new average and update pointer

update_vario_average
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    VARIO,w
        movwf   R1              ; add this full on proper reading to vario average

do_vario_average
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    VARIOPTR,w
        movwf   fsr
        movf    R1,w
        movwf   indf            ; store in buffer
        incf    fsr,w           ; increment pointer
        sublw   VARIOBFR+8      ; past end of buffer?
        btfsc   status,z            ; zero if yes
        movlw   8               ; move back buffer size
        sublw   VARIOBFR+8      ; restore check
        movwf   VARIOPTR        ; store updated pointer

; ... average all of buffer

        clrf    R6              ; extended sign of vario reading read
        clrf    R7
        clrf    R8              ; sum into R7R8
        movlw   8
        movwf   R5              ; countdown all buffer in R5
        movlw   VARIOBFR
        movwf   fsr
average_vario_loop
        bcf     status,c        ; clear carry ready
        movlw   0xff            ; extend indf sign word into R5
        movwf   R6              ;
        btfss   indf,7          ;
        clrf    R6              ;
        movf    indf,w          ; 16 bit add
        addwf   R8,f            ;
        btfsc   status,c        ;
        incf    R7,f            ;
        movf    R6,w            ;
        addwf   R7,f            ;
        incf    fsr,f           ; then move onto next
        decfsz  R5,f
        goto    average_vario_loop

; Now need to divide R7R8 by 8 so shift right by three; then have average

        rrf     R7,f
        rrf     R8,f
        rrf     R7,f
        rrf     R8,f
        rrf     R7,f
        rrf     R8,w
        movwf   AVERAGE_VARIO

; Now update PWM output for latest vario reading ...

; - Use Timer 2 with prescaler of 4
; - Period to load = 151 - vario (1.66kHz) limited to +/- 2000 fpm
; - Frequency duty cycle set to 50%
; - Audible duty cycle set to 150ms on, 100ms off if up
;                             250ms on,   0ms off if down

; First obtain period in R1, and update overall (audio) duty cycle
; coefficients for 10ms interrupt to use.  Clip vario to 20up/dn for
; sound.  Actual control is in 10ms interrupt.

update_vario_pwm


#ifdef SOUND_DEBUG
        movf    HEADWIND,w
        addlw   d'128'

        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R1              ; store in R1 for use below

        movlw   d'15'
        movwf   BEEP_ON_PERIOD
        movlw   d'10'
        movwf   BEEP_OFF_PERIOD ; units of 10ms

        btfsc   t2con,1         ; if bit 1 is set then 1:16 prescaler so need to alter.
        bcf     t2con,1         ; clear so now have 1:4 prescaler; prescale counter cleared.

        comf    R1,w
        addlw   d'44'+1
        goto    load_pwm
#else
; FIRST - clip beeps to something sane
; ~~~~~

        movf    AVERAGE_VARIO,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        addlw   d'128'          ; add 128 so 128 is 0, 129 is 0.2up etc.

        addlw   d'256'-d'228'   ; carry set if > 20 up
        btfsc   status,c
        movlw   0               ; clip to 20up (in a second)
        addlw   d'228'          ; restore check

        addlw   d'256'-d'28'    ; carry set if > 20 dn
        btfss   status,c
        movlw   0

        movwf   R1              ; store in R1 for use below

; R1 now contains clipped vario relative to 20 down; ie 100 = 0, 101 = 0.2up ...


; SECOND- setup duty cycle and pitch according to up/down
; ~~~~~~
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        btfsc   AVERAGE_VARIO,7  ; if msb set then going down
        goto    down_beeps

up_beeps
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   d'15'
        movwf   BEEP_ON_PERIOD
        movlw   d'10'
        movwf   BEEP_OFF_PERIOD ; units of 10ms

; Prescaler of 4
; Period = 200 - vario
; R1 = clipped vario = 100 for 0, 101 for 0.2 up ...
;
; therefore ...
;
; Period = 200 - (R1 - 100) = 300 - R1 = 44 - R1

        btfsc   t2con,1         ; if bit 1 is set then 1:16 prescaler so need to alter.
        bcf     t2con,1         ; clear so now have 1:4 prescaler; prescale counter cleared.

        comf    R1,w
        addlw   d'44'+1
        goto    load_pwm


down_beeps
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movlw   d'25'
        movwf   BEEP_ON_PERIOD
        movlw   d'0'
        movwf   BEEP_OFF_PERIOD ; units of 10ms

; Prescaler of 16
; Period = 195 - ((clipped vario / 5) * 5)
;
; ie. don't change tone for changes of 0.2up, only of 1 up

        btfss   t2con,1         ; if bit 1 is clear then 1:4 prescaler so need to alter.
        bsf     t2con,1         ; set so now have 1:16 prescaler; prescale counter cleared.

        clrf    R2
        movf    R1,w
        movlw   d'256'-d'5'
divide_by_five
        addwf   R1,f           ; R1 = R1 - 5
        btfsc   R1,7           ; msb set when all done
        goto    done_div_by_five
        incf    R2,f
        goto    divide_by_five
done_div_by_five

        movlw   0              ; initialise
        incf    R2,f
multiply_by_five
        addlw   5
        decfsz  R2,f
        goto    multiply_by_five
        addlw   d'256'-5      ; subtract 5 as will always be one over

        sublw   d'195'        ; wreg = 195 - wreg.  Now have pwm value.
#endif


; Load period with wreg

load_pwm
        movwf   R1              ; store for use below
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        subwf   pr2,w
        btfsc   status,z
        goto    pwm_done        ; already playing this frequency
        movf    R1,w
        movwf   pr2             ; load timer 2 with period
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        bcf     status,c        ; clear status so shift in zero ...
        rrf     R1,w            ; halve period to get duty cycle
        movwf   ccpr2l          ; (don't bother with bits in ccp2con)

pwm_done
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        retlw   0               ; return to measure or scheduler


; ==========================================================================
;                            INTERRUPT HANDLER
; ==========================================================================
;
; Have the following interrupts vectoring through here:
;
;       timer1          10ms
;       Tx UART         -               service
;       Rx UART         -               service
;
; Wreg, Status (banks, flags etc.) and PClath (page) saved prior to a call
; to this, as need to select this page before do jump.  All of these
; restored on exit.  These copies are kept in registers accessible from any
; bank.


interrupt

        bcf     status, rp0     ;
        bcf     status, rp1     ; select bank 0 for direct
        bcf     status, irp     ; and bank 0/1 for indirect
        movf    fsr,w           ;
        movwf   FSR_COPY        ; save pointer register

        btfsc   pir1,tmr1if     ; service timer interrupt
        call    timer_int       ; (returns in bank 1)
        bcf     status,rp0      ; ... to bank 0          <--- **** NOTE ****

        btfsc   pir1,rcif
        call    read_uart       ; read from uart
                                ; must return in bank 0
        btfsc   pir1,txif
        call    tx_uart         ; transmit to uart
                                ; must return in bank 0

        movf    FSR_COPY,w      ;
        movwf   fsr             ; restore pointer register
        movf    PCLATH_COPY,w   ;
        movwf   pclath          ; restore pclath
        movf    STATUS_COPY,w   ;
        movwf   status          ; restore status
        movf    WREG_COPY,w     ; restore wreg
        retfie                  ; return, re-enabling interrupts




; *******************
; * TIMER INTERRUPT *
; *******************
;
; NOTE - called in bank 0, returns in bank 1

timer_int
        bcf     t1con,0         ; stop timer
        movlw   TIMER_LO        ;
        addwf   tmr1l,f         ;
        btfsc   status,c        ;
        incf    tmr1h,f         ;
        movlw   TIMER_HI        ; reset by adding on time period (minus
        addwf   tmr1h,f         ; time it takes for this, ie 8 cycles)
        bcf     pir1,tmr1if     ; clear interrupt flag
        bsf     t1con,0         ; restart timer


; Control vario sounder

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****

        btfss   MMIFLAGS,4
        goto    beep_off        ; skip sound if master disabled

        movlw   b'00000011'
        andwf   MMIFLAGS,w
        btfsc   status,z
        goto    beep_off        ; skip sound if off

        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        incf    BEEP_DUTY_COUNT,f
        btfss   BEEP_DUTY_COUNT,7
        goto    in_off_period
in_on_period
        movlw   b'01111111'
        andwf   BEEP_DUTY_COUNT,w
        subwf   BEEP_ON_PERIOD,w  ; w = BEEP_DUTY_COUNT - BEEP_ON_PERIOD
        btfsc   status,z                  ; zero set if met
        goto    switch_to_off
        btfsc   status,c          ; carry clear if exceeded
        goto    try_beep_on       ; maintain vario beep on if possible
switch_to_off
        movf    BEEP_OFF_PERIOD,w
        btfsc   status,z          ; are we continually on?
        goto    switch_to_on      ; zero so yes, so reset back to on state
        clrf    BEEP_DUTY_COUNT
        goto    beep_off          ; switch vario beep off
in_off_period
        movf    BEEP_DUTY_COUNT,w
        subwf   BEEP_OFF_PERIOD,w ; w = BEEP_DUTY_COUNT - BEEP_OFF_PERIOD
        btfsc   status,z                  ; zero set if met
        goto    switch_to_on
        btfsc   status,c          ; carry clear if exceeded
        goto    beep_off          ; maintain vario beep off
switch_to_on
        movlw   b'10000000'
        movwf   BEEP_DUTY_COUNT
        goto    try_beep_on       ; switch vario beep on if possible


; Vario beep
; ~~~~~~~~~~

; Physical routines to maintain vario beep on and off now.  Vario on
; is more involved as must also meet the following conditions:
;
; if VARIO >= BEEP_UP_THRESH AND MMIFLAGS.0 is set
; if VARIO >= BEEP_DN_THRESH AND MMIFLAGS<1:0> is greater than 2

try_beep_on

#ifdef SOUND_DEBUG
beep_off
        btfss   t2con,2         ; only set t2con.2 bit if not already set, otherwise
        bsf     t2con,2         ; will clear prescaler counter and may continually be running
        movlw   b'00001100'
        iorwf   ccp2con,f       ; set to PWM mode
#else
        btfsc   VARIO,7         ; are we going down?
        goto    check_down      ; branch if yes

check_up
        movf    VARIO,w
        addlw   d'256'-BEEP_UP_THRESH  ; carry set if >= BEEP_UP_THRESH
        btfss   status,c
        goto    beep_off               ; not going up enough to beep

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        btfsc   MMIFLAGS,0
        goto    beep_on         ; beep according to
        goto    beep_off        ; pilot settings

check_down
        movf    VARIO,w
        addlw   BEEP_DN_THRESH-1       ; carry set if < BEEP_DN_THRESH down
        btfsc   status,c
        goto    beep_off               ; not going down enough to beep

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    MMIFLAGS,w
        andlw   b'00000011'
        addlw   d'256'-2        ; carry set if >= 2
        btfss   status,c
        goto    beep_off        ; beep according to pilot settings

beep_on
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        btfss   t2con,2         ; only set t2con.2 bit if not already set, otherwise
        bsf     t2con,2         ; will clear prescaler counter and may continually be running
        movlw   b'00001100'
        iorwf   ccp2con,f       ; set to PWM mode
        goto    done_vario_beep

beep_off
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    ccp2con,w
        clrf    ccp2con         ; explicitely clear all of ccp2con to ensure latch cleared
        andlw   b'11110011'
        movwf   ccp2con         ; reload period, but stay out of PWM mode
        bcf     t2con,2         ; stop timer 2
        bcf     portc,1         ; reset portc.1 bit (set high by other bit ops if pwm had control)
#endif

done_vario_beep


; Check if auto-timer should start or stop
; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        btfss   MMIFLAGS,2
        goto    done_autotimer_check
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        btfsc   VARIO,7         ; are we going down?
        goto    auto_down       ; branch if yes

auto_up
        movf    VARIO,w
        addlw   d'256'-AUTO_UP_THRESH  ; carry set if >= AUTO_UP_THRESH
        bsf     status,rp0             ; ... to bank 1   <--- **** NOTE ****
        btfsc   status,c
        goto    inc_consec_beeps
        goto    clear_consec_beeps

auto_down
        movf    VARIO,w
        addlw   AUTO_DN_THRESH-1      ; carry set if < AUTO_DN_THRESH
        bsf     status,rp0            ; ... to bank 1   <--- **** NOTE ****
        btfss   status,c
        goto    inc_consec_beeps
        goto    clear_consec_beeps

inc_consec_beeps
        incf    ACTIVE_TICK,f
        movf    ACTIVE_TICK,w
        sublw   CONSECUTIVE_BEEPS
        btfss   status,z
        goto    done_autotimer_check  ; not quite ready to kick autotimer

kick_autotimer
        clrf    ACTIVE_TICK     ; clear count of consecutive beeps
        clrf    AUTO_TICK       ; clear auto timer start/stop increment
        btfsc   FLAGS,2
        goto    done_autotimer_check
        bsf     FLAGS,3         ; explicit display update please
        clrf    TIMER_SEC
        clrf    TIMER_MIN
        clrf    TIMER_HOUR
        bsf     FLAGS,2         ; reset and start timer

; Modify barograph tick to invoke a barograph write as soon as possible

        movf    BARO_RATE,w
        addlw   d'256'-1
        movwf   BARO_TICK       ; set barotick ready to start update next
        goto    done_autotimer_check

clear_consec_beeps
        clrf    ACTIVE_TICK     ; reset count of consecutive vario beeps
        
done_autotimer_check



; Now get on with the rest of it ...
; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        incf    KEY_TICKS,f     ; increment key tick debounce counter

        incf    TEN_MS_TICK,f
        movf    TEN_MS_TICK,w
        sublw   d'25'
        btfss   status,z
        retlw   0               ; not reached 250ms yet
        clrf    TEN_MS_TICK

        bsf     FLAGS,0         ; mark 250ms has occured
        incf    QUARTER_TICK,f
        btfss   QUARTER_TICK,2  ; reached a whole second yet?
        retlw   0               ; return if no

        clrf    QUARTER_TICK    ; reset quarter tick and
        incf    SECOND_TICK,f   ; update second tick if yes

        movf    PEAK_VARIO_TICK,w
        btfss   status,z
        decf    PEAK_VARIO_TICK,f       ; countdown to zero

        btfss   FLAGS,2         ; is timer running?
        retlw   0               ; return if isn't


; ----- Increment airtime timer

        incf    TIMER_SEC,w
        sublw   d'60'           ; check for wrap
        btfsc   status,z        ; skip if no
        movlw   d'60'           ; reset
        sublw   d'60'           ; restore check
        movwf   TIMER_SEC       ; store

        btfss   status,z        ; did we just wrap?
        goto    inc_autotick    ; nope, so skip next bit

        incf    TIMER_MIN,w
        sublw   d'60'           ; check for wrap
        btfsc   status,z        ; skip if no
        movlw   d'60'           ; reset
        sublw   d'60'           ; restore check
        movwf   TIMER_MIN       ; store

        btfsc   status,z        ; did we just wrap?
        incf    TIMER_HOUR,f    ; yes, so increment hour


; ----- Increment auto timeout tick if operating

inc_autotick

        btfss   MMIFLAGS,2
        goto    inc_barotick

        movf    AUTO_TICK,w
        sublw   AUTOSTOP_TIMEOUT
        btfsc   status,z
        goto    inc_barotick            ; branch if already timed out

        incf    AUTO_TICK,f

        movf    AUTO_TICK,w
        sublw   AUTOSTOP_TIMEOUT
        btfss   status,z
        goto    inc_barotick            ; branch if not ready to time out

        setpage terminate_baro          ; terminate barograph
        call    terminate_baro
        bcf     FLAGS,2                 ; stop timer
        retlw   0                       ; return


; ----- Increment barograph tick, set baro update flag if now time

inc_barotick

        incf    BARO_TICK,f
        movf    BARO_TICK,w
        subwf   BARO_RATE,w     ; contains time between updates
        btfss   status,z
        retlw   0               ; return if not time to update yet

        bsf     FLAGS,1         ; tell barograph to update
        clrf    BARO_TICK       ; clear tick
        retlw   0


; *********************
; * UART RX INTERRUPT *
; *********************
;

read_uart

        btfss   rcsta,oerr
        goto    read_char       ; go and read char if no over-run error
        bcf     rcsta,cren      ;
        bsf     rcsta,cren      ; reset oerr by disabling then enabling rx
        goto    rx_err          ;

read_char
        btfsc   rcsta,ferr
        goto    rx_err          ; if ferr set then frame error
        movf    rcreg,w         ; read character - resets rcif in process
        movwf   INT_WORK        ; save it temporarily


; ----- pre-process character; ie. strip anything nasty out!
;
; Allowed characters are 10, 13, and 32 to 122
; If don't strip, can cause command look-up code to crash

        sublw   d'10'      ; is 10?
        btfsc   status,z
        goto    rx_char_valid
        sublw   d'10'      ; restore check

        sublw   d'13'      ; is 13?
        btfsc   status,z
        goto    rx_char_valid
        sublw   d'13'      ; restore check

        addlw   d'256'-d'32'    ; carry should set if >= 32
        btfss   status,c
        retlw   0              ; return if < 32
        addlw   d'32'      ; restore check

        addlw   d'255'-d'122'   ; carry clear if <= 122
        btfsc   status,c
        retlw   0              ; return if > 122

rx_char_valid


; ----- echo the character back to tx buffer, and increment buffer

        movf    TX_END,w
        movwf   fsr
        movf    INT_WORK,w      ;
        movwf   indf            ; echo character back through tx

        incf    TX_END,w        ; increment TX_END to wreg
        sublw   TX_BUFFER+d'15' ; see if past end of buffer
        btfss   status,c        ; carry clear if is
        movlw   d'15'           ; load wreg with 15 so at start after ...
        sublw   TX_BUFFER+d'15' ; restore [l - (l-w) = w]
        movwf   TX_END          ; circularly incremented

; ----- skip case change if LF

        movf    INT_WORK,w      ; restore character to accumulator
        sublw   0d              ; see if LF;  wreg = 0d - wreg
        btfsc   status,z
        goto    store_char      ; store it directly if is

; ----- skip case change if CR (and change CR to LF)

        addlw   0a-0d           ; add 0a and subtract 0d to check if was 0a
        btfss   status,z
        goto    convert_case    ; if not then convert case
        movlw   0d
        movwf   INT_WORK        ; else convert
        goto    store_char      ; and store

; ----- convert to lower case

convert_case
        movf    INT_WORK,w      ; restore acc
        sublw   61              ; wreg = 61 - wreg.  61 hex = 'a', so ...
        btfsc   status,c        ; skip if >= and so OK
        bsf     INT_WORK,5      ; set bit 5 to convert to lower case

; ----- store processed character and increment buffer

store_char
        movf    RX_END,w        ; old end of buffer
        movwf   fsr             ; also copy to pointer register
        movf    INT_WORK,w
        movwf   indf            ; store character in buffer

        incf    RX_END,w        ; increment RX_END to wreg
        sublw   RX_BUFFER+d'15' ; see if past end of buffer
        btfss   status,c        ; carry clear if is
        movlw   d'15'           ; load wreg with 15 so at start after ...
        sublw   RX_BUFFER+d'15' ; restore [l - (l-w) = w]
        movwf   RX_END          ; circularly incremented

; ----- start off tx uart (if not already going) for echo

        setpage initiate
        call    initiate        ; set TXEN, TXIF & TXIE - txuart will be
        setpage done_initiate   ; called on exit of this interrupt
done_initiate

; ----- return

        retlw   0

rx_err
        movf    rcreg,w         ; read (trash) char to reset ferr, rcif etc.
        retlw   0               ; return without doing anything




; *********************
; * UART TX INTERRUPT *
; *********************
;
; Called on transmit buffer emptying, or (indirectly) by new transmit being
; initiated.  To transmit a message the procedure shall be:
;       Copy into buffer
;       txen=1          ensure uart enabled (sets txif as well)
;       txie=1          ensure interrupts enabled
; This routine upon finding the buffer empty will disable the transmit
; interrupt.  It will then of course cease to reset UART_DOWN on each
; call of the routine.  When this reaches 0 in the timer interrupt
; TXEN will be set to zero.  Note that the UART doesn't like having
; this cleared then immediately set again!

tx_uart

        movf    TX_PTR,w
        subwf   TX_END,w        ; wreg = TX_END - wreg
        btfsc   status,z
        goto    stop_tx         ; if no chars left then cease tx interrupt

        movf    TX_PTR,w
        movwf   fsr             ; point to character
        movf    indf,w          ; get it

        movwf   txreg           ; send it
        incf    TX_PTR,f        ; increment pointer
        movf    TX_PTR,w        ; copy it
        sublw   TX_BUFFER+d'16' ; see if past end
        btfss   status,z
        retlw   0               ; if not then go ahead and return

        movlw   TX_BUFFER
        movwf   TX_PTR          ; otherwise set to start of buffer
        retlw   0

stop_tx
        bsf     status,rp0      ; ... to bank 1          <--- **** NOTE ****
        bcf     pie1,txie       ; disable interrupt
        bcf     status,rp0      ; ... to bank 0          <--- **** NOTE ****
        retlw   0               ; return


; ==========================================================================
;                       VARIO SOUNDER LOOKUP TABLE
; =========================================================================
;
; Jump table in own area so know in 256 byte boundary
;
; For up, simply reverse on and off tables.  That is 20 up is
; on for 2 and off for 3, the opposite of 20 down.

_beeptbl code

get_on_period
        addwf   pcl,f           ; add offset to go to jump vector
        retlw   3               ; 20 dn (never get this)
        retlw   4
        retlw   4
        retlw   4
        retlw   4
        retlw   4
        retlw   5
        retlw   5
        retlw   6
        retlw   6
        retlw   7
        retlw   7
        retlw   8
        retlw   10
        retlw   11
        retlw   13
        retlw   17
        retlw   22
        retlw   33
        retlw   67
        retlw   67              ; 0

get_off_period
        addwf   pcl,f           ; add offset to go to jump vector
        retlw   2               ; 20 down
        retlw   2
        retlw   2
        retlw   2
        retlw   2
        retlw   2
        retlw   2
        retlw   3
        retlw   3
        retlw   3
        retlw   3
        retlw   4
        retlw   4
        retlw   5
        retlw   6
        retlw   7
        retlw   8
        retlw   11
        retlw   17
        retlw   33
        retlw   33              ; 0 up

; ==========================================================================

        global  update_vario, do_vario_average

        end
