; *************************************************************************
;                         Free Flight Computer
;
;                             MATHS MODULE
;
; *************************************************************************
;
; Development Tool:     MPLAB v4.12.12
; File Creation:        09/04/00                RNH
; Last Update:          03/04/02                RNH
; Code Issue:           A                       RNH
;
        list    P=16F76                         ; PIC 16F76
        include "P16C76r.INC"                   ; r is my lower case version
        include "macro.inc"                     ; my macros

        extern  R0, R1, R2, R3, R4, R5, R6, R7, R8
        extern  R9, R10, R11, R12, R13, R14, R15, R16
        extern  R17, R18, R19, R20, R21, R22, R23, R24
        extern  R25, R26, R27, R28, R29, R30

; ==========================================================================
;
; These routines are derived directly from Microchip's own maths libraries.
; Error reporting has been simplified to "OK" or "Not OK" and registers are
; called more general purpose names.
;
; IMPORTANT NOTE:
;   MULTIPLY_FLOAT and DIVIDEFLOAT use slightly different mappings
;
; shared bank   R0      TEMPB0
;               R1      TEMPB1
;               R2              AARGB3
;               R3              AARGB4
;               R4              AARGB5
;               R5      EARGB3
;               R6      SIGN
;               R7      TEMP
;               R8              AEXP, EXP
;               R9              AARGB0
; bank 0        R10             AARGB1
;               R11                     BEXP
;               R12                     BARGB0
;               R13                     BARGB1
;               R14             AARGB2
;               R15                     BARGB2
;               R16                     BARGB3
; bank 1        R17                             CEXP
;               R18                             CARGB0
;               R19                             CARGB1
;               R20                             CARGB2
;               R21                                     DEXP
;               R22                                     DARGB0
;               R23                                     DARGB1
;               R24                                     DARGB2
;               R28     TEMPB2
;               R29     LOOPCOUNT
;               R30             AARGB7


; =========================================================================
;                          Polynomial Macros
; =========================================================================

POL24   macro   COF,N

;       32 bit evaluation of polynomial of degree N, PN(AARG), with coefficients COF,
;       and where AARG is assumed have been be saved in DARG when N>1.
;       The result is in AARG.
;
;       Rounding ignored for simplicity in my version - to hell with the error!

        local   i
        variable        i = N

        movlw   COF#v(i)
        movwf   R11
        movlw   COF#v(i)#v(0)
        movwf   R12
        movlw   COF#v(i)#v(1)
        movwf   R13
        movlw   COF#v(i)#v(2)
        movwf   R14
        setpage fpm32
        call    fpm32
        variable        i = i - 1

        movlw   COF#v(i)
        movwf   R11
        movlw   COF#v(i)#v(0)
        movwf   R12
        movlw   COF#v(i)#v(1)
        movwf   R13
        movlw   COF#v(i)#v(2)
        movwf   R14
        setpage fpa32
        call    fpa32

        variable        i = i - 1

        while   i >= 0

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R21,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R11
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R22,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R12
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R23,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R13
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R24,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R15
        setpage fpm32
        call    fpm32

        movlw   COF#v(i)
        movwf   R11
        movlw   COF#v(i)#v(0)
        movwf   R12
        movlw   COF#v(i)#v(1)
        movwf   R13
        movlw   COF#v(i)#v(2)
        movwf   R14
        setpage fpa32
        call    fpa32

        variable        i = i - 1

        endw

        endm


; --------------------------------------------------------------------------


POLL124 macro   COF,N

;       32 bit evaluation of polynomial of degree N, PN(AARG), with coefficients COF,
;       with leading coefficient of one, and where AARG is assumed have been be saved
;       in DARG when N>1.  The result is in AARG.

;       Rounding ignored for simplicity in my version - to hell with the error!

        local   i
        variable        i = N

        variable        i = i - 1


        movlw   COF#v(i)
        movwf   R11
        movlw   COF#v(i)#v(0)
        movwf   R12
        movlw   COF#v(i)#v(1)
        movwf   R13
        movlw   COF#v(i)#v(2)
        movwf   R14
        setpage fpa32
        call    fpa32

        variable        i = i - 1

        while   i >= 0

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R21,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R11
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R22,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R12
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R23,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R13
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R24,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R15
        setpage fpm32
        call    fpm32

        movlw   COF#v(i)
        movwf   R11
        movlw   COF#v(i)#v(0)
        movwf   R12
        movlw   COF#v(i)#v(1)
        movwf   R13
        movlw   COF#v(i)#v(2)
        movwf   R14
        setpage fpa32
        call    fpa32

        variable        i = i - 1

        endw

        endm


; --------------------------------------------------------------------------


UMUL2416L       macro
        movlw   0x08
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R29

loopum2416a
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        rrf     R13,f
        btfsc   status,c
        goto    alum2416nap
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  R29,f
        goto    loopum2416a
        movwf   R29
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

loopum2416b
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        rrf     R12,f
        btfsc   status,c
        goto    blum2416nap
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  R29,f
        goto    loopum2416b
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        clrf    R9
        clrf    R10
        clrf    R14
        retlw   0x00

blum2416nap
        bcf     status,c
        goto    blum2416na

alum2416nap
        bcf     status,c
        goto    alum2416na

aloopum2416
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        rrf     R13,f
        btfss   status,c
        goto    alum2416na
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R28,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        addwf   R14,f
        movf    R1,w
        btfsc   status,c
        incfsz  R1,w
        addwf   R10,f
        movf    R0,w
        btfsc   status,c
        incfsz  R0,w
        addwf   R9,f

alum2416na
        rrf     R9,f
        rrf     R10,f
        rrf     R14,f
        rrf     R2,f
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  R29, f
        goto    aloopum2416

        movlw   0x08
        movwf   R29

bloopum2416
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        rrf     R12,f
        btfss   status,c
        goto    blum2416na
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R28,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        addwf   R14,f
        movf    R1,w
        btfsc   status,c
        incfsz  R1,w
        addwf   R10,f
        movf    R0,w
        btfsc   status,c
        incfsz  R0,w
        addwf   R9,f

blum2416na
        rrf     R9,f
        rrf     R10,f
        rrf     R14,f
        rrf     R2,f
        rrf     R3,f
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  R29,f
        goto    bloopum2416
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        endm


; --------------------------------------------------------------------------


UMUL2424L       macro
        movlw   0x08
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R29

loopum2424a
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        rrf     R15,f
        btfsc   status,c
        goto    alum2424nap
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  R29,f
        goto    loopum2424a

        movwf   R29

loopum2424b
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        rrf     R13,f
        btfsc   status,c
        goto    blum2424nap
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  R29,f
        goto    loopum2424b

        movwf   R29

loopum2424c
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        rrf     R12,f
        btfsc   status,c
        goto    clum2424nap
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  R29,f
        goto    loopum2424c
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        clrf    R9
        clrf    R10
        clrf    R14
        retlw   0x00

clum2424nap
        bcf     status,c
        goto    clum2424na

blum2424nap
        bcf     status,c
        goto    blum2424na

alum2424nap
        bcf     status,c
        goto    alum2424na

aloopum2424
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        rrf     R15,f
        btfss   status,c
        goto    alum2424na
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R28,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        addwf   R14,f
        movf    R1,w
        btfsc   status,c
        incfsz  R1,w
        addwf   R10,f
        movf    R0,w
        btfsc   status,c
        incfsz  R0,w
        addwf   R9,f

alum2424na
        rrf     R9,f
        rrf     R10,f
        rrf     R14,f
        rrf     R2,f
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  R29,f
        goto    aloopum2424

        movlw   0x08
        movwf   R29

bloopum2424
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        rrf     R13,f
        btfss   status,c
        goto    blum2424na
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R28,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        addwf   R14,f
        movf    R1,w
        btfsc   status,c
        incfsz  R1,w
        addwf   R10,f
        movf    R0,w
        btfsc   status,c
        incfsz  R0,w
        addwf   R9,f

blum2424na
        rrf     R9,f
        rrf     R10,f
        rrf     R14,f
        rrf     R2,f
        rrf     R3,f
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  R29,f
        goto    bloopum2424

        movlw   0x08
        movwf   R29

cloopum2424
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        rrf     R12,f
        btfss   status,c
        goto    clum2424na
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R28,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        addwf   R14,f
        movf    R1,w
        btfsc   status,c
        incfsz  R1,w
        addwf   R10,f
        movf    R0,w
        btfsc   status,c
        incfsz  R0,w
        addwf   R9,f

clum2424na
        rrf     R9,f
        rrf     R10,f
        rrf     R14,f
        rrf     R2,f
        rrf     R3,f
        rrf     R4,f
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        decfsz  R29,f
        goto    cloopum2424
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        endm


; --------------------------------------------------------------------------


_math1  code


; =========================================================================
;                          CONVERT 2 FLOAT (SIGNED)
; =========================================================================
;
; Called and returned in bank 0.
;
; Converts the 16 bit 2s complement integer in R9R10
; into a 24bit floating point in R8R9R10
;
; Alters registers: R6, R7


conv2float_signed

        movlw   d'127'+d'15'    ; initialize exponent and add bias
        movwf   R8
        movf    R9,w
        movwf   R6
        setpage nrm2424
        btfss   R9,7            ; test sign
        goto    nrm2424

        comf    R10,f           ; if < 0, negate and set MSB in SIGN
        comf    R9,f
        incf    R10,f
        btfsc   status,z
        incf    R9,f
        setpage nrm2424
        goto    nrm2424



; =========================================================================
;                         CONVERT 2 FLOAT (UNSIGNED)
; =========================================================================
;
; Called and returned in bank 0.
;
; Converts the 16 unsigned integer in R9R10
; into a 24bit floating point in R8R9R10
;
; Alters registers: R6, R7


conv2float_unsigned

        movlw   d'127'+d'15'    ; initialize exponent and add bias
        movwf   R8
        clrf    R6              ; this is a positive number, no matter
        setpage nrm2424
        goto    nrm2424         ; what R9 bit 7 says!



; =========================================================================
;                            CONVERT 2 INT (SIGNED)
; =========================================================================
;
; Called and returned in bank 0.
;
; Converts the 24 bit floating point number in R8R9R10
; into a 16 bit 2s complement integer in R9R10
;
; Alters registers: R6


conv2int_signed
        setpage returnzero
        movf    R8,w
        btfsc   status,z
        goto    returnzero      ; test for zero argument as unique case of 000000 is in fact 0!

        movf    R9,w
        movwf   R6              ; copy for sign bit checking later
        bsf     R9,7            ; make msb explicit

        movlw   d'127'+d'15'    ; remove bias from exponent
        subwf   R8,f
        setpage math_error
        btfss   R8,7
        goto    math_error      ; branch if will be integer overflow
        setpage tshift2416_s

conv2int_signed_b
        comf    R8,f
        incf    R8,f

        movlw   8               ; do byte shift if exponent >= 8
        subwf   R8,w
        btfss   status,c
        goto    tshift2416_s
        movwf   R8
        rlf     R10,f           ; rotate next bit for rounding
        movf    R9,w
        movwf   R10
        clrf    R9

        movlw   8               ; do byte shift if exponent >= 8
        subwf   R8,w
        btfss   status,c
        goto    tshift2416_s
        movwf   R8
        rlf     R10,f           ; rotate next bit for rounding
        clrf    R10
        movf    R8,w
        btfss   status,z
        bcf     status,c
        goto    shift2416ok_s


tshift2416_s
        movf    R8,w            ; shift completed if exponent = 0
        btfsc   status,z
        goto    shift2416ok_s

shift2416_s
        bcf     status,c
        rrf     R9,f            ; right shift by EXP
        rrf     R10,f           ; right shift by EXP
        decfsz  R8,f
        goto    shift2416_s

shift2416ok_s
        btfss   R10,0
        goto    int2416ok_s
        btfss   status,c        ; round if next bit is set
        goto    int2416ok_s
        incf    R10,f
        btfsc   status,z
        incf    R9,f
        setpage math_error
        btfsc   R9,7            ; test for overflow
        goto    math_error

int2416ok_s
        setpage math_ok
        btfss   R6,7
        goto    math_ok
        comf    R10,f           ; if sign bit set, then negate
        comf    R9,f
        incf    R10,f
        btfsc   status,z
        incf    R9,f
        goto    math_ok



; =========================================================================
;                            CONVERT 2 INT (UNSIGNED)
; =========================================================================
;
; Called and returned in bank 0.
;
; Converts the 24 bit floating point number in R8R9R10
; into a 16 bit unsigned integer in R9R10
;
; Alters registers: no others altered


conv2int_unsigned
        setpage returnzero
        movf    R8,w
        btfsc   status,z
        goto    returnzero      ; test for zero argument as unique case of 000000 is in fact 0!

        setpage math_error
        btfsc   R9,7            ; if bit is set, then negative.
        goto    math_error      ; Can't do as we're unsigned!

        movf    R9,w
        movwf   R6              ; copy for sign bit checking later
        bsf     R9,7            ; make msb explicit

        movlw   d'127'+d'15'+1  ; remove bias and check from exponent
        subwf   R8,f
        setpage math_error
        btfss   R8,7
        goto    math_error      ; branch if will be integer overflow

        incf    R8,f            ; restore check
        setpage conv2int_signed_b
        goto    conv2int_signed_b



; =========================================================================
;                               MULTIPLY FLOAT
; =========================================================================
;
; Multiplies 24 bit floating point number in R8R9R10 with R11R12R13
; Returns result in R8R9R10
;
; Corrupts:     R7, R12 (bit 7), R14, R15, R16

multiplyfloat
        setpage returnzero
        movf    R8,w            ; test for zero arguments
        btfss   status,z
        movf    R11,w
        btfsc   status,z
        goto    returnzero
        setpage m24bne0

m24bne0
        movf    R9,w
        xorwf   R12,w
        movwf   R14             ; save sign in R14
        movf    R11,w
        addwf   R8,f
        movlw   d'127'-1
        btfss   status,c
        goto    mtun24
        subwf   R8,f
        setpage math_error
        btfsc   status,c
        goto    math_error      ; multiply overflow
        setpage mok24
        goto    mok24

mtun24
        subwf   R8,f
        setpage math_error
        btfss   status,c
        goto    math_error
        setpage mok24
mok24
        movf    R9,w
        movwf   R15             ; move result to AARG
        movf    R10,w
        movwf   R16
        bsf     R15,7           ; make argument MSB's explicity
        bsf     R12,7

        bcf     status,c
        clrf    R9              ; clear intial partial product
        clrf    R10
        movlw   d'16'
        movwf   R7              ; initialise counter
mloop24
        btfss   R16,0           ; test next bit
        goto    mnoadd24
madd24
        movf    R13,w
        addwf   R10,f
        movf    R12,w
        btfsc   status,c
        incfsz  R12,w
        addwf   R9,f
mnoadd24
        rrf     R9,f
        rrf     R10,f
        rrf     R15,f
        rrf     R16,f
        bcf     status,c
        decfsz  R7,f
        goto    mloop24
        btfsc   R9,7            ; check for postnormalisation
        goto    mround24
        rlf     R15,f
        rlf     R10,f
        rlf     R9,f
        decf    R8,f

mround24
        btfss   R10,0
        goto    mul24ok
        btfss   R15,7           ; round if next bit is set
        goto    mul24ok
        incf    R10,f
        btfsc   status,z
        incf    R9,f
        btfss   status,z        ; has rounding caused carryout?
        goto    mul24ok
        rrf     R9,f            ; if so, right shift
        rrf     R10,f
        incf    R8,f
        setpage math_error
        btfsc   status,z        ; check for overflow
        goto    math_error

mul24ok
        setpage math_ok
        btfss   R14,7
        bcf     R9,7            ; clear explicit msb if positive
        goto    math_ok



; =========================================================================
;                               DIVIDE FLOAT
; =========================================================================
;
; Divides 24 bit floating point number in R8R9R10 by R11R12R13
; Returns result in R8R9R10
;
; Corrupts:     R6, R7, R12, R14, R15, R16

dividefloat
        setpage math_error
        movf    R11,w           ; test for divide by zero
        btfsc   status,z
        goto    math_error

        setpage returnzero
        movf    R8,w
        btfsc   status,z
        goto    returnzero

        setpage d24bne0
d24bne0
        movf    R9,w
        xorwf   R12,w
        movwf   R14             ; save sign in R14
        bsf     R9,7            ; make argument MSB's explicit
        bsf     R12,7
talign24
        clrf    R7              ; clear align increment
        movf    R9,w
        movwf   R15             ; test for alignment
        movf    R10,w
        movwf   R16
        movf    R13,w
        subwf   R16,f
        movf    R12,w
        btfss   status,c
        incfsz  R12,w
        subwf   R15,f
        clrf    R15
        clrf    R16
        btfss   status,c
        goto    dalign24ok
        bcf     status,c        ; align if necessary
        rrf     R9,f
        rrf     R10,f
        rrf     R15,f
        movlw   0x01
        movwf   R7              ; save align increment

dalign24ok
        movf    R11,w           ; compare R8 and R11
        subwf   R8,f
        btfss   status,c
        goto    altb24

ageb24
        movlw   d'127'-1
        addwf   R7,w
        addwf   R8,f
        setpage math_error
        btfsc   status,c
        goto    math_error      ; overflow
        setpage dargok24
        goto    dargok24

altb24
        movlw   d'127'-1
        addwf   R7,w
        addwf   R8,f
        setpage math_error
        btfss   status,c
        goto    math_error      ; underflow
        setpage dargok24

dargok24
        movlw   d'16'           ; initialise counter
        movwf   R6

dloop24
        rlf     R16,f           ; left shift
        rlf     R15,f
        rlf     R10,f
        rlf     R9,f
        rlf     R7,f
        movf    R13,w           ; subtract
        subwf   R10,f
        movf    R12,w
        btfss   status,c
        incfsz  R12,w
        subwf   R9,f
        rlf     R12,w
        iorwf   R7,f
        btfss   R7,0            ; test for restore
        goto    drest24
        bsf     R16,0
        goto    dok24
drest24
        movf    R13,w           ; restore if necessary
        addwf   R10,f
        movf    R12,w
        btfsc   status,c
        incf    R12,w
        addwf   R9,f
        bcf     R16,0
dok24
        decfsz  R6,f
        goto    dloop24

dround24
        btfss   R16,0
        goto    div24ok
        bcf     status,c
        rlf     R10,f           ; compute next significant bit
        rlf     R9,f            ; for rounding
        rlf     R7,f
        movf    R13,w           ; subtract
        subwf   R10,f
        movf    R12,w
        btfss   status,c
        incfsz  R12,w
        subwf   R9,f
        rlf     R12,w
        iorwf   R7,w
        andlw   0x01
        addwf   R16,f
        btfsc   status,c
        incf    R15,f
        btfss   status,z        ; test if rounding caused carryout
        goto    div24ok
        rrf     R15,f
        rrf     R16,f
        incf    R8,f
        setpage math_error
        btfsc   status,z        ; test for overflow
        goto    math_error

div24ok
        setpage math_ok
        btfss   R14,7
        bcf     R15,7           ; clear explicit MSB if positive
        movf    R15,w
        movwf   R9              ; move result to AARG
        movf    R16,w
        movwf   R10
        goto    math_ok



; =========================================================================
;                               POWER FLOAT
; =========================================================================
;
; Returns 24bit float R8R9R10 ^ R11R12R13 in R8R9R10
;
; Alters:       R1 - R27
; (and takes an absolute age!)

power

; ----- Save R11R12R13 in R25R26R27 as gets corrupted in log routine

        movf    R11,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R25
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    R12,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R26
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    R13,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R27
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        setpage log24
        call    log24           ; take log2 of R8R9R10, result in R8R9R10
        setpage math_error
        btfss   status,c
        goto    math_error      ; abort if error


; ----- Restore R11R12R13 and perform multiply

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R25,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R11
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R26,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R12
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R27,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R13

        setpage multiplyfloat
        call    multiplyfloat   ; multiply by R11R12R13, result in R8R9R10
        setpage math_error
        btfss   status,c
        goto    math_error      ; abort if error


; ----- Finally, conclude the power calculation

        setpage exp24
        call    exp24           ; take exponent2 of R8R9R10, result in same
        setpage math_error
        btfss   status,c
        goto    math_error      ; abort if error
        setpage math_ok
        goto    math_ok         ; otherwise all done!



; =========================================================================
;                             32 bit NORMALISATION
; =========================================================================
;
;       Input:  32 bit unnormalized floating point number in R8, R9, R10,
;          R14, with R6 in R6,7
;       Output: 32 bit normalized floating point number in R8, R9, R10, R14
;       Result: AARG  <--  NORMALIZE( AARG )
;
;       Alters: R7-R10, R14

nrm3232
nrm32   clrf    R7              ; clear exponent decrement
        movf    R9,w            ; test if highbyte=0
        btfss   status,z
        goto    norm3232
        movf    R10,w           ; if so, shift 8 bits by move
        movwf   R9
        movf    R14,w
        movwf   R10
        clrf    R14
        bsf     R7,3            ; increase decrement by 8

        movf    R9,w            ; test if highbyte=0
        btfss   status,z
        goto    norm3232
        movf    R10,w           ; if so, shift 8 bits by move
        movwf   R9
        clrf    R10
        bcf     R7,3            ; increase decrement by 8
        bsf     R7,4

        setpage returnzero32
        movf    R9,w            ; if highbyte=0, result=0
        btfsc   status,z
        goto    returnzero32

norm3232
        movf    R7,w
        subwf   R8,f
        setpage math_error
        btfss   status,z
        btfss   status,c
        goto    math_error

        bcf     status,c        ; clear carry bit

norm3232a
        setpage fixr632
        btfsc   R9,7            ; if 7=1, normalization done
        goto    fixr632
        setpage norm3232a
        rlf     R14,f           ; otherwise, shift left and
        rlf     R10,f           ; decrement EXP
        rlf     R9,f
        decfsz  R8,f
        goto    norm3232a

        setpage math_error
        goto    math_error      ; underflow if EXP=0

fixr632
        btfss   R6,7
        bcf     R9,7            ; clear explicit msb if positive
        setpage math_ok
        goto    math_ok


; =========================================================================
;                         40 to 32 bit NORMALISATION
; =========================================================================
;
;       Input:  40 bit unnormalized floating point number in R8, R9, R10,
;         R14, R2 with R6 in R6,7
;       Output: 32 bit normalized floating point number in R8, R9, R10, R14,
;         R2
;       Result: AARG  <--  NORMALIZE( AARG )
;
;       Alters: R2, R7-R10, R14

nrm4032
        clrf    R7              ; clear exponent decrement
        movf    R9,w            ; test if highbyte=0
        btfss   status,z
        goto    norm4032
        movf    R10,w           ; if so, shift 8 bits by move
        movwf   R9
        movf    R14,w
        movwf   R10
        movf    R2,w
        movwf   R14
        clrf    R2
        bsf     R7,3            ; increase decrement by 8

        movf    R9,w            ; test if highbyte=0
        btfss   status,z
        goto    norm4032
        movf    R10,w           ; if so, shift 8 bits by move
        movwf   R9
        movf    R14,w
        movwf   R10
        clrf    R14
        bcf     R7,3            ; increase decrement by 8
        bsf     R7,4

        movf    R9,w            ; test if highbyte=0
        btfss   status,z
        goto    norm4032
        movf    R10,w           ; if so, shift 8 bits by move
        movwf   R9
        clrf    R10
        bsf     R7,3            ; increase decrement by 8

        setpage returnzero32
        movf    R9,w            ; if highbyte=0, result=0
        btfsc   status,z
        goto    returnzero32

norm4032
        setpage math_error
        movf    R7,w
        subwf   R8,f
        btfss   status,z
        btfss   status,c
        goto    math_error
        setpage norm4032a

        bcf     status,c        ; clear carry bit

norm4032a
        btfsc   R9,7            ; if msb=1, normalization done
        goto    nrmrnd4032
        rlf     R2,f            ; otherwise, shift left and
        rlf     R14,f           ; decrement EXP
        rlf     R10,f
        rlf     R9,f
        decfsz  R8,f
        goto    norm4032a

        setpage math_error
        goto    math_error      ; underflow if EXP=0

nrmrnd4032
        setpage fixr632
        btfss   R14,0
        goto    fixr632
        btfss   R2,7            ; round if next bit is set
        goto    fixr632
        incf    R14,f
        btfsc   status,z
        incf    R10,f
        btfsc   status,z
        incf    R9,f

        btfss   status,z        ; has rounding caused carryout?
        goto    fixr632
        rrf     R9,f            ; if so, right shift
        rrf     R10,f
        rrf     R14,f
        incf    R8,f
        setpage math_error
        btfsc   status,z        ; check for overflow
        goto    math_error
        setpage fixr632
        goto    fixr632


; =========================================================================
;                            32 bit MULTIPLY
; =========================================================================
;
;       Input:  32 bit floating point number in R8, R9, R10, R14
;         32 bit floating point number in R11, R12, R13, R15
;       Output: 32 bit floating point product in R8, R9, R10, R14
;       Result: AARG  <--  AARG * BARG
;
;       Alters: R2-R10, R12, R14

fpm32
        setpage returnzero32
        movf    R8,w            ; test for zero arguments
        btfss   status,z
        movf    R11,w
        btfsc   status,z
        goto    returnzero32
        setpage m32bne0

m32bne0
        movf    R9,w
        xorwf   R12,w
        movwf   R6              ; save in R6

        movf    R11,w
        addwf   R8,f
        movlw   d'127'-1
        btfss   status,c
        goto    mtun32

        subwf   R8,f
        setpage math_error
        btfsc   status,c
        goto    math_error      ; set multiply overflow flag
        setpage mok32
        goto    mok32

mtun32
        subwf   R8,f
        setpage math_error
        btfss   status,c
        goto    math_error
        setpage mok32

mok32
        movf    R9,w
        movwf   R2
        movf    R10,w
        movwf   R3
        movf    R14,w
        movwf   R4
        bsf     R2,7            ; make argument msb's explicit
        bsf     R12,7
        bcf     status,c
        clrf    R9              ; clear initial partial product
        clrf    R10
        clrf    R14
        movlw   d'24'
        movwf   R7              ; initialize counter

mloop32
        btfss   R4,0            ; test next bit
        goto    mnoadd32

madd32
        movf    R15,w
        addwf   R14,f
        movf    R13,w
        btfsc   status,c
        incfsz  R13,w
        addwf   R10,f

        movf    R12,w
        btfsc   status,c
        incfsz  R12,w
        addwf   R9,f

mnoadd32
        rrf     R9,f
        rrf     R10,f
        rrf     R14,f
        rrf     R2,f
        rrf     R3,f
        rrf     R4,f
        bcf     status,c
        decfsz  R7,f
        goto    mloop32

        btfsc   R9,7            ; check for postnormalization
        goto    mround32
        rlf     R2,f
        rlf     R14,f
        rlf     R10,f
        rlf     R9,f
        decf    R8,f

mround32
        btfss   R14,0
        goto    mul32ok
        btfss   R2,7
        goto    mul32ok
        incf    R14,f
        btfsc   status,z
        incf    R10,f
        btfsc   status,z
        incf    R9,f

        btfss   status,z        ; has rounding caused carryout?
        goto    mul32ok
        rrf     R9,f            ; if so, right shift
        rrf     R10,f
        rrf     R14,f
        incf    R8,f
        setpage math_error
        btfsc   status,z        ; check for overflow
        goto    math_error

mul32ok
        setpage math_ok
        btfss   R6,7
        bcf     R9,7            ; clear explicit msb if positive
        goto    math_ok


; =========================================================================
;                               32 bit DIVIDE
; =========================================================================
;
;       Input:  32 bit floating point dividend in R8, R9, R10, R14
;         32 bit floating point divisor in R11, R12, R13, R15
;       Output: 32 bit floating point quotient in R8, R9, R10, R14
;       Result: AARG  <--  AARG / BARG
;
;       Alters: R1-R10, R12, R14

fpd32
        setpage math_error
        movf    R11,w           ; test for divide by zero
        btfsc   status,z
        goto    math_error

        setpage returnzero32
        movf    R8,w
        btfsc   status,z
        goto    returnzero32

        setpage d32bne0
d32bne0
        movf    R9,w
        xorwf   R12,w
        movwf   R6              ; save R6 in R6
        bsf     R9,7            ; make argument msb's explicit
        bsf     R12,7

talign32
        clrf    R7              ; clear align increment
        movf    R9,w
        movwf   R2              ; test for alignment
        movf    R10,w
        movwf   R3
        movf    R14,w
        movwf   R4

        movf    R15,w
        subwf   R4,f
        movf    R13,w
        btfss   status,c
        incfsz  R13,w

ts1align32
        subwf   R3,f
        movf    R12,w
        btfss   status,c
        incfsz  R12,w

ts2align32
        subwf   R2,f
        clrf    R2
        clrf    R3
        clrf    R4

        btfss   status,c
        goto    dalign32ok

        bcf     status,c        ; align if necessary
        rrf     R9,f
        rrf     R10,f
        rrf     R14,f
        rrf     R2,f
        movlw   0x01
        movwf   R7              ; save align increment

dalign32ok
        movf    R11,w           ; compare R8 and R11
        subwf   R8,f
        btfss   status,c
        goto    altb32

ageb32
        movlw   d'127'-1
        addwf   R7,w
        addwf   R8,f
        setpage math_error
        btfsc   status,c
        goto    math_error
        setpage dargok32
        goto    dargok32        ; set overflow flag

altb32
        movlw   d'127'-1
        addwf   R7,w
        addwf   R8,f
        setpage math_error
        btfss   status,c
        goto    math_error      ; set underflow flag
        setpage dargok32

dargok32
        movlw   d'24'           ; initialize counter
        movwf   R1

dloop32
        rlf     R4,f            ; left shift
        rlf     R3,f
        rlf     R2,f
        rlf     R14,f
        rlf     R10,f
        rlf     R9,f
        rlf     R7,f

        movf    R15,w           ; subtract
        subwf   R14,f
        movf    R13,w
        btfss   status,c
        incfsz  R13,w
ds132
        subwf   R10,f

        movf    R12,w
        btfss   status,c
        incfsz  R12,w
ds232
        subwf   R9,f

        rlf     R12,w
        iorwf   R7,f

        btfss   R7,0            ; test for restore
        goto    drest32

        bsf     R4,0
        goto    dok32

drest32
        movf    R15,w           ; restore if necessary
        addwf   R14,f
        movf    R13,w
        btfsc   status,c
        incfsz  R13,w
darest32
        addwf   R10,f

        movf    R12,w
        btfsc   status,c
        incf    R12,w
        addwf   R9,f

        bcf     R4,0

dok32
        decfsz  R1,f
        goto    dloop32

dround32
        btfss   R4,0
        goto    div32ok
        bcf     status,c
        rlf     R14,f           ; compute next significant bit
        rlf     R10,f           ; for rounding
        rlf     R9,f
        rlf     R7,f

        movf    R15,w           ; subtract
        subwf   R14,f
        movf    R13,w
        btfss   status,c
        incfsz  R13,w
        subwf   R10,f

        movf    R12,w
        btfss   status,c
        incfsz  R12,w
        subwf   R9,f

        rlf     R12,w
        iorwf   R7,w
        andlw   0x01

        addwf   R4,f
        btfsc   status,c
        incf    R3,f
        btfsc   status,z
        incf    R2,f

        btfss   status,z        ; test if rounding caused carryout
        goto    div32ok
        rrf     R2,f
        rrf     R3,f
        rrf     R4,f
        incf    R8,f
        setpage math_error
        btfsc   status,z        ; test for overflow
        goto    math_error

div32ok
        setpage math_ok
        btfss   R6,7
        bcf     R2,7            ; clear explicit 7 if positive

        movf    R2,w
        movwf   R9              ; move result to AARG
        movf    R3,w
        movwf   R10
        movf    R4,w
        movwf   R14

        goto    math_ok


; =========================================================================
;                              32 bit SUBTRACT
; =========================================================================
;
;       Input:  32 bit floating point number in R8, R9, R10, R14
;         32 bit floating point number in R11, R12, R13, R15
;       Output: 32 bit floating point sum in R8, R9, R10, R14
;       Result: AARG  <--  AARG - BARG
;
;       Alters: R2, R4-R16

fps32
        movlw   0x80
        xorwf   R12,f


; =========================================================================
;                               32 bit ADD
; =========================================================================
;
;       Input:  32 bit floating point number in R8, R9, R10, R14
;         32 bit floating point number in R11, R12, R13, R15
;       Output: 32 bit floating point sum in R8, R9, R10, R14
;       Result: AARG  <--  AARG - BARG
;
;       Alters: R2, R4-R16

fpa32
        movf    R9,w            ; exclusive or of R6s in R7
        xorwf   R12,w
        movwf   R7

        clrf    R2              ; clear extended byte
        clrf    R16

        movf    R8,w            ; use AARG if R8 >= R11
        subwf   R11,w
        btfss   status,c
        goto    usea32

        movf    R11,w           ; use BARG if R8 < R11
        movwf   R4              ; therefore, swap AARG and BARG
        movf    R8,w
        movwf   R11
        movf    R4,w
        movwf   R8

        movf    R12,w
        movwf   R4
        movf    R9,w
        movwf   R12
        movf    R4,w
        movwf   R9

        movf    R13,w
        movwf   R4
        movf    R10,w
        movwf   R13
        movf    R4,w
        movwf   R10

        movf    R15,w
        movwf   R4
        movf    R14,w
        movwf   R15
        movf    R4,w
        movwf   R14

usea32
        setpage math_ok
        movf    R11,w           ; return AARG if BARG = 0
        btfsc   status,z
        goto    math_ok
        setpage aligned32

        movf    R9,w
        movwf   R6              ; save R6 in R6
        bsf     R9,7            ; make msb's explicit
        bsf     R12,7

        movf    R11,w           ; compute shift count in R11
        subwf   R8,w
        movwf   R11
        btfsc   status,z
        goto    aligned32

        movlw   8
        subwf   R11,w
        btfss   status,c        ; if R11 >= 8, do byte shift
        goto    alignb32
        movwf   R11
        movf    R15,w           ; keep for postnormalization
        movwf   R16
        movf    R13,w
        movwf   R15
        movf    R12,w
        movwf   R13
        clrf    R12

        movlw   8
        subwf   R11,w
        btfss   status,c        ; if R11 >= 8, do byte shift
        goto    alignb32
        movwf   R11
        movf    R15,w           ; keep for postnormalization
        movwf   R16
        movf    R13,w
        movwf   R15
        clrf    R13

        movlw   8
        subwf   R11,w
        btfss   status,c        ; if R11 >= 8, BARG = 0 relative to AARG
        goto    alignb32
        movf    R6,w
        movwf   R9
        setpage math_ok
        goto    math_ok

alignb32
        movf    R11,w           ; already aligned if R11 = 0
        btfsc   status,z
        goto    aligned32

aloopb32
        bcf     status,c        ; right shift by R11
        rrf     R12,f
        rrf     R13,f
        rrf     R15,f
        rrf     R16,f
        decfsz  R11,f
        goto    aloopb32

aligned32
        btfss   R7,7            ; negate if signs opposite
        goto    aok32

        comf    R16,f
        comf    R15,f
        comf    R13,f
        comf    R12,f
        incf    R16,f
        btfsc   status,z
        incf    R15,f
        btfsc   status,z
        incf    R13,f
        btfsc   status,z
        incf    R12,f

aok32
        movf    R16,w
        addwf   R2,f
        movf    R15,w
        btfsc   status,c
        incfsz  R15,w
        addwf   R14,f
        movf    R13,w
        btfsc   status,c
        incfsz  R13,w
        addwf   R10,f
        movf    R12,w
        btfsc   status,c
        incfsz  R12,w
        addwf   R9,f

        btfsc   R7,7
        goto    acomp32
        setpage nrmrnd4032
        btfss   status,c
        goto    nrmrnd4032

        rrf     R9,f            ; shift right and increment EXP
        rrf     R10,f
        rrf     R14,f
        rrf     R2,f
        incfsz  R8,f
        goto    nrmrnd4032
        setpage math_error
        goto    math_error

acomp32
        setpage nrm4032
        btfsc   status,c
        goto    nrm4032         ; normalize and fix R6
        setpage nrm32

        comf    R2,f
        comf    R14,f           ; negate, toggle R6 bit and
        comf    R10,f           ; then normalize
        comf    R9,f
        incf    R2,f
        btfsc   status,z
        incf    R14,f
        btfsc   status,z
        incf    R10,f
        btfsc   status,z
        incf    R9,f

        movlw   0x80
        xorwf   R6,f
        goto    nrm32


; =========================================================================
;                               24 bit LOG
; =========================================================================
;
;       Input:  24 bit floating point number in R8, R9, R10
;       Output: 24 bit floating point number in R8, R9, R10
;       Result: AARG  <--  LOG( AARG )
;
;       This approximation of the natural log function is based upon the
;       expansion
;
;       log(x) = log(2) * log2(x) = log(2) * ( n + log2(f) )
;
;       where .5 <= f < 1 and n is an integer.  The additional transformation
;
;           |   2*f-1, f < 1/sqrt(2), n=n-1
;       z = |
;           |   f-1, otherwise
;
;       produces a naturally segmented representation of log2(1+z) on the
;       intervals [1/sqrt(2)-1,0] and [0,sqrt(2)-1], utilizing minimax rational
;       approximations.
;
;       Alters most of R0 - R30!

log24
        setpage math_error
        clrf    R14
        btfsc   R9,7            ; test for negative argument
        goto    math_error
        movf    R8,w            ; test for zero argument
        btfsc   status,z
        goto    math_error

        movf    R8,w
        movwf   R5
        movlw   d'127'-1
        subwf   R5,f
        movwf   R8

        movlw   0xF3            ; .70710678118655 = 7E3504F3
        subwf   R14,w
        movlw   0x04
        movwf   R7
        btfss   status,c
        incfsz  R7,w
        subwf   R10,w
        movlw   0x35
        movwf   R7
        btfss   status,c
        incfsz  R7,w
        subwf   R9,w

        setpage log24l
        btfss   status,c
        goto    log24l
        setpage log24h


;       minimax rational approximation on [0,.sqrt(2)-1]

log24h
        movlw   0x7F
        movwf   R11
        clrf    R12
        clrf    R13
        clrf    R15

        setpage fps32
        call    fps32

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R8,w
        movwf   R21
        movf    R9,w
        movwf   R22
        movf    R10,w
        movwf   R23
        movf    R14,w
        movwf   R24
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        POLL124 LOG24HQ,2

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R8,w
        movwf   R17
        movf    R9,w
        movwf   R18
        movf    R10,w
        movwf   R19
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    R14,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R20

        movf    R21,w
        movwf   R8
        movf    R22,w
        movwf   R9
        movf    R23,w
        movwf   R10
        movf    R24,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R14

        POL24   LOG24HP,1

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R17,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R11
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R18,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R12
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R19,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R13
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R20,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R15

        setpage fpd32
        call    fpd32

        setpage log24ok
        goto    log24ok

;       minimax rational approximation on [1/sqrt(2)-1,0]

log24l
        incf    R8,f
        movlw   0x7F
        movwf   R11
        clrf    R12
        clrf    R13
        clrf    R15

        setpage fps32
        call    fps32

        decf    R5,f

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R8,w
        movwf   R21
        movf    R9,w
        movwf   R22
        movf    R10,w
        movwf   R23
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    R14,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R24
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        POLL124 LOG24LQ,2

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R8,w
        movwf   R17
        movf    R9,w
        movwf   R18
        movf    R10,w
        movwf   R19
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    R14,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R20

        movf    R21,w
        movwf   R8
        movf    R22,w
        movwf   R9
        movf    R23,w
        movwf   R10
        movf    R24,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R14

        POL24   LOG24LP,1

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R17,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R11
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R18,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R12
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R19,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R13
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R20,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R15

        setpage fpd32
        call    fpd32

log24ok
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R21,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R11
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R22,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R12
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R23,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R13
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R24,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R15

        setpage fpm32
        call    fpm32

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R8,w
        movwf   R21
        movf    R9,w
        movwf   R22
        movf    R10,w
        movwf   R23
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    R14,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R24
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        clrf    R9
        movf    R5,w
        movwf   R10
        btfsc   R10,7
        comf    R9,f
        setpage conv2float_signed
        call    conv2float_signed
        clrf    R14

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R21,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R11
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R22,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R12
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R23,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R13
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R24,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R15

        setpage fpa32
        call    fpa32

;       fixed point multiplication by log(2)

        setpage returnzero
        movf    R8,w
        btfsc   status,z
        goto    returnzero

        movf    R9,w
        movwf   R5
        bsf     R9,7

        movlw   0xB1
        movwf   R12
        movlw   0x72
        movwf   R13
        movlw   0x18
        movwf   R15

        setpage fxm2424u
        call    fxm2424u

        setpage log24done
        btfsc   R9,7
        goto    log24done
        rlf     R2,f
        rlf     R14,f
        rlf     R10,f
        rlf     R9,f
        decf    R8,f

log24done
        btfss   R5,7
        bcf     R9,7
        setpage math_ok
        goto    math_ok                 ; Don't do any rounding - keep things simple!


;       minimax rational coefficients for log2(1+x)/x on [1/sqrt(2)-1,0]

LOG24HP0        equ     0x81    ; LOG24HP0 = .73551298732E+1
LOG24HP00       equ     0x6B
LOG24HP01       equ     0x5D
LOG24HP02       equ     0x39

LOG24HP1        equ     0x81    ; LOG24HP1 = .40900513905E+1
LOG24HP10       equ     0x02
LOG24HP11       equ     0xE1
LOG24HP12       equ     0xB3

LOG24HQ0        equ     0x81    ; LOG24HQ0 = .50982159260E+1
LOG24HQ00       equ     0x23
LOG24HQ01       equ     0x24
LOG24HQ02       equ     0x96

LOG24HQ1        equ     0x81    ; LOG24HQ1 = .53849258895E+1
LOG24HQ10       equ     0x2C
LOG24HQ11       equ     0x51
LOG24HQ12       equ     0x50

LOG24HQ2        equ     0x7F    ; LOG24HQ2 = 1.0
LOG24HQ20       equ     0x00
LOG24HQ21       equ     0x00
LOG24HQ22       equ     0x00

;       minimax rational coefficients for log2(1+x)/x on [0,sqrt(2)-1]

LOG24LP0        equ     0x82    ; LOG24LP0 = .103115556038E+2
LOG24LP00       equ     0x24
LOG24LP01       equ     0xFC
LOG24LP02       equ     0x22

LOG24LP1        equ     0x81    ; LOG24LP1 = .457749066375E+1
LOG24LP10       equ     0x12
LOG24LP11       equ     0x7A
LOG24LP12       equ     0xCE

LOG24LQ0        equ     0x81    ; LOG24LQ0 = .714746549793E+1
LOG24LQ00       equ     0x64
LOG24LQ01       equ     0xB8
LOG24LQ02       equ     0x0A

LOG24LQ1        equ     0x81    ; LOG24LQ1 = .674551124538E+1
LOG24LQ10       equ     0x57
LOG24LQ11       equ     0xDB
LOG24LQ12       equ     0x3A

LOG24LQ2        equ     0x7F    ; LOG24LQ2 = 1.0
LOG24LQ20       equ     0x00
LOG24LQ21       equ     0x00
LOG24LQ22       equ     0x00

;       Maximum argument to EXP24
MAXLOG24EXP     equ     0x85            ; 88.7228391117 = log(2**128)
MAXLOG24B0      equ     0x31
MAXLOG24B1      equ     0x72

;       Minimum argument to EXP24
MINLOG24EXP     equ     0x85            ; -87.3365447506 = log(2**-126)
MINLOG24B0      equ     0xAE
MINLOG24B1      equ     0xAC


; =========================================================================
;                               24 bit EXP
; =========================================================================
;
;       Input:  24 bit floating point number in R8, R9, R10
;       Output: 24 bit floating point number in R8, R9, R10
;       Result: AARG  <--  EXP( AARG )
;
;       This approximation of the exponential function is based upon the
;       expansion
;
;       exp(x) = e**x = 2**(x/log(2)) = 2**z * 2**n,
;
;               x/log(2) = z + n,
;
;       where 0 <= z < 1 and n is an integer, evaluated during range reduction.
;       Segmented third degree minimax polynomial approximations are used to
;       estimate 2**z on the intervals [0,.25], [.25,.5], [.5,.75] and [.75,1].
;
;       Alters lots

exp24
        setpage exp24one
        movlw   0x66            ; test for |x| < 2**(-24)/2
        subwf   R8,w
        movwf   R0
        btfsc   R0,7
        goto    exp24one        ; return e**x = 1

        setpage tnexp24
        btfsc   R9,7            ; determine sign
        goto    tnexp24
tpexp24
        movf    R8,w            ; positive domain check
        sublw   MAXLOG24EXP
        setpage math_error
        btfss   status,c
        goto    math_error
        setpage exp24argok
        btfss   status,z
        goto    exp24argok

        movf    R9,w
        sublw   MAXLOG24B0
        setpage math_error
        btfss   status,c
        goto    math_error
        setpage exp24argok
        btfss   status,z
        goto    exp24argok

        movf    R10,w
        sublw   MAXLOG24B1
        setpage math_error
        btfss   status,c
        goto    math_error
        setpage exp24argok
        goto    exp24argok

tnexp24
        movf    R8,w            ; negative domain check
        sublw   MINLOG24EXP
        setpage math_error
        btfss   status,c
        goto    math_error
        setpage exp24argok
        btfss   status,z
        goto    exp24argok

        movf    R9,w
        sublw   MINLOG24B0
        setpage math_error
        btfss   status,c
        goto    math_error
        setpage exp24argok
        btfss   status,z
        goto    exp24argok

        movf    R10,w
        sublw   MINLOG24B1
        setpage math_error
        btfss   status,c
        goto    math_error
        setpage exp24argok

exp24argok
        setpage rrexp24
        call    rrexp24         ; range reduction

        setpage exp24l
        movlw   0x7E
        subwf   R8,w
        btfss   status,z
        goto    exp24l

exp24h
        setpage exp24hl
        btfss   R9,6
        goto    exp24hl
        POL24   EXP24HH,3       ; minimax approximation on [.75,1]
        setpage exp24ok
        goto    exp24ok

exp24hl
        POL24   EXP24HL,3       ; minimax approximation on [.5,.75]
        setpage exp24ok
        goto    exp24ok

exp24l
        setpage exp24ll
        movlw   0x7D
        subwf   R8,w
        btfss   status,z
        goto    exp24ll

        POL24   EXP24LH,3       ; minimax approximation on [.25,.5]

        setpage exp24ok
        goto    exp24ok

exp24ll
        POL24   EXP24LL,3       ; minimax approximation on [0,.25]

exp24ok
        setpage math_ok
        movf    R5,w
        addwf   R8,f
        goto    math_ok

exp24one
        setpage math_ok
        movlw   d'127'          ; return e**x = 1.0
        movwf   R8
        clrf    R9
        clrf    R10
        clrf    R14
        goto    math_ok



;       Range reduction routine for the exponential function
;
;       x/log(2) = z + n

rrexp24
        movf    R9,w            ; save R6
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R22
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        bsf     R9,7            ; make MSB explicit

        movf    R9,w
        movwf   R12
        movf    R10,w
        movwf   R13

        movlw   0xB8            ; 1/ln(2) = 1.44269504089
        movwf   R9
        movlw   0xAA
        movwf   R10
        movlw   0x3B
        movwf   R14

        setpage fxm2416u
        call    fxm2416u        ; x * (1/ln2)   no rounding!

        incf    R8,f

        setpage rrexp24yok
        btfsc   R9,7
        goto    rrexp24yok
        rlf     R2,f
        rlf     R14,f
        rlf     R10,f
        rlf     R9,f
        decf    R8,f

rrexp24yok
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        btfss   R22,7
        bcf     R9,7
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        setpage rnd4032
        call    rnd4032

        movf    R8,w
        movwf   R11             ; save z in BARG
        movf    R9,w
        movwf   R12
        movf    R10,w
        movwf   R13
        movf    R14,w
        movwf   R15

        setpage floor24
        call    floor24
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R8,w
        movwf   R21             ; save float(n) in DARG
        movf    R9,w
        movwf   R22
        movf    R10,w
        movwf   R23
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        setpage conv2int_signed
        call    conv2int_signed ; n = [ x * (1/ln2) ]

        movf    R10,w
        movwf   R5              ; save n in EARG

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R21,w
        movwf   R8
        movf    R22,w
        movwf   R9
        movf    R23,w
        movwf   R10
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        clrf    R14

        movlw   0x80            ; toggle R6
        xorwf   R9,f

        setpage fpa32
        call    fpa32

        setpage rnd4032
        call    rnd4032

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R8,w
        movwf   R21             ; save z in DARG
        movf    R9,w
        movwf   R22
        movf    R10,w
        movwf   R23
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    R14,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R24
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        setpage math_ok
        goto    math_ok


;       third degree minimax polynomial coefficients for 2**(x) on [.75,1]

EXP24HH0        equ     0x7E    ; EXP24HH0 = .99103284632
EXP24HH00       equ     0x7D
EXP24HH01       equ     0xB4
EXP24HH02       equ     0x54

EXP24HH1        equ     0x7E    ; EXP24HH1 = .73346850266
EXP24HH10       equ     0x3B
EXP24HH11       equ     0xC4
EXP24HH12       equ     0x97

EXP24HH2        equ     0x7C    ; EXP24HH2 = .17374128273
EXP24HH20       equ     0x31
EXP24HH21       equ     0xE9
EXP24HH22       equ     0x3C

EXP24HH3        equ     0x7B    ; EXP24HH3 = .10175678143
EXP24HH30       equ     0x50
EXP24HH31       equ     0x65
EXP24HH32       equ     0xDC

;       third degree minimax polynomial coefficients for 2**(x) on [.5,.75]

EXP24HL0        equ     0x7E    ; EXP24HL0 = .99801686089
EXP24HL00       equ     0x7F
EXP24HL01       equ     0x7E
EXP24HL02       equ     0x08

EXP24HL1        equ     0x7E    ; EXP24HL1 = .70586404164
EXP24HL10       equ     0x34
EXP24HL11       equ     0xB3
EXP24HL12       equ     0x81

EXP24HL2        equ     0x7C    ; EXP24HL2 = .21027360637
EXP24HL20       equ     0x57
EXP24HL21       equ     0x51
EXP24HL22       equ     0xF7

EXP24HL3        equ     0x7B    ; EXP24HL3 = .85566912730E-1
EXP24HL30       equ     0x2F
EXP24HL31       equ     0x3D
EXP24HL32       equ     0xB5

;       third degree minimax polynomial coefficients for 2**(x) on [.25,.5]

EXP24LH0        equ     0x7E    ; EXP24LH0 = .99979384559
EXP24LH00       equ     0x7F
EXP24LH01       equ     0xF2
EXP24LH02       equ     0x7D

EXP24LH1        equ     0x7E    ; EXP24LH1 = .69545887384
EXP24LH10       equ     0x32
EXP24LH11       equ     0x09
EXP24LH12       equ     0x98

EXP24LH2        equ     0x7C    ; EXP24LH2 = .23078300446
EXP24LH20       equ     0x6C
EXP24LH21       equ     0x52
EXP24LH22       equ     0x61

EXP24LH3        equ     0x7B    ; EXP24LH3 = .71952910179E-1
EXP24LH30       equ     0x13
EXP24LH31       equ     0x5C
EXP24LH32       equ     0x0C

;       third degree minimax polynomial coefficients for 2**(x) on [0,.25]

EXP24LL0        equ     0x7E    ; EXP24LL0 = .99999970657
EXP24LL00       equ     0x7F
EXP24LL01       equ     0xFF
EXP24LL02       equ     0xFB

EXP24LL1        equ     0x7E    ; EXP24LL1 = .69318585159
EXP24LL10       equ     0x31
EXP24LL11       equ     0x74
EXP24LL12       equ     0xA1

EXP24LL2        equ     0x7C    ; EXP24LL2 = .23944330933
EXP24LL20       equ     0x75
EXP24LL21       equ     0x30
EXP24LL22       equ     0xA0

EXP24LL3        equ     0x7A    ; EXP24LL3 = .60504944237E-1
EXP24LL30       equ     0x77
EXP24LL31       equ     0xD4
EXP24LL32       equ     0x08


_math2  code

; =========================================================================
;             24x24 Bit Unsigned Fixed Point Multiply 24x24 -> 48
; =========================================================================
;
;       Input:  24 bit unsigned fixed point multiplicand in AARGB0
;              24 bit unsigned fixed point multiplier in BARGB0
;       Output: 48 bit unsigned fixed point product in AARGB0
;       Result: AARG  <--  AARG x BARG

fxm2424u
        clrf    R2              ; clear partial product
        clrf    R3
        clrf    R4
        movf    R9,w
        movwf   R0
        movf    R10,w
        movwf   R1
        movf    R14,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R28
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        UMUL2424L
        setpage math_ok
        goto    math_ok




; =========================================================================
;          24x16 Bit Unsigned Fixed Point Multiply 24x16 -> 40
; =========================================================================
;
;       Input:  24 bit unsigned fixed point multiplicand in R9
;              16 bit unsigned fixed point multiplier in BARGB0
;       Output: 40 bit unsigned fixed point product in R9
;       Result: AARG  <--  AARG x BARG

fxm2416u
        clrf    R2              ; clear partial product
        clrf    R3
        movf    R9,w
        movwf   R0
        movf    R10,w
        movwf   R1
        movf    R14,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R28
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        UMUL2416L
        setpage math_ok
        goto    math_ok



; =========================================================================
;                               24bit FLOOR
; =========================================================================
;
;       Input:  24 bit floating point number in AEXP, R9, R10
;       Output: 24 bit floating point number in AEXP, R9, R10
;       Result: AARG  <--  FLOOR( AARG )
;
;       floor(x) evaluates the largest integer, as a float, not greater than x.

floor24
        clrf    R14             ; test for zero argument
        movf    R8,w
        btfsc   status,z
        retlw   0x00

        movf    R9,w
        movwf   R2              ; save mantissa
        movf    R10,w
        movwf   R3

        movlw   d'127'          ; computed unbiased exponent
        subwf   R8,w
        movwf   R1
        btfsc   R1,7
        goto    floor24zero

        sublw   0x10-1
        movwf   R0              ; save number of zero bits in R0
        movwf   R1

        btfsc   R1,3            ; divide by eight
        goto    floor24maskh

floor24maskl
        movlw   0x07            ; get remainder for mask pointer
        andwf   R0,f
        movlw   LOW floor24masktable
        addwf   R0,f
        movlw   HIGH floor24masktable
        btfsc   status,c
        addlw   0x01
        movwf   pclath
        incf    R0,w

        call    floor24masktable        ; access table for mask

        andwf   R10,f
        btfss   R9,7            ; if negative, round down
        goto    fl24_math_ok

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R30
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    R3,w
        subwf   R10,w
        btfss   status,z
        goto    floor24rndl
        goto    fl24_math_ok

floor24rndl
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        comf    R30,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R1
        incf    R1,w
        addwf   R10,f
        btfsc   status,z
        incf    R9,f
        btfss   status,z        ; has rounding caused carryout?
        goto    fl24_math_ok
        rrf     R9,f
        rrf     R10,f
        incfsz  R8,f            ; check for overflow
        goto    fl24_math_ok
        setpage math_error
        goto    math_error

floor24maskh
        movlw   0x07                    ; get remainder for mask pointer
        andwf   R0,f
        movlw   LOW floor24masktable
        addwf   R0,f
        movlw   HIGH floor24masktable
        btfsc   status,c
        addlw   0x01
        movwf   pclath
        incf    R0,w

        call    floor24masktable        ; access table for mask

        andwf   R9,f
        clrf    R10
        btfss   R9,7            ; if negative, round down
        goto    fl24_math_ok

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   R30
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    R3,w
        subwf   R10,w
        btfss   status,z
        goto    floor24rndh
        movf    R2,w
        subwf   R9,w
        btfss   status,z
        goto    floor24rndh
        goto    fl24_math_ok

floor24rndh
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        comf    R30,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R1
        incf    R1,w
        addwf   R9,f
        btfss   status,c                        ; has rounding caused carryout?
        goto    fl24_math_ok
        rrf     R9,f
        rrf     R10,f
        incfsz  R8,f
        goto    fl24_math_ok
        setpage math_error
        goto    math_error              ; check for overflow

floor24zero
        btfsc   R9,7
        goto    floor24minusone
        clrf    R8
        clrf    R9
        clrf    R10
        goto    fl24_math_ok

floor24minusone
        movlw   0x7F
        movwf   R8
        movlw   0x80
        movwf   R9
        clrf    R10

fl24_math_ok
        setpage math_ok
        goto    math_ok


;       table for least significant byte requiring masking, using pointer from
;       the remainder of the number of zero bits divided by eight.

floor24masktable
        movwf           pcl
        retlw           0xFF
        retlw           0xFE
        retlw           0xFC
        retlw           0xF8
        retlw           0xF0
        retlw           0xE0
        retlw           0xC0
        retlw           0x80
        retlw           0x00


; =========================================================================
;                       40-32bit nearest neighbour rounding
; =========================================================================
;       Nearest neighbor rounding
;       Input:  40 bit floating point number in AEXP, R9, R10, R14, R2
;       Output: 32 bit floating point number in AEXP, R9, R10, R14
;       Result: AARG  <-- RND( AARG )

rnd4032
        setpage math_ok
        btfss   R2,7            ; is MSB < 0x80?
        goto    math_ok

        bsf     status,c        ; set carry for rounding
        movlw   0x7F
        andwf   R2,w
        btfsc   status,z
        rrf     R14,w           ; select even if NSB = 0x80

        movf    R9,w
        movwf   R6              ; save sign
        bsf     R9,7            ; make MSB explicit

        bcf     status,z
        btfsc   status,c        ; round
        incf    R14,f
        btfsc   status,z
        incf    R10,f
        btfsc   status,z
        incf    R9,f

        setpage rnd4032ok
        btfss   status,z        ; has rounding caused carryout?
        goto    rnd4032ok
        rrf     R9,f            ; if so, right shift
        rrf     R10,f
        rrf     R14,f
        incf    R8,f            ; test for floating point overflow
        setpage math_error
        btfsc   status,z
        goto    math_error

rnd4032ok
        setpage math_ok
        btfss   R6,7
        bcf     R9,7            ; clear sign bit if positive
        goto    math_ok




; =========================================================================
;                               COMMON ROUTINES
; =========================================================================

; =====================
; Normalisation routine
; =====================
;
; Input in R8R9R10, with msb and other bits zero.  Sign in R6.
; Outputs normalised floating point number in R8R9R10
;
; Corrupts R7 (TEMP)

nrm2424
        clrf    R7              ; clear exponent decrement
        movf    R9,w            ; test if highbyte=0
        btfss   status,z
        goto    norm2424
        movf    R10,w           ; if so, shift 8 bits by move
        movwf   R9
        setpage returnzero
        btfsc   status,z        ; if highbyte=0, result=0
        goto    returnzero
        clrf    R10
        bsf     R7,3

norm2424
        setpage math_error
        movf    R7,w
        subwf   R8,f
        btfss   status,z
        btfss   status,c
        goto    math_error
        bcf     status,c        ; clear carry bit

norm2424a
        setpage norm2424a
        btfsc   R9,7            ; if MSB=1, normalization done
        goto    fixsign24
        rlf     R10,f           ; otherwise, shift left and
        rlf     R9,f            ; decrement exponent
        setpage math_error
        decfsz  R8,f
        goto    norm2424a
        goto    math_error      ; underflow if exponent = 0

fixsign24
        setpage math_ok
        btfss   R6,7
        bcf     R9,7            ; clear explicit MSB if positive
        goto    math_ok


; ===========================
; Returns from maths routines
; ===========================

returnzero32
        clrf    R9              ; result equals zero
        clrf    R10
        clrf    R14
        clrf    R2
        clrf    R8
        bsf     status,c        ; success!
        retlw   0

returnzero:                     ; special case
        clrf    R8
        clrf    R9
        clrf    R10

math_ok:
        bsf     status,c
        retlw   0


math_error:
        bcf     status,c
        retlw   0


; ==========================================================================


        global  conv2float_signed, conv2int_signed
        global  conv2float_unsigned, conv2int_unsigned
        global  multiplyfloat, dividefloat, power
        global  fpa32, fps32


        END

