; *************************************************************************
;                         Free Flight Computer
;
;                          MEASUREMENT MODULE
;
; *************************************************************************
;
; Development Tool:     MPLAB v4.12.12
; File Creation:        20/01/00                RNH
; Last Update:          09/08/04                RNH
; Code Issue:           A                       RNH
;
        list        P=16F76                     ; PIC 16F76
        include "P16C76r.INC"                   ; b is my lower case version
        include "macro.inc"                     ; my macros
        include "gpio.inc"                      ; hardware/configuration

        extern  R0, R1, R2, R3, R4, R5, R6, R7, R8
        extern  R9, R10, R11, R12, R13, R14, R15, R16
        extern  HEADWIND, DEST_HEIGHT, PRESSURE, IMPACTPRESSURE
        extern  ALTITUDE, TAS, TEMPERATURE, VARIO, PEAK_VARIO, AVERAGE_VARIO
        extern  VARIOBFR, VARIOPTR, PEAK_VARIO_TICK
        extern  ALT_M, ALT_C, ASI_M, ASI_C, MAXTAS, MAXALTITUDE
        extern  ADC0BFR, ADCTICK, ADCFRAME, ADCPTR, FLAGS
        extern  GLIDE, GLIDE_DISTANCE, MMIFLAGS, QFE_OFF, QNH_OFF
        extern  BEEP_ON_PERIOD, BEEP_OFF_PERIOD
        extern  DGV_MSB, DGV_LSB, DGV_INIT

        extern  conv2int_signed, conv2int_unsigned
        extern  conv2float_unsigned, conv2float_signed
        extern  multiplyfloat, dividefloat, power
        extern  fpa32, fps32, wait_10ms
        extern  write_eeprom, update_vario, do_vario_average


; ==========================================================================
;                                  MEASURE
; ==========================================================================
;
; Reads instruments and updates variables accordingly.  This includes
; doing the maths for altitude etc..
;
; Called every time the external ADC has a reading at 25Hz except when change
; channels to read temperature.  Then two adc outputs are skipped while the AD7705
; adjusts itself.  Hence need to compensate for this.
;
; Pressure readings are read into a 16 word buffer on every tick for averaging.
; Except when reading temperature of course!
;
; 0  Read vario, avg, pk.  Adj. sound.  13  -Update screen (read pressure, nMeasure low, select temp_adc)
; 1  -Update screen                     14                 (miss if changing adc)
; 2  Read headwind                      15                 (miss if changing adc)
; 3  Read destination height            16                 (read temp, select pressure, nMeasure high)
; 4  Read asi impact pressure           17                 (miss if changing adc)
; 5                                     18                 (miss if changing adc)
; 6  Calculate asi & s2f                19
; 7  -Update screen                     20  -Update screen
; 8  --Store barograph tas         (4)  21  --Store barograph qnh msb (0)
; 9  --Store barograph temperature (3)  22  --Store barograph qnh lsb (1)
; 10 --Store barograph vario       (2)  23
; 11 Calc pressure & altitude           24  Calc pressure & altitude
; 12 Adjust vario sound.
;
; Note that temperature (ticks 13 to 18) is only read every 10 frames.  Need
; to still read the pressure on tick 13 for two reasons:
;               1)      We have it
;               2)      Need to read before reset conversion (FSYNC -> 0)
; Hopefully won't notice the slightly out of date altitude reading when this happens!
;
; Note that ticks 8 to 10 and 21 to 22 are executed on return from this routine.

_measure        code

measure


; Read altitude adc if ADCFRAME < 9 or ADCTICK < 13 or ADCTICK > 18
; (ie. read unless frame is 9 and tick is 13 to 18)

        bsf     status,rp1      ; ... to bank 2   <--- **** NOTE ****

        movf    ADCFRAME,w      ;
        addlw   d'256'-d'9'     ;
        btfss   status,c        ; carry set if frame >= 9
        goto    read_pressure   ; else ok to read pressure adc

        movf    ADCTICK,w       ;
        addlw   d'256'-d'13'    ;
        btfss   status,c        ; carry set if tick >= 13
        goto    read_pressure   ; else ok to read pressure adc

        movf    ADCTICK,w       ;
        addlw   d'256'-d'19'    ;
        btfsc   status,c        ; carry clear if tick <= 18
        goto    read_pressure   ; else ok to read pressure adc


read_temperature

; ---------------------------------------------------
; ----- Read temperature special case, do these ticks
; ---------------------------------------------------
;
; 13  Update screen, read pressure, select temp_adc, nMeasure low, increment past misses
; 14                (miss if changing adc)
; 15                (miss if changing adc)
; 16  Read temp, select pressure, nMeasure high, increment past misses
; 17                (miss if changing adc)
; 18                (miss if changing adc)


; ..... tick 13 (also covers 14 and 15)

        movf    ADCTICK,w
        sublw   d'13'
        btfss   status,z
        goto    m_tempread

m_tempstart
        bsf     FLAGS,3         ; request screen update

        incf    ADCTICK,f
        incf    ADCTICK,f       ; these ticks will be missed as change channels

        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****

        movlw   b'00000000'     ; Drive nMEASURE low to switch on temperature sensor
        movwf   porta

        movlw   PRESSURE_ADC
        movwf   R2
        call    read_ext_adc_no_wait    ; returns in R2R3; alters R0, R1
        movf    R2,w
        movwf   R4
        movf    R3,w
        movwf   R5                      ; copy pressure reading to R4R5

        movlw   TEMP_ADC        ; need to have read ADC reading before can
        movwf   R2              ; start conversion (see page 21 of AD7705 d'sheet)
        movlw   TEMP_GAIN
        movwf   R3
        call    set_ext_adc     ; select temperature channel for use by temperature frame

        movf    R4,w
        movwf   R2
        movf    R5,w
        movwf   R3
        goto    store_p_adc     ; store pressure reading obtained



; ..... tick 16 (also covers 17 and 18)

m_tempread
        incf    ADCTICK,f
        incf    ADCTICK,f       ; these ticks will be missed as change channels

        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        movlw   TEMP_ADC
        movwf   R2
        call    read_ext_adc_no_wait    ; returns in R2R3; alters R0, R1

        movf    R2,w
        movwf   R4
        movf    R3,w
        movwf   R5                      ; copy R2R3 into R4R5

        movlw   b'00000100'     ; Drive nMEASURE back high as finished with temperature now
        movwf   porta

; Re-select pressure adc for scheduled reads (corrupts R0 to R3)

        movlw   PRESSURE_ADC
        movwf   R2
        movlw   PRESSURE_GAIN
        movwf   R3
        call    set_ext_adc

; Have raw temperature adc reading in R4R5, now need to convert into
; internal representation:
;       Subtract 0.132V (3460 decimal, 0d84 hex) for -10 degrees C
;       Divide by 52.43 for 0.2 degree C (lsb) increments, say 52 (0.5C error full scale)

process_temperature

        movlw   d'256'-084      ; subtract lsb.  carry set if >= 84
        addwf   R5,f            ;                zero set if = 84
        btfsc   status,c        ; if < c2 (ie c + z clear) need to borrow
        goto    sub_lsb_done
        btfsc   status,z
        goto    sub_lsb_done
        decf    R4,f
sub_lsb_done
        movlw   d'256'-00d
        addwf   R4,f            ; easy for msb subtraction!

; Now divide by 52

        clrf    R0              ; store R4R5 divided by 52 in R0R1
        clrf    R1              ; remainder minus 52 will be in R4R5
divide_temp
        movlw   d'256'-d'52'    ; subtract 52
        addwf   R5,f            ; from R3
        btfsc   status,c        ; if < c2 (ie c + z clear) need to borrow
        goto    div_temp_ok
        btfsc   status,z
        goto    div_temp_ok
        decf    R4,f            ; borrow from upper byte (ie. R4)
        movf    R4,w
        sublw   0ff             ; gone too far?
        btfsc   status,z
        goto    div_temp_done   ; if yes, then all done here.
div_temp_ok                     ; Subtracted 52 OK, so increment
        incf    R1,f            ; divide count and go back for
        btfsc   status,z        ; more
        incf    R0,f
        goto    divide_temp
div_temp_done

; Now have R4R5/52 in R0R1
; This is the temperature in 0.2degrees per lsb starting from -10 degrees C!
; As only interested in one byte (ie -10 to +41) clip to max if greater, otherwise
; just store.

        movlw   0ff
        movf    R0,f            ; move onto itself to set zero flag if applicable
        btfss   status,z
        movwf   R1              ; limit to +41 degrees (255) if non-zero upper byte

        movf    R1,w
        movwf   TEMPERATURE

        goto    done_measure


; --------------------------------------------------------------
; ----- Read external pressure adc, then execute sheduler proper
; --------------------------------------------------------------

read_pressure
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        movlw   PRESSURE_ADC
        movwf   R2
        call    read_ext_adc_no_wait    ; returns in R2R3; alters R0, R1

store_p_adc
        bsf     status,rp1      ; ... to bank 2   <--- **** NOTE ****
        bsf     status,irp      ; ... to bank 2/3 for indirect    <--- **** NOTE ****
        movf    ADCPTR,w        ; get ADCPTR
        addlw   ADC0BFR         ; add start of adc0 buffer
        movwf   fsr             ; point current position using fsr
        movf    R2,w
        movwf   indf
        incf    fsr,f
        movf    R3,w
        movwf   indf            ; write reading to buffer
        bcf     status,irp      ; ... to bank 0/1 for indirect    <--- **** NOTE ****

        incf    ADCPTR,f
        incf    ADCPTR,w
        addlw   d'256'-d'32'
        btfsc   status,z        ; zero flag set if adcptr = 32
        movlw   d'256'-d'32'    ; if so then alter wreg so adcptr end up zero
        addlw   d'32'
        movwf   ADCPTR          ; incremented and wrapped adc pointer


; --------------------------------------------------------
; ----- Request update screen if adctick is 1, 7, 13 or 20
; --------------------------------------------------------

m_check0
        movf    ADCTICK,w
        sublw   d'1'
        btfsc   status,z
        goto    m_check0pass

        movf    ADCTICK,w
        sublw   d'7'
        btfsc   status,z
        goto    m_check0pass

        movf    ADCTICK,w
        sublw   d'13'
        btfsc   status,z
        goto    m_check0pass

        movf    ADCTICK,w
        sublw   d'20'
        btfss   status,z
        goto    m_check1

m_check0pass
        bsf     FLAGS,3         ; request screen update
        goto    done_measure    ; exit via this code so updates ADCTICK



; ----------------------------------
; ----- Read headwind if adctick = 2
; ----------------------------------

m_check1
        movf    ADCTICK,w
        sublw   d'2'
        btfss   status,z
        goto    m_check2
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****

; This is given by an analogue potentiometer
; Therefore 128 = 0mph, 255=lots of headwind

        movlw   HEAD_ADC
        call    single_adc      ; result in adres, screws up R0
        movf    adres,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        addlw   d'256'-d'128'   ; subtract 128 to store in 2s complement
        movwf   HEADWIND
        goto    done_measure



; -----------------------------------------------
; ----- Read required glide height if adctick = 3
; -----------------------------------------------

m_check2
        movf    ADCTICK,w
        sublw   d'3'
        btfss   status,z
        goto    m_check3
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****

; This is given by an analogue potentiometer

        movlw   DEST_HEIGHT_ADC
        call    single_adc      ; result in adres, screws up R0
        movf    adres,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   DEST_HEIGHT
        goto    done_measure


m_check3

;       ---------------------------------------------------------
;       ----- Calculate vario if adctick is 0
;       ----- Update vario sound if adctick is 0 or 12
;       ---------------------------------------------------------
        movf    ADCTICK,w
        btfsc   status,z
        goto    m_check3pass

        movf    ADCTICK,w
        sublw   d'12'
        btfss   status,z
        goto    m_check4

;       If approximate a linear relationship between altitude and
;       pressure then get 0.05mbar is 1up.  By a strange quirk of
;       fate this works out to be 5 x lsb readings.  So by just
;       subtracting the difference, and adding 128, we have a value
;       for VARIO!
;
;       The pressure adc reading we have just taken is in R2R3
;
;       If going up, then reading will be lower.
;               ie. VARIO = (old - current)
;       or as don't want to corrupt R2R3 reading:
;                    VARIO = -(current - old)

m_check3pass

; Obtain average of previous pressure readings into R2R3
; In bank 2 when arrive here

        bsf     status,irp      ; ... to bank 2/3 for indirect    <--- **** NOTE ****
        movf    ADCPTR,w        ; get ADCPTR
        addlw   ADC0BFR-8       ; add start of adc0 buffer, go back 4 x 2 byte samples
        sublw   ADC0BFR         ; w = ADC0BFR-w, carry clear if w was > ADC0BFR
        btfsc   status,z
        goto    done_wrap               ; skip if result zero (start of buffer)
        btfsc   status,c        ; also skip if carry clear (within buffer)
        addlw   d'256'-d'32'    ; add on length of buffer (16 words) if so
done_wrap
        sublw   ADC0BFR         ; restore check
        movwf   fsr

        movlw   d'4'            ; average four samples
        movwf   R5
        clrf    R4
        clrf    R2
        clrf    R3              ; add words into R4R2R3

sum_samples
        incf    fsr,f           ; point to low byte of word
        movf    indf,w
        addwf   R3,f
        btfss   status,c        ; carry?
        goto    do_highbyte     ; branch if no
        incf    R2,f            ; yes, so increment R2
        btfsc   status,z        ; as incf only affects zero bit!
        incf    R4,f            ; and increment R4 if this overflows also
do_highbyte
        decf    fsr,f           ; point to high byte of word
        movf    indf,w
        addwf   R2,f
        btfsc   status,c        ; carry?
        incf    R4,f            ; yes, so increment R4

        incf    fsr,f
        incf    fsr,f           ; point to next word

        movf    fsr,w
        addlw   d'256'-d'32'
        sublw   ADC0BFR                 ; past end of the buffer?
        btfss   status,z                ; zero set if yes
        goto    loop_samples            ; nope
        movlw   ADC0BFR
        movwf   fsr

loop_samples
        decfsz  R5,f
        goto    sum_samples     ; do all samples

; now divide by four, don't care about clearing carry as going to discard R1

        rrf     R4,f
        rrf     R2,f
        rrf     R3,f            ; divide by two

        rrf     R4,f
        rrf     R2,f
        rrf     R3,f            ; divide by four

        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        bcf     status,irp      ; ... to bank 0/1 for indirect    <--- **** NOTE ****

        movf    DGV_INIT,w
        btfsc   status,z
        goto    dgv_initialised ; already initialised

        decfsz  DGV_INIT,f
        goto    done_measure    ; insufficient readings to init yet

        movf    R2,w            ; initialise
        movwf   DGV_MSB
        movf    R3,w
        movwf   DGV_LSB
        goto    done_measure


        ; Calculate difference into R1R0 without modifying
        ; DGV or R2R3.

dgv_initialised

        comf    R2,w
        movwf   R0
        comf    R3,w
        movwf   R1              ; R0R1 = 1s complement of R2R3
        incf    R1,f
        btfsc   status,z
        incf    R0,f            ; R0R1 = -R2R3 in 2s complement

        movf    DGV_MSB,w
        addwf   R0,f
        movf    DGV_LSB,w
        addwf   R1,f
        btfsc   status,c
        incf    R0,f            ; R0R1 = DGV - R2R3

        ; Now clip
        btfsc   R0,7            ; set if negative
        goto    clip_down
clip_up
        movf    R0,w
        btfsc   status,z
        goto    have_digi_vario ; if msb is zero, then reading in lsb fine
        movlw   0x7F
        movwf   R1
        goto    have_digi_vario
clip_down
        incf    R0,w
        btfsc   status,z
        goto    have_digi_vario ; if msb is 0xff, then reading in lsb fine
        movlw   0x80
        movwf   R1


;       Now have our difference reading, clipped, in R1

have_digi_vario

        bsf     status,rp1      ; ... to bank 2   <--- **** NOTE ****
        movf    ADCTICK,w
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        btfsc   status,z
        goto    fullupdate


;       Take an intermediate reading and add it to averaged buffer and
;       update tone.  OK to do this as all averaging out over even samples.

partialupdate

        rlf     R1,w            ; double as only half-way through
        andlw   b'01111110'     ; vario differentiation period
        btfsc   R1,7            ;
        iorlw   b'10000000'     ;
        movwf   R1

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        setpage do_vario_average
        call    do_vario_average
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        setpage done_measure
        goto    done_measure    ; done


;       Full update of vario based on delta from last adc reading

fullupdate
        movf    R1,w
        movwf   VARIO

        movf    R2,w                    ; Update previous DGV
        movwf   DGV_MSB                 ; alti reading
        movf    R3,w
        movwf   DGV_LSB

        setpage update_vario
        goto    update_vario


; ----------------------------------------------
; ----- Read asi impact pressure if adctick is 4
; ----------------------------------------------

m_check4
        movf    ADCTICK,w
        sublw   d'4'
        btfss   status,z
        goto    m_check5

        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        movlw   ASI_ADC
        call    single_adc      ; result in adres, screws up R0
        movf    adres,w
        movwf   IMPACTPRESSURE  ; store latest reading here

        addlw   d'256'-2        ; overflow if pressure >= 2
        btfss   status,c        ; if so, then carry set
        clrf    IMPACTPRESSURE  ; if not, then zero reading as in the noise

        goto    done_measure



; ------------------------------------------------
; ----- Calculate ASI and S2F info if adctick is 6
; ------------------------------------------------

m_check5
        movf    ADCTICK,w
        sublw   d'6'
        btfss   status,z
        goto    m_check6


; ----- Calculate ASI first

        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****

        clrf    R9
        movf    IMPACTPRESSURE,w
        movwf   R10

; ..... convert asi impact pressure reading to floating

        setpage conv2float_unsigned
        call    conv2float_unsigned     ; returns in R8R9R10; alters R11, R12

        movf    ASI_M,w
        movwf   R11
        movf    ASI_M+1,w
        movwf   R12
        movf    ASI_M+2,w
        movwf   R13
        setpage multiplyfloat   ; returns R8R9R10 x R11R12R13
        call    multiplyfloat   ; impact pressure in mbar in R8R9R10
        clrf    R14             ; (alters R7,R12,R14,R15,R16)

        movf    ASI_C,w
        movwf   R11
        movf    ASI_C+1,w
        movwf   R12
        movf    ASI_C+2,w
        movwf   R13
        clrf    R15                     ; Copy ASI_C into
        setpage fpa32                   ; R11R12R13 and clear R15 (32 bit)
        call    fpa32                   ; add together into R8R9R10R14


; ..... calculate true air speed
;
; ASI = ((2 * impact pressure) / rho) ^ 0.5
;
; where rho is the density of air, and any unit conversions needed.
;
; For impact pressure in mbar and velocity in mph density
; rho is 0.00245.  Therefore if impact pressure of 3 mbar get
; asi of ((2 * 3.00) / 0.00245) ^ 0.5 = 49.5 mph.
;
; If want in kph instead then rho is 0.0009452 instead.
;
; This is all in IAS, which is more useful for polars etc. as alters
; with pressure.  To turn into TAS we use the following approximation:
;
; EXAMPLE: Given IAS is 140kt and ALT is 6,000 feet. Find TAS.
;       2% x 6 = 12% (.12)
;       140 (IAS) x 1.12 = 156.8 kt. (TAS)
;
; If greater than MAXTAS then update MAXTAS accordingly

calculate_tas

; Multiply by two by just incrementing mantissa (?) in R8R9R10
        incf    R8,f

; Now divide by 0.00245 which is 762090 in floating point
        movlw   0x76
        movwf   R11
        movlw   0x20
        movwf   R12
        movlw   0x90
        movwf   R13

        setpage dividefloat     ; divide R8R9R10 by 0.00245
        call    dividefloat     ; stores in R8R9R10
                                ; alters R6, R7, R14, R15, R16

; Now find the square root by calling power function.  Takes a while!

        movlw   0x7E
        movwf   R11
        clrf    R12
        clrf    R13             ; R11R12R13 = 0x7e0000 = 0.5
        setpage power           ; R8R9R10 = R8R9R10 ^ R11R12R13
        call    power           ; alters R1 - R27
        setpage have_ias

; Now have IAS in R8R9R10
; Calculate percentage correction to apply
;
; SUMMARY:  Divide ALTITUDE (signed 2s complement) by 1000 feet to find % multiplier required
;
; DETAIL:
; As altitude increases , air density decreases. The impact pressure at
; the port of the pitot tube is less at higher altitudes. The airplane is actually
; traveling through the air faster than indicated on the ASI. Consequently as altitude
; increase, Indicated Airspeed decreases.
;
; A mathematical correction factor must be applied to Indicated Airspeed
; (or Calibrated Airspeed) to arrive at a correct True Airspeed (TAS). This calculation
; can be made or an approximate correction can be made by adding 2 percent per 1,000 feet
; of altitude to the IAS.
;
; EXAMPLE: Given IAS is 140kt and ALT is 6,000 feet. Find TAS.
;
; 2% x 6 = 12% (.12)
;
; 140 x 0.12 = 16.8
; 140 + 16.8 = 156.8 kt. (TAS)


have_ias
        btfsc   ALTITUDE,7      ; is qne negative?  if so TAS approx IAS
        goto    have_tas

        movf    ALTITUDE,w
        movwf   R1
        movf    ALTITUDE+1,w
        movwf   R0              ; R1R0 = qne, will divide by 1000

        movlw   0x7f
        movwf   R11
        clrf    R12             ; correction factor will be in R11R12R13
        clrf    R13             ; 1.00 = 7f0000,  1.02 = 7f028f,  1.04 = 7f051e ...
find_ias2tas_factor
        movlw   d'256'-03       ; 03e8h = 1000
        addwf   R1,f
        btfss   status,c        ; if carry then still positive
        goto    apply_ias2tas
        movlw   d'256'-0e8
        addwf   R0,f
        btfsc   status,c        ; if carry then still positive
        goto    inc_factor
        movf    R1,f            ; test R1 for zero
        btfsc   status,z
        goto    apply_ias2tas   ; finished divide if msb already zero
        decf    R1,f
inc_factor
        movlw   0x8f
        addwf   R13,f
        btfsc   status,c
        incf    R12,f           ; add to lsb of correction factor
        movlw   0x02
        addwf   R12,f           ; add to msb of correction factor
        btfsc   status,c
        incf    R11,f           ; just in case > 50,000 feet!
        goto    find_ias2tas_factor

apply_ias2tas
        setpage multiplyfloat
        call    multiplyfloat   ; R8R9R10 = R8R9R10 (IAS) x R11R12R13 (TAS correction)
        clrf    R14             ; (alters R7,R12,R14,R15,R16)

have_tas                                ; R8R9R10 now contains true air speed
        setpage conv2int_unsigned       ; convert into R9R10
        call    conv2int_unsigned       ;
        setpage done_tas

        movf    R10,w
        movwf   TAS                     ; now have air speed in mph
        
        subwf   MAXTAS,w        ; w= TAS - MAXTAS; carry set if MAXTAS > TAS
        btfsc   status,c
        goto    done_tas
        movf    TAS,w
        movwf   MAXTAS
done_tas


; ----- Now calculate S2F angles, arrive here in bank 0

; First check if going backwards.  If yes then set glide angle and distance to zero.


; convert to mph as currently HEADWIND is in 2s complement with 4 x lsb = 1mph

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    HEADWIND,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R0
        bcf     status,c
        rrf     R0,f
        rrf     R0,f            ; divide by four to get mph
        movf    R0,w
        btfss   R0,5            ; is headwind negative?
        goto    pos_headwind
        movlw   b'11000000'
        iorwf   R0,f
        comf    R0,f
        incf    R0,w            ; set upper bits to negative, then make all positive into wreg
        addwf   TAS,w           ; add on windspeed; so wreg now same as tas + tailwind
        goto    check_if_lifty
pos_headwind
        subwf   TAS,w           ; carry clear if HEADWIND > TAS (ie going backwards)
        btfss   status,c
        goto    zero_glide
        btfsc   status,z
        goto    zero_glide

; Now check if going up

check_if_lifty
        btfss   VARIO,7
        goto    zero_glide       ; msb of vario clear if going up

; We're now ok to calculate glide angle
;   angle = TAS - HEADWIND / sink
;         = TAS + TAILWIND / sink
;         = wreg / sink

; temporarily store tas-headwind
        movwf   R0

; get sink into R11R12R13
        clrf    R9
        comf    VARIO,w
        movwf   R10
        incf    R10,f                   ; sink (5 = 100fpm) in R9R10
        setpage conv2float_unsigned
        call    conv2float_unsigned     ; returns in R8R9R10; alters R11, R12
        movf    R8,w
        movwf   R11
        movf    R9,w
        movwf   R12
        movf    R10,w
        movwf   R13

; convert TAS-HEADWIND into R8R9R10
        clrf    R9
        movf    R0,w
        movwf   R10
        setpage conv2float_unsigned
        call    conv2float_unsigned     ; returns in R8R9R10; alters R11, R12

; now divide
        setpage dividefloat             ; divides R8R9R10 by R11R12R13
        call    dividefloat             ; into R8R9R10
                                        ; alters R6, R7, R14, R15, R16

; R8R9R10 = (TAS-HEADWIND) in mph / sink in (5=100fpm)
; now multiply by appropriate magic number to get angle
;
; 1mph = 88fpm = 0.88up/down
; multiply by 88 to get in fpm on top
; then divide by 20 to get in 1/20ths fpm on top
; ie.  multiply by 4.4
;
; However, actually want glide angle * 10 to display, so magic number
; we multiply by becomes 44 (floating point 843000)
        movlw   084
        movwf   R11
        movlw   030
        movwf   R12
        movlw   000
        movwf   R13
        setpage multiplyfloat           ; returns R8R9R10 x R11R12R13
        call    multiplyfloat           ; in R8R9R10; alters R7,R12,R14,R15,R16

; now convert to integer

        setpage conv2int_signed
        call    conv2int_signed         ; glide angle * 10 in R9R10

        setpage store_glide
        movf    R9,w
        btfsc   status,z                ; R9 hopefully zero, if not huge angle so limit
        goto    store_glide
        movlw   0xff
        movwf   R10                     ; clip to glide of 25.5:1 !!!

store_glide
        movf    R10,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   GLIDE


; Calculate glide distance
;
; get destheight in floating point, as an altitude (multiply by 15.625, 827A00)

        clrf    R9
        movf    DEST_HEIGHT,w   ; 0 = 0feet, 64 = 1000feet, 128 = 2000feet ...
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R10
        setpage conv2float_unsigned
        call    conv2float_unsigned     ; returns in R8R9R10; alters R11, R12

        movlw   082
        movwf   R11
        movlw   07a
        movwf   R12
        movlw   000
        movwf   R13
        setpage multiplyfloat           ; returns R8R9R10 x R11R12R13
        call    multiplyfloat           ; in R8R9R10; alters R7,R12,R14,R15,R16

; now have dest_height in R8R9R10; copy to R1R2R3

        movf    R8,w
        movwf   R1
        movf    R9,w
        movwf   R2
        movf    R10,w
        movwf   R3

; Now obtain altitude in floating point form, in R8R9R10

; note - altitude is current indicated height (qnh/qfe) so work out
;        what it is.

        setpage calc_altitude
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        btfsc   MMIFLAGS,6
        goto    use_qnh
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    QFE_OFF,w
        movwf   R9
        movf    QFE_OFF+1,w
        movwf   R10
        goto    calc_altitude
use_qnh
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    QNH_OFF,w
        movwf   R9
        movf    QNH_OFF+1,w
        movwf   R10
calc_altitude
        movf    ALTITUDE+1,w
        addwf   R10,f
        btfsc   status,c
        incf    R9,f
        movf    ALTITUDE,w
        addwf   R9,f            ; altitude now in R1R0, signed

        setpage conv2float_signed
        call    conv2float_signed     ; returns in R8R9R10; alters R11, R12

; now calculate altitude - dest_height

        movf    R1,w
        movwf   R11
        movf    R2,w
        movwf   R12
        movf    R3,w
        movwf   R13

        clrf    R14
        clrf    R15
        setpage fps32
        call    fps32          ;  R8R9R10 = R8R9R10 (altitude) - R11R12R13 (destheight)


; if negative, ie dest_height >= altitude, then zero glide distance
; Check this by examining msb of R9; is set if negative

        setpage zero_glide_distance
        btfsc   R9,7
        goto    zero_glide_distance

; Save in R1R2R3

        movf    R8,w
        movwf   R1
        movf    R9,w
        movwf   R2
        movf    R10,w
        movwf   R3

; Now can work out how far we can go ...
;
; 1mile = 1.61km = 5280 feet  (therefore 1km = 3280 feet)
; distance in km = ((altitude - destheight) * glideangle)/3280  (892C80)
;
; GLIDE angle is stored as glide angle * 10 so in fact get distance * 10
; as a result of this; which is what we want anyway.

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    GLIDE,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movwf   R10
        clrf    R9
        setpage conv2float_signed
        call    conv2float_signed     ; returns in R8R9R10; alters R11, R12

        movf    R1,w
        movwf   R11
        movf    R2,w
        movwf   R12
        movf    R3,w
        movwf   R13                     ; recover altitude-destheight

        setpage multiplyfloat           ; returns R8R9R10 x R11R12R13
        call    multiplyfloat           ; in R8R9R10; alters R7,R12,R14,R15,R16

        movlw   089
        movwf   R11
        movlw   02C
        movwf   R12
        movlw   080
        movwf   R13

        setpage dividefloat             ; divides R8R9R10 by R11R12R13
        call    dividefloat             ; into R8R9R10
                                        ; alters R6, R7, R14, R15, R16

        setpage conv2int_unsigned
        call    conv2int_unsigned       ; distance * 10 in R9R10

; Clip if > 255 (ie 25.5km)

        setpage max_glide_distance
        movf    R9,w
        btfss   status,z
        goto    max_glide_distance      ; clip if R9 > 0

        movf    R10,w                   ; have distance ( x 10 of course)!
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        goto    store_glide_distance


; Going backwards / stationary / going up so glide angle meaningless.
zero_glide
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        clrf    GLIDE
zero_glide_distance
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        clrf    GLIDE_DISTANCE
        goto    done_measure


; Clip distance
max_glide_distance
        movlw   0xff
store_glide_distance
        movwf   GLIDE_DISTANCE
        goto    done_measure


; -----------------------------------------------
; ----- Calculate altitude if adctick is 15 or 31
; -----------------------------------------------

m_check6
        movf    ADCTICK,w
        sublw   d'15'
        btfsc   status,z
        goto    m_check6pass

        movf    ADCTICK,w
        sublw   d'31'
        btfss   status,z
        goto    done_measure            ; All done, no more scheduled events

m_check6pass


; ----- Average pressure sensor adc buffer

        movlw   ADC0BFR         ;
        movwf   fsr             ; point to start of buffer
        bsf     status,irp      ; ... to bank 2/3 for indirect    <--- **** NOTE ****
        setpage average_word_buffer
        call    average_word_buffer
        bcf     status,irp      ; ... to bank 0/1 for indirect    <--- **** NOTE ****

; Now have averaged pressure adc reading in R9R10 ready for conversion
; into floating point and a pressure reading


; ----- Convert to floating point pressure reading

        bcf     status,rp1              ; ... to bank 0   <--- **** NOTE ****
        setpage conv2float_unsigned
        call    conv2float_unsigned     ; returns in R8R9R10; alters R11, R12
        setpage convert_adc_to_pressure

convert_adc_to_pressure
        movf    ALT_M,w
        movwf   R11
        movf    ALT_M+1,w
        movwf   R12
        movf    ALT_M+2,w
        movwf   R13
        setpage multiplyfloat           ; returns R8R9R10 x R11R12R13
        call    multiplyfloat           ; in R8R9R10; alters R7,R12,R14,R15,R16
        clrf    R14

        movf    ALT_C,w
        movwf   R11
        movf    ALT_C+1,w
        movwf   R12
        movf    ALT_C+2,w
        movwf   R13
        clrf    R15                     ; Copy ALT_C into
        setpage fpa32                   ; R11R12R13 and clear R15 (32 bit)
        call    fpa32                   ; add together into R8R9R10R14
        setpage store_pressure


; ----- Store pressure

store_pressure
        bsf     status,rp0              ; ... to bank 1   <--- **** NOTE ****
        movf    R8,w
        movwf   PRESSURE
        movf    R9,w
        movwf   PRESSURE+1
        movf    R10,w
        movwf   PRESSURE+2


; ----- Calculate altitude  (from international formula)
;
;       Altitude /feet = [1-(Pressure/1013)^0.1903]*145366.4446
;       Store in ALTITUDE as unsigned word
;       Add QFE_OFF to working copy
;       If greater than MAXALTITUDE then update MAXALTITUDE accordingly

        movf    PRESSURE,w
        movwf   R8
        movf    PRESSURE+1,w
        movwf   R9
        movf    PRESSURE+2,w
        movwf   R10
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        movlw   0x88
        movwf   R11
        movlw   0x7d
        movwf   R12
        movlw   0x40
        movwf   R13
        setpage dividefloat             ; divides R8R9R10 by R11R12R13
        call    dividefloat             ; into R8R9R10
                                        ; alters R6, R7, R14, R15, R16
        movlw   0x7c
        movwf   R11
        movlw   0x42
        movwf   R12
        movlw   0xde
        movwf   R13
        setpage power                   ; returns (R8R9R10)^R11R12R13
        call    power                   ; in R8R9R10; alters R1-R24

        movf    R8,w
        movwf   R11
        movf    R9,w
        movwf   R12
        movf    R10,w
        movwf   R13                     ; copy (pressure/1013) to R11R12R13
        clrf    R15                     ; and clear 32 bit part

        movlw   0x7f
        movwf   R8
        clrf    R9
        clrf    R10
        clrf    R14                     ; R8R9R10 = 1

        setpage fps32
        call    fps32                   ; R8R9R10 = 1-(pressure/1013)

        movlw   0x90
        movwf   R11
        movlw   0x0d
        movwf   R12
        movlw   0xf5
        movwf   R13
        setpage multiplyfloat
        call    multiplyfloat           ; multiply by (temp/lapse_rate) for ...
        setpage conv2int_signed
        call    conv2int_signed         ; altitude in R9R10 in feet
        setpage store_altitude

store_altitude
        movf    R9,w
        movwf   ALTITUDE
        movf    R10,w
        movwf   ALTITUDE+1


; ----- Perform 16 bit signed compare to see if update MAXALTITUDE
;
; Code by David Cary 2001-03-30
;
; Y = MAXALTITUDE
; X = ALTITUDE

compare_signed16
        movf    MAXALTITUDE,w  ; Yhi
        xorlw   0x80
        movwf   R10             ; temp
        movf    ALTITUDE,w      ; Xhi
        xorlw   0x80
        subwf   R10,w           ; temp, result is Y-X
        goto    are_they_equal

compare_unsigned_16
        movf    ALTITUDE,w      ; Xhi
        subwf   MAXALTITUDE,w   ; Yhi, result is Y-X

are_they_equal
        btfss   status,z
        goto    results16
        movf    ALTITUDE+1,w    ; Xlo
        subwf   MAXALTITUDE+1,w ; Ylo, result is Y-X

results16
        ; if X=Y then now Z=1.
        ; if Y<X then now C=0.
        ; if X<=Y then now C=1.
        btfsc   status,c
        goto    done_altitude
        movf    ALTITUDE,w
        movwf   MAXALTITUDE
        movf    ALTITUDE+1,w
        movwf   MAXALTITUDE+1
done_altitude
;; Not needed as next line!
;        goto    done_measure



; ---------------------------------------------
; ----- Done, return to scheduler (in any bank)
; ---------------------------------------------

done_measure
        bcf     status,rp1      ; ... to bank 0   <--- **** NOTE ****
        retlw   0


; ==========================================================================
;                            MEASUREMENT ROUTINES
; ==========================================================================

; ***********************
; * AVERAGE WORD BUFFER *
; ***********************
;
; Average 16 word buffer into R9R10
;
; Altered on exit:      R0, R1 returning in R9R10
;
; Entry conditions:     fsr and irp must be setup


average_word_buffer
        movlw   d'16'           ; sum all 16 words in buffer
        movwf   R0
        clrf    R1
        clrf    R9
        clrf    R10             ; add words into R1R9R10

sum_buffer
        incf    fsr,f           ; point to low byte of word
        movf    indf,w
        addwf   R10,f
        btfss   status,c        ; carry?
        goto    sum_highbyte    ; branch if no
        incf    R9,f            ; yes, so increment R9
        btfsc   status,z        ; as incf only affects zero bit!
        incf    R1,f            ; and increment R1 if this overflows also
sum_highbyte
        decf    fsr,f           ; point to high byte of word
        movf    indf,w
        addwf   R9,f
        btfsc   status,c        ; carry?
        incf    R1,f            ; yes, so increment R1
sum_doneword
        incf    fsr,f
        incf    fsr,f           ; point to next word
        decfsz  R0,f
        goto    sum_buffer      ; sum all 16 words

; now divide by sixteen, don't care about clearing carry as going
; to discard R1

        rrf     R1,f
        rrf     R9,f
        rrf     R10,f           ; divide by two

        rrf     R1,f
        rrf     R9,f
        rrf     R10,f           ; divide by four

        rrf     R1,f
        rrf     R9,f
        rrf     R10,f           ; divide by eight

        rrf     R1,f
        rrf     R9,f
        rrf     R10,f           ; divide by sixteen

        retlw   0



; ****************************************
; * SINGLE TASKING ANALOGUE CHANNEL READ *
; ****************************************
;
; Takes a sample from the input channel selected in WREG, result is output
; approx (20us + sample time) later and can be read from 'adres' in bank 0.
;
; Altered on exit:      R0, WREG, ADCON0
;
; Entry conditions:     Bank 0 must be selected
;

single_adc
        movwf   R0
        bcf     status,c        ; clear carry - gets rotated through
        rlf     R0,f
        rlf     R0,f
        rlf     R0,f
        movf    adcon0,w
        andlw   b'11000111'     ; mask out old channel
        iorwf   R0,w            ; or in new shifted channel
        movwf   adcon0          ; and select

        bsf     adcon0,adon     ; turn on a/d module

        movlw   d'256'-6        ;
settle                          ;
        addlw   1               ; wait min 4us from last acq, 12us to settle
        btfss   status,z        ; each instruction is 1us
        goto    settle          ;

        bsf     adcon0,go_done  ; set conversion going
get_reading
        btfsc   adcon0,go_done  ; wait to read value
        goto    get_reading
        bcf     adcon0,adon     ; turn off a/d module
        retlw   0



; ******************************
; * RESET EXTERNAL ADC ROUTINE *
; ******************************
;
; Resets the single adc interface, and sets up the ADC with
; our requirements

reset_ext_adc
        call    reset_adc_interface     ; initialise interface
        call    calibrate_ext_adc       ; calibrate channels & init setup
        retlw   0



; ************************************
; * SINGLE TASKING EXTERNAL ADC READ *
; ************************************
;
; Input channel (1 or 2) supplied in R2.
; Returns 16 bit ADC reading in R2R3
;
; Also altered on exit: R0, R1, WREG
;                       R2 corrected if > 1
; Entry conditions:     Bank 0 must be selected
;
; Assumes nADCREADY is low as just goes ahead and reads.  Should be as
; everything scheduled from this.

read_ext_adc
        btfsc   porta,nADCREADY
        goto    read_ext_adc    ; wait for interface to be ready

read_ext_adc_no_wait
        movlw   CHANNELMASK
        andwf   R2,w
        addlw   READ_OP + DATA_REG      ; select channel for next conversion and read current
        movwf   R0
        call    write_byte      ; write instruction to command register
        call    read_word       ; read word into R2R3

; Note that could put to sleep here, then wake up 3/update rate before
; needed.  Lot of effort though and could screw up input impedance?
;
; The AD7705/6 only consumes 0.7mA worst case anyway; so just return!

        retlw   0



; ===================
; Calibrates channels
; ===================
;
; Do channel 1 first, then channel 0.  Need to do this as want channel 0
; to be the selected channel when exit.


calibrate_ext_adc


; ----> Do channel 1

        movlw   CLOCK_REG + CHANNEL1
        movwf   R0
        call    write_byte
        movlw   UPDATE_25Hz
        movwf   R0
        call    write_byte              ; init clock register

        movlw   SETUP_REG + CHANNEL1    ; select setup register
        movwf   R0
        call    write_byte
        movlw   SELFCAL_MODE + TEMP_GAIN + UNIPOLAR + BUFFERED
        movwf   R0
        call    write_byte

wait_for_cal1
        clrwdt
        btfsc   porta,nADCREADY         ; set back to decent prescaler when
        goto    wait_for_cal1           ; return to main.


; ----> Do channel 0

        movlw   CLOCK_REG + CHANNEL0
        movwf   R0
        call    write_byte
        movlw   UPDATE_25Hz
        movwf   R0
        call    write_byte              ; init clock register

        movlw   SETUP_REG + CHANNEL0    ; select setup register
        movwf   R0
        call    write_byte
        movlw   SELFCAL_MODE + PRESSURE_GAIN + UNIPOLAR + BUFFERED
        movwf   R0
        call    write_byte

wait_for_cal0
        clrwdt
        btfsc   porta,nADCREADY         ; set back to decent prescaler when
        goto    wait_for_cal0           ; return to main.

        retlw   0


; ====================
; Set channel and gain
; ====================
;
; Called and returns in bank 0, corrupts R0 and R1
;
; Selects the channel (in R2) and gain (in R3 and pre-shifted format)
;
; Note when read the data word, also need to supply the channel for conversion!

set_ext_adc
        movlw   CHANNELMASK
        andwf   R2,w                    ; select channel and next access
        addlw   SETUP_REG               ; as setup register
        movwf   R0
        call    write_byte

        movlw   NORMAL_MODE + UNIPOLAR + BUFFERED
        addwf   R3,w                    ; select gain
        movwf   R0
        call    write_byte
        retlw   0


; ===================
; Reset ADC Interface
; ===================
; Called and returns in bank 0, corrupts R1
; Resets the single wire interface to the state where it's expecting
; a write operation.

reset_adc_interface

        setpage wait_10ms
        call    wait_10ms       ; wait for ADC's osc to startup
        call    wait_10ms
        setpage reset_adc_int0
reset_adc_int0

        retlw   0



; ==========
; Write Byte
; ==========
; Called and returns in bank 0
; Write R0 to adc, corrupts R1

write_byte

        bsf     portc,SCLK      ; clock high, and select ADC below ...
        bcf     portc,nADCCS    ; select adc

        movlw   d'8'
        movwf   R1              ; prepare to write byte

write_byte_loop
        rlf     R0,f            ; get next bit to send in carry
        bcf     portc,SCLK      ; clock low
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bcf     trisa,ADCDATA
        btfsc   status,c
        bsf     trisa,ADCDATA   ; set up nADCDATA accordingly
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        bsf     portc,SCLK      ; clock high
        decfsz  R1,f
        goto    write_byte_loop
        bsf     portc,nADCCS    ; deselect adc

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bsf     trisa,ADCDATA   ; put data back to default of input (ie. o/c)
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****

        retlw   0



; =========
; Read Word
; =========
; Called and returns in bank 0
; Reads 16 bit word into R2R3, corrupts R1

read_word

        bsf     portc,SCLK      ; clock high, and select ADC below ...
        nop                     ; pipeline
        bcf     portc,nADCCS    ; select adc
        clrf    R2              ; Clear word
        clrf    R3              ;

        movlw   d'8'
        movwf   R1
read_word_loop1
        bcf     status,c
        rlf     R2,f
        bcf     portc,SCLK      ; clock low
        nop                     ; wait for data to settle
        btfsc   porta,ADCDATA
        bsf     R2,0
        bsf     portc,SCLK      ; clock high
        decfsz  R1,f
        goto    read_word_loop1

        movlw   d'8'
        movwf   R1
read_word_loop2
        bcf     status,c
        rlf     R3,f
        bcf     portc,SCLK      ; clock low
        nop                     ; wait for data to settle
        btfsc   porta,ADCDATA
        bsf     R3,0
        bsf     portc,SCLK      ; clock high
        decfsz  R1,f
        goto    read_word_loop2

        bsf     portc,nADCCS    ; deselect adc

        retlw   0               ; done

; =========================================================================

        global  measure, reset_ext_adc, calibrate_ext_adc
        global  single_adc, read_ext_adc, set_ext_adc

        END
