; *************************************************************************
;                         Free Flight Computer
;
;                              MMI MODULE
;
; *************************************************************************
;
; Development Tool:     MPLAB v4.12.12
; File Creation:        20/01/00                RNH
; Last Update:          09/08/04                RNH
; Code Issue:           A                       RNH
;
        list        P=16F76                     ; PIC 16F76
        include "P16C76r.INC"                   ; r is my lower case version
        include "eealloc.inc"                   ; eeprom allocations
        include "macro.inc"                     ; my macros
        include "gpio.inc"                      ; hardware/configuration

        extern  R0, R1, R2, R3, R4, R5, R6, R7, R8, R9, FLAGS
        extern  INT_WORK, PCLATH_COPY, STATUS_COPY, WREG_COPY

        extern  KEY, KEY_TICKS, KEY_DIV, MMIFLAGS, SCREEN, CURSORPOS, SELECT
        extern  TIMER_SEC, TIMER_MIN, TIMER_HOUR, QFE_OFF, QNH_OFF, ALTITUDE
        extern  BARO_TICK, BARO_LEN, BARO_RATE, BAROP_L, BAROP_H
        extern  BARO_END_H, BARO_END_L

        extern  draw_screen, send_4bit_cmd, send_4bit_data
        extern  write_eeprom, write_eeprom_16, wait_10ms
        extern  terminate_baro


; ==========================================================================


_mmi    code

KEY_NONE        equ 0
KEY_PLUS        equ 1
KEY_MINUS       equ 2
KEY_SET         equ 3
KEY_RESET       equ 4
KEY_SCREEN      equ 5


; ==========================================================================
;                                Scan Keypad
; ==========================================================================

; Key codes as follows:
;
;   0  no key
;   1  +
;   2  -
;   3  set/timer
;   4  reset/timer
;   5  screen
; +8   = long press
;
; +128 = debouncing
; +64  = waiting for key to be released now
;
; MMIFLAGS.4 (sound/nosound) updated here as well



scan_keypad

; ---- Read in key, simple stylie

        movlw   0               ; read key being pressed now into R0

        bsf     portb,5
        bcf     portb,6         ; select column 0
        btfsc   portc,2         ; row 0
        movlw   1
        btfsc   portc,3         ; row 0
        movlw   3
        btfsc   portc,4         ; row 0
        movlw   5

        bcf     portb,5
        bsf     portb,6         ; select column 1
        btfsc   portc,2         ; row 0
        movlw   2
        btfsc   portc,3         ; row 0
        movlw   4

        movwf   R0


; ----- Scan sound / no sound while we're at it

        movf    portc,w
        movwf   R2              ; accessible in all banks
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bcf     MMIFLAGS,4
        btfsc   R2,4            ; row 0
        bsf     MMIFLAGS,4
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****


; ---- Finished scan, do something with it now

        bcf     portb,6         ; de-select columns

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****

        btfsc   KEY,7
        goto    debouncing              ; debouncing at present

        btfsc   KEY,6
        goto    waiting_for_release     ; waiting to release key

        clrf    KEY             ; clear any (now actioned) keypress
        movlw   1               ; setup rapid fire divider to 1 so
        movwf   KEY_DIV         ; first 'fire' gets actioned quickly if used
        movf    R0,w            ; new key in R0
        btfss   status,z
        goto    start_new_key   ; a key is being pressed
        goto    keypad_return   ; no key, so just return


; ---- Arrive here if new key
;

start_new_key
        movf    R0,w
        movwf   KEY             ; copy new key to KEY
        bsf     KEY,7           ; needs debouncing
        clrf    KEY_TICKS       ; clear debounce tick
        goto    keypad_return


; ---- Arrive here if debouncing

debouncing

; If ticks < short debounce return

        movf    KEY_TICKS,w
        addlw   d'256'-SHORT_DEBOUNCE
        btfss   status,c        ; carry set if ticks >= short_debounce
        goto    keypad_return


; If released then assign key according to key ticks, or not as case may be

        movf    R0,w
        btfsc   status,z        ; key being pressed?
        goto    key_released    ; nope


; If key different then start again

        movf    KEY,w
        andlw   b'01111111'     ; clear debounce flag in KEY copy
        subwf   R0,w
        btfss   status,z        ; zero set if same key still
        goto    start_new_key


; Still being pressed, check for long debounce

        movf    KEY_TICKS,w
        addlw   d'256'-LONG_DEBOUNCE
        btfss   status,c        ; carry set if ticks >= long_debounce
        goto    keypad_return


; Now exceed long debounce, so mark key as finished debounce but waiting
; to be released.

        bcf     KEY,7           ; not waiting for debounce now
        bsf     KEY,6           ; waiting for release
        bsf     KEY,3           ; long keypress
        goto    keypad_return


key_released

; If <= short_debounce then clear key else normal press
; Long keypress picked up by code above

        movf    KEY_TICKS,w
        addlw   d'255'-SHORT_DEBOUNCE
        btfss   status,c        ; carry set if ticks > short_debounce
        goto    debounce_fail
debounce_success
        bcf     KEY,7           ; no longer debouncing
        goto    keypad_return
debounce_fail
        clrf    KEY
        goto    keypad_return



; ---- Arrive here if key long debounce previously exceeded and
;      waiting for key to be released.  Just wait for no key then clear flag.

waiting_for_release

        movf    R0,w
        btfss   status,z
        goto    keypad_return

        bcf     KEY,6           ; no longer waiting for release
        goto    keypad_return



keypad_return
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        retlw   0



; ==========================================================================
;                                    MMI
; ==========================================================================

do_mmi

; First check if screen button pressed, if so - change screens!

;   0  no key
;   1  +
;   2  -
;   3  set/timer
;   4  reset/timer
;   5  screen
; +8   = long press
;
; +128 = debouncing
; +64  = waiting for key to be released now


        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    KEY,w
        btfsc   status,z
        goto    done_mmi        ; return if no keypress


        btfsc   KEY,7           ; debouncing?
        goto    done_mmi        ; return if yes

        btfss   KEY,6           ; waiting for key to release?
        goto    action_key      ; branch and do mmi stuff if no


; If key is waiting for release and is SCREEN key, then clear content (so don't
; action again) and call change screens.

        movf    KEY,w
        andlw   b'00000111'
        sublw   KEY_SCREEN
        btfss   status,z
        goto    check_special_case

        movlw   b'01001000'
        movwf   KEY
        goto    change_screen


; Arrive here if waiting for key release and not screen key
; Only allow through to mmi code in this special case:
;
;       KEY = 1 or 2 (+ or -)
;       SCREEN = 2
;       SELECT = 1 or 2 (QFE or QNH offset)

check_special_case

        movf    KEY,w
        andlw   b'00000111'
        addlw   d'255'-2        ; carry set if > 2
        btfsc   status,c
        goto    done_mmi        ; return if not these keys

        movf    SCREEN,w
        sublw   d'2'
        btfss   status,z
        goto    done_mmi        ; return if not screen 2

        movf    SELECT,w
        btfsc   status,z
        goto    done_mmi        ; return if nothing selected

        addlw   d'255'-2        ; carry set if > 2
        btfsc   status,c
        goto    done_mmi        ; return if nothing selected

        decfsz  KEY_DIV,f       ; decrement rapid fire slowdown divider
        goto    done_mmi        ; return if not at zero yet
        movlw   d'100'          ; set rapid fire divider for next time around
        movwf   KEY_DIV         ; before allowing special case to continue

action_key

; Now have a sensible keypress
; Screen key pressed?

        movf    KEY,w
        sublw   KEY_SCREEN
        btfsc   status,z
        goto    change_screen

; No, so do screen specific stuff ...

        movf    SCREEN,w
        btfsc   status,z
        goto    screen0_mmi
        addlw   d'256'-1
        btfsc   status,z
        goto    screen1_mmi
        addlw   d'256'-1
        btfsc   status,z
        goto    screen2_mmi
        goto    screen3_mmi


change_screen
        setpage store_new_settings
        bsf     FLAGS,6            ; update time remaining (if go into screen 3)
        btfsc   MMIFLAGS,7         ; do we need to store new settings in eeprom?
        call    store_new_settings ; yes
        setpage do_change

do_change
        clrf    SELECT          ; ready for next screen

        btfsc   KEY,3
        goto    long_screen     ; bit 3 set if was long key press

        movf    SCREEN,w
        addlw   d'255'-1        ; see if SCREEN was <= 1
        btfss   status,c        ; carry set if isn't
        goto    increment_set_1
increment_set_2
        incf    SCREEN,f
        movf    SCREEN,w
        sublw   d'4'            ; gone past end?
        btfss   status,z
        goto    done_screen     ; nope
        movlw   d'2'
        movwf   SCREEN
        goto    done_screen     ; wrap

increment_set_1
        movf    SCREEN,w
        sublw   1               ; screen = 1 - screen
        movwf   SCREEN
        goto    done_screen

long_screen
        movf    SCREEN,w
        addlw   d'255'-1        ; see if SCREEN was <= 1
        btfsc   status,c        ; carry clear if yes
        goto    go_set_1
go_set_2
        movlw   2
        movwf   SCREEN
        goto    done_screen
go_set_1
        clrf    SCREEN
        goto    done_screen

done_screen
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        setpage draw_screen
        call    draw_screen
        retlw   0



; ----- Screen 0 MMI
;
; Toggle QFE/QNH on + (1) or - (2) key

screen0_mmi
        movf    KEY,w
        sublw   3               ; select pressed?
        btfss   status,z
        goto    done_mmi

        bsf     FLAGS,3         ; explicit display update please

        movlw   b'00100000'
        bcf     status,c        ; clear carry so don't corrupt lower bits
        addwf   MMIFLAGS,f      ; increment through qfe/qnh/qne
        btfss   MMIFLAGS,6
        goto    done_mmi        ; bit 6 clear, so either 00 or 01
        btfss   MMIFLAGS,5      ; bit 5 clear, so now 10
        goto    done_mmi
        movlw   b'10011111'
        andwf   MMIFLAGS,f      ; wrap from 11 to 00
        goto    done_mmi



; ----- Screen 1 MMI
;
; Control timer; start and stop on set (3), clear on reset (4)


screen1_mmi
        movf    KEY,w
        sublw   d'3'
        btfsc   status,z
        goto    start_stop_timer

        movf    KEY,w
        sublw   d'4'
        btfsc   status,z
        goto    reset_timer
        goto    done_mmi


start_stop_timer
        movlw   b'00000100'
        xorwf   FLAGS,f

        btfsc   FLAGS,2                 ;  if timer is stopped, terminate barograph
        goto    immediate_baro_start

        setpage terminate_baro
        call    terminate_baro
        setpage done_mmi
        goto    done_mmi


immediate_baro_start

; Modify barograph tick to invoke a barograph write as soon as possible

        movf    BARO_RATE,w
        addlw   d'256'-1
        movwf   BARO_TICK       ; set barotick ready to start update next
        goto    done_mmi


reset_timer
        clrf    TIMER_SEC
        clrf    TIMER_MIN
        clrf    TIMER_HOUR
        bsf     FLAGS,3         ; explicit display update please
        goto    done_mmi



; ---- Screen 2 MMI
;
; Configuration, need to select and alter the following:
;       qfe offset      SELECT 1
;       qnh offset      SELECT 2
;       beep (up/down)  SELECT 3
;       tmr (auto ...)  SELECT 4
; using the following keys:
;       1  + adjust by 1 foot, if waiting release then rapid
;       2  - adjust by 1 foot, if waiting release then rapid
;       3  set/timer - select
;       4  reset/timer - zero qfe offset
;       5  screen
; and save the setting to eeprom when change screen, if something
; has changed.


screen2_mmi
        btfss   KEY,6           ; don't explicitely update screen if rapid fire
        bsf     FLAGS,3         ; update screen asap please

        movf    KEY,w
        andlw   b'00000111'

        sublw   KEY_RESET       ; are we simply resetting the qfe offset?
        btfsc   status,z
        goto    clear_qfeoff    ; yes
        sublw   KEY_RESET       ; restore check

        sublw   KEY_SET         ; select pressed?
        btfsc   status,z
        goto    screen2_select
        sublw   KEY_SET

        movf    SELECT,w
        btfsc   status,z
        goto    done_mmi        ; don't do anything else if nothing selected

        addlw   d'256'-1        ; subtract 1
        btfsc   status,z
        goto    alter_qfeoff

        addlw   d'256'-1        ; subtract 1
        btfsc   status,z
        goto    alter_qnhoff

        movf    KEY,w
        addlw   d'255'-2        ; carry will be set if key > 2
        btfsc   status,c
        goto    done_mmi        ; not a valid key for us

        bsf     MMIFLAGS,7      ; will need to store whatever changes

        movf    SELECT,w
        addlw   d'256'-3        ; beep?
        btfsc   status,z
        goto    toggle_beep

        goto    toggle_timer    ; must be timer then

toggle_beep
        movf    KEY,w
        addlw   d'256'-1
        btfsc   status,z
        goto    inc_beep
dec_beep
        movf    MMIFLAGS,w
        andlw   b'00000011'
        addlw   d'256'-1
        btfsc   status,c
        addlw   d'4'            ; wrap
        movwf   R0
        goto    store_beep
inc_beep
        movf    MMIFLAGS,w
        andlw   b'00000011'
        addlw   1
        movwf   R0
        btfsc   R0,2
        clrf    R0              ; wrap R0 if incremented to 4
store_beep
        movlw   b'11111100'
        andwf   MMIFLAGS,f
        movf    R0,w
        iorwf   MMIFLAGS,f
        goto    done_mmi

toggle_timer
        movlw   b'00000100'
        xorwf   MMIFLAGS,f
        goto    done_mmi

alter_qfeoff
        bsf     FLAGS,4         ; partial display update of qfe will be required
        movf    KEY,w
        andlw   b'00000111'     ; want rapid-fire, no waiting for key!
        sublw   KEY_PLUS
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        btfss   status,z
        goto    do_qfeminus
        incf    QFE_OFF+1,f
        movf    QFE_OFF+1,w
        btfsc   status,z
        incf    QFE_OFF,f
        goto    done_mmi
do_qfeminus
        decf    QFE_OFF+1,f
        movf    QFE_OFF+1,w
        sublw   0xff
        btfsc   status,z
        decf    QFE_OFF,f
        goto    done_mmi

alter_qnhoff
        bsf     FLAGS,5         ; partial display update of qnh will be required
        movf    KEY,w
        andlw   b'00000111'     ; want rapid-fire, no waiting for key!
        sublw   KEY_PLUS
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        btfss   status,z
        goto    do_qnhminus
        incf    QNH_OFF+1,f
        movf    QNH_OFF+1,w
        btfsc   status,z
        incf    QNH_OFF,f
        goto    done_mmi
do_qnhminus
        decf    QNH_OFF+1,f
        movf    QNH_OFF+1,w
        sublw   0xff
        btfsc   status,z
        decf    QNH_OFF,f
        goto    done_mmi

clear_qfeoff
        bsf     MMIFLAGS,7      ; update sometime please
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        movf    ALTITUDE,w
        movwf   QFE_OFF
        movf    ALTITUDE+1,w
        movwf   QFE_OFF+1
        comf    QFE_OFF,f
        comf    QFE_OFF+1,f
        incf    QFE_OFF+1,f
        movf    QFE_OFF+1,w
        btfsc   status,z
        incf    QFE_OFF,f       ; QFE_OFF = -ALTITUDE
        goto    done_mmi

screen2_select
        incf    SELECT,w        ; increment select from 1 to 4 and wrap
        sublw   d'5'            ; w = 5 - SELECT
        btfsc   status,z
        movlw   d'4'
        sublw   d'5'            ; w = 5 - (5 - SELECT) or 5 - 4
        movwf   SELECT

        setpage select_lookup
        decf    SELECT,w        ; adjust to lookup cursor position
        call    select_lookup
        movwf   R2              ; store in R2
        setpage display_cursor
        goto    display_cursor



; ---- Screen 3 MMI
;
; barorate (SELECT 1), replace stored Yes (SELECT 2) / No (SELECT 3)
; add 6 to SELECT then decrement for jump table

screen3_mmi
        movf    KEY,w
        addlw   d'255'-3        ; carry set if > 3
        btfsc   status,c
        goto    done_mmi

        addlw   1
        btfsc   status,z
        goto    screen3_select

        movf    SELECT,w
        btfsc   status,z
        goto    done_mmi        ; nothing selected, so nothing to do
       
        addlw   d'256'-1
        btfsc   status,z
        goto    alter_barorate

        clrf    SELECT          ; plus / minus pressed, and on "yes" or "no"
                                ; so need to reset select as job done
        addlw   d'256'-1
        btfss   status,z        ; skip if on "yes"
        goto    hide_cursor     ; else on "no" so job done

; First stop timer, and hence writing to current barograph.

finish_current_barograph
        bcf     FLAGS,2         ; stop timer
        setpage terminate_baro
        call    terminate_baro
        setpage swap_barograph
        bsf     status,rp0      ; select bank 1   <--- **** NOTE ****

; Reset barograph to use other bank (code from main.asm)

swap_barograph
        bsf     MMIFLAGS,7       ; mark as eeprom update required
        btfss   FLAGS,2          ;  if timer is running, stop it and
        goto    change_barograph ;  reset barograph ...
        bcf     FLAGS,2
        setpage terminate_baro
        call    terminate_baro
        setpage change_barograph
        bsf     status,rp0      ; select bank 1   <--- **** NOTE ****

change_barograph
        movlw   b'00001000'
        xorwf   MMIFLAGS,f      ; toggle barograph store
        btfsc   MMIFLAGS,3      ; which barograph store are we using?
        goto    resetforbaro2

resetforbaro1
        movlw   EE_BARO1_H
        movwf   BAROP_H
        movlw   EE_BARO1_L
        movwf   BAROP_L
        movlw   EE_BARO1_ENDH
        movwf   BARO_END_H
        movlw   EE_BARO1_ENDL
        movwf   BARO_END_L
        goto    reset_ee_barop

resetforbaro2
        movlw   EE_BARO2_H
        movwf   BAROP_H
        movlw   EE_BARO2_L
        movwf   BAROP_L
        movlw   EE_BARO2_ENDH
        movwf   BARO_END_H
        movlw   EE_BARO2_ENDL
        movwf   BARO_END_L

reset_ee_barop                  ; reset barograph pointer in eeprom NOW
        movlw   EE_MMIFLAGS_H
        movwf   R1
        movlw   EE_MMIFLAGS_L
        movwf   R0
        movf    MMIFLAGS,w
        movwf   R8
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        setpage write_eeprom
        call    write_eeprom

        setpage wait_10ms
        call    wait_10ms       ; wait for it to write
        clrwdt

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   EE_BAROP_H
        movwf   R1
        movlw   EE_BAROP_L
        movwf   R0
        movf    BAROP_H,w
        movwf   R8
        movf    BAROP_L,w
        movwf   R9
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        setpage write_eeprom_16
        call    write_eeprom_16

        setpage wait_10ms
        call    wait_10ms       ; wait for it to write
        clrwdt

        setpage hide_cursor
        goto    hide_cursor


alter_barorate
        btfss   FLAGS,2         ;  if timer is running, stop it and
        goto    change_rate     ;  reset barograph ...
        bcf     FLAGS,2
        setpage terminate_baro
        call    terminate_baro
        setpage change_rate
        bsf     status,rp0      ; select bank 1   <--- **** NOTE ****

change_rate
        bsf     FLAGS,6         ; recalculate barograph time remaining
        bsf     FLAGS,3         ; explicit display update please
        movf    KEY,w
        sublw   KEY_PLUS
        btfsc   status,z
        incf    BARO_RATE,f

        movf    KEY,w
        sublw   KEY_MINUS
        btfsc   status,z
        decf    BARO_RATE,f

        movf    BARO_RATE,w
        addlw   d'256'-1
        btfsc   status,c        ; carry set if >= 1
        goto    done_mmi        ; done if baro rate > 1 second

        movlw   d'1'
        movwf   BARO_RATE
        goto    done_mmi        ; else reset to minimum of 1 second

screen3_select
        incf    SELECT,w        ; increment select from 1 to 3 and wrap
        sublw   d'4'            ; w = 4 - SELECT
        btfsc   status,z
        movlw   d'3'
        sublw   d'4'            ; w = 4 - (4 - SELECT) or 4 - 3
        movwf   SELECT

        setpage select_lookup
        movlw   d'3'
        addwf   SELECT,w        ; adjust to lookup cursor position
        call    select_lookup
        movwf   R2              ; store in R2
        setpage display_cursor
        goto    display_cursor


; ----- Move and display cursor

display_cursor
        bsf     FLAGS,3         ; explicit display update please
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    CURSORPOS,w
        sublw   0xff
        btfsc   status,z
        goto    newcursor       ; don't blank old is cursor fresh on screen

        setpage send_4bit_cmd
        movf    CURSORPOS,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        call    send_4bit_cmd   ; move to position
        movlw   ':'             ; blank old cursor
        call    send_4bit_data  ;

newcursor
        setpage send_4bit_cmd
        movf    R2,w
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movwf   CURSORPOS
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        call    send_4bit_cmd   ; move to position
        movlw   '='             ; Display our cursor
        call    send_4bit_data  ;
        setpage done_mmi
        goto    done_mmi



; ----- Hide cursor

hide_cursor
        bsf     FLAGS,3         ; explicit display update please
        setpage send_4bit_cmd
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    CURSORPOS,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        call    send_4bit_cmd   ; move to position
        movlw   ':'             ; Blank cursor
        call    send_4bit_data  ;
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   0xff
        movwf   CURSORPOS
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        setpage done_mmi
        goto    done_mmi



; ----- Store updated settings in eeprom, ie:
;
;       MMIFLAGS, QFE_OFF,QNH_OFF,BARO_RATE
;       Arrive here in bank 1, need to return in bank 1.

store_new_settings
        bcf     MMIFLAGS,7      ; clear update flag

        movlw   EE_MMIFLAGS_H
        movwf   R1
        movlw   EE_MMIFLAGS_L
        movwf   R0
        movf    MMIFLAGS,w
        movwf   R8              ; store in R8 for write_eeprom routine

        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        setpage write_eeprom
        call    write_eeprom

        setpage wait_10ms
        call    wait_10ms       ; wait for it to write
        clrwdt

        movlw   EE_QFEOFF_H
        movwf   R1
        movlw   EE_QFEOFF_L
        movwf   R0
        movf    QFE_OFF,w
        movwf   R8              ; store in R8R9 for write_eeprom_16 routine
        movf    QFE_OFF+1,w
        movwf   R9
        setpage write_eeprom_16
        call    write_eeprom_16

        setpage wait_10ms
        call    wait_10ms       ; wait word to write
        call    wait_10ms
        clrwdt

        movlw   EE_QNHOFF_H
        movwf   R1
        movlw   EE_QNHOFF_L
        movwf   R0
        movf    QNH_OFF,w
        movwf   R8              ; store in R8R9 for write_eeprom_16 routine
        movf    QNH_OFF+1,w
        movwf   R9
        setpage write_eeprom_16
        call    write_eeprom_16

        setpage wait_10ms
        call    wait_10ms       ; wait word to write
        call    wait_10ms
        clrwdt

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movlw   EE_BARORATE_H
        movwf   R1
        movlw   EE_BARORATE_L
        movwf   R0
        movf    BARO_RATE,w
        movwf   R8              ; store in R8 for write_eeprom routine
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        setpage write_eeprom
        call    write_eeprom

        setpage wait_10ms
        call    wait_10ms       ; wait for it to write
        clrwdt

        setpage done_settings
done_settings
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        retlw   0



; ---- Done MMI - return to bank 0 and return

done_mmi
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        retlw   0


; ==========================================================================

_sel_lookup     code

; Following table returns cursor position to show what selected

select_lookup
        addwf   pcl,f

; screen 2 ...
        retlw   LINE1+d'3'      ; qfe offset
        retlw   LINE2+d'3'      ; qnh offset
        retlw   LINE1+d'14'     ; beep
        retlw   LINE2+d'14'     ; tmr

; screen 3 ...
        retlw   LINE1+d'4'      ; barorate
        retlw   LINE4+d'0'      ; Yes
        retlw   LINE4+d'10'     ; No


; ==========================================================================


        global  scan_keypad, do_mmi

        END

