; *************************************************************************
;                         Free Flight Computer
;
;                              UART MODULE
;
; *************************************************************************
;
; Development Tool:     MPLAB v4.12.12
; File Creation:        20/01/00                RNH
; Last Update:          22/04/04                RNH
; Code Issue:           A                       RNH
;
        list        P=16F76                     ; PIC 16F76
        include "P16C76r.INC"                   ; r is my lower case version
        include "eealloc.inc"                   ; eeprom allocations
        include "macro.inc"                     ; my macros
        include "gpio.inc"                      ; hardware/configuration

        extern  R0, R1, R2, R3, R4, R5, R6, R7, R8, R9, R10, R11, R12
        extern  R13, R14, R15, R16
        extern  ALT_M, ALT_C, ASI_M
        extern  RX_BUFFER, RX_PTR, RX_TMP, RX_END
        extern  TX_BUFFER, TX_PTR, TX_END, BARO_END_H, BARO_END_L

        extern  read_eeprom, write_eeprom
        extern  single_adc, read_ext_adc, set_ext_adc
        extern  read_eeprom_16, write_eeprom_16
        extern  read_eeprom_24, write_eeprom_24
        extern  wait_10ms, digitise

; ==========================================================================

; Transmission and reception done automatically in interrupt routines.  This
; module interprets engineering commands and actions them.
;
; Has to process and action the following commands:
;
; 0     TBD             spare command, unused as yet
; 1     RD xx yy        where xx and yy is EEPROM location
; 2     WR xx yy zz     writes zz to xx yy
; 3     SEND x y        where x is barograph 1 or 2 and y is trace from 0 on ...
; 4     DUMP            dump entire ram contents to UART
; 5     RESET           resets FFC
; 6     ADC x y         Read ADC (x=0 internal, x=1 external, y=channel)


_uart   code


; =========================================================================
;                               SERVICE UART
; =========================================================================


service_uart

        movf    RX_TMP,w
        subwf   RX_END,w
        btfsc   status,z
        retlw   0               ; return if RX_TMP = RX_END

check_char
        movf    RX_TMP,w        ; Point to character to check
        movwf   fsr             ; and increment ready for next check (below)

        incf    RX_TMP,w        ; increment RX_TMP to wreg
        sublw   RX_BUFFER+d'15' ; see if past end of buffer
        btfss   status,c        ; carry clear if is
        movlw   d'15'           ; set so wraps
        sublw   RX_BUFFER+d'15' ; restore check
        movwf   RX_TMP          ; circularly incremented

        movf    indf,w
        sublw   0d
        btfsc   status,z
        goto    process_cmd     ; if found LF then have command to check out
        movf    RX_TMP,w
        subwf   RX_END,w
        btfss   status,z
        goto    check_char      ; continue checking until done all so far
        retlw   0               ; return


; ----- Have found LF, so check from last RX_PTR to interpret command

process_cmd

; ----- Strip LFs

        movf    RX_PTR,w
        movwf   fsr
        movf    indf,w
        sublw   0d
        btfss   status,z
        goto    lookup_cmd      ; jump when stripped leading LFs from old

        incf    RX_PTR,w        ; increment RX_PTR to wreg
        sublw   RX_BUFFER+d'15' ; see if past end of buffer
        btfss   status,c        ; carry clear if is
        movlw   d'15'           ; set so wraps
        sublw   RX_BUFFER+d'15' ; restore check
        movwf   RX_PTR          ; circularly incremented

        subwf   RX_END,w
        btfss   status,z        ; skip if RX_END = RX_PTR
        goto    process_cmd     ; no, so continue stripping
        movf    RX_END,w
        movwf   RX_TMP          ; set RX_TMP = RX_END (= RX_PTR)
        retlw   0               ; nothing but LFs - return


; ----- Look up command
;
; This is painful in a PIC.  Have to call CMD_TABLE with pointer to
; character there to see if matches what have.  Very cumbersome.
;
; Routine uses a copy of RX_PTR in RX_TMP to compare rx buffer against
; each command.  When comes across zero have compared all of string so
; all is well and have found command.  If doesn't match then RX_TMP reset
; to RX_PTR and the table pointer advanced to the next command.  If 0ff
; (end of table) is found then rx buffer is flushed and we exit.


lookup_cmd
        clrf    R1              ; command number checking = 0
        movlw   0ff
        movwf   R0              ; command table pointer = -1
        movf    RX_PTR,w
        movwf   RX_TMP          ; update RX_TMP, used to dig here

lookup_loop
        incf    R0,f            ; increment table pointer and ...
        movf    RX_TMP,w        ;
        movwf   fsr             ; point indf to rx_character (used below)

        setpage cmd_table

        movf    R0,w            ; wreg = pointer
        call    cmd_table       ; returns with wreg = char at that position
        andlw   0ff             ; zero flag not affected by above, so do now
        btfss   status,z        ; see if is zero
        goto    chk_cmd_end     ; no - check if end of table
        movf    RX_TMP,w
        movwf   RX_PTR          ; if it is then have found it.
        goto    found_cmd       ; update RX_PTR (to next char) before go on

chk_cmd_end
        addlw   d'256'-d'255'   ; see if is end of table  (subtract 0ff)
        btfss   status,z
        goto    compare_char    ; no - must be a character then
        movf    RX_END,w
        movwf   RX_TMP
        movwf   RX_PTR          ; flush buffer and exit ...
        retlw   0               ; ... as haven't found command yet then
                                ;     probably all a load of rubbish
compare_char
        addlw   0ff             ; add back subtraction to restore character
        subwf   indf,w          ; compare to that in rx buffer
        btfsc   status,z
        goto    ok_so_far       ; if does then go to increment & continue
        movf    RX_PTR,w
        movwf   RX_TMP          ; else reset temp rx ptr to what are finding
next_cmd                        ; and sweep table for next command
        incf    R0,f

        setpage cmd_table

        movf    R0,w            ; wreg = pointer
        call    cmd_table       ; returns with wreg = char at that position
        andlw   0ff             ; test to see if zero (above does not set z)
        btfss   status,z
        goto    next_cmd        ; keep looping until wreg = 0 (end of cmd)
        incf    R1,f            ; update command number
        goto    lookup_loop     ; go check new command

ok_so_far                       ; circularly increment RX_TMP
        incf    RX_TMP,w        ; increment RX_TMP to wreg
        sublw   RX_BUFFER+d'15' ; see if past end of buffer
        btfss   status,c        ; carry clear if is
        movlw   d'15'           ; set so wraps
        sublw   RX_BUFFER+d'15' ; restore check
        movwf   RX_TMP          ; circularly incremented
        goto    lookup_loop     ; and go to check next character


; =========================================================================
;                         ENGINEERING COMMANDS
; =========================================================================

; ----- Have found a command, jump to appropriate routine.  RX_PTR and
;       RX_TMP point to char immediately after command name
;
; Echo a CRLF as we beat the proper echoing to the LF part of the
; incoming CR.  I think.

found_cmd
        call    send_crlf       ; send CRLF
        setpage cmd_jmp
        movf    R1,w            ; get command
        goto    cmd_jmp         ; vector to command


;----- Command 0        TBD
;
; Spare command, not used at present

cmd0
        goto    error_flush     ; return, flush buffer and say error!



;----- Command 1        RD xxx yyy
;
; Reads the byte from EEPROM location xxx yyy (decimal bytes)

cmd1
        call    readnum         ; xxx
        movf    R11,w
        sublw   2               ; expect to find a space after number
        btfss   status,z
        goto    error_flush     ; return if not, flush buffer and say error!
        movf    R12,w
        movwf   R1              ; save xxx in R1

        call    readnum         ; yyy
        movf    R11,w
        sublw   3               ; Expect error 3 (LF found)
        btfss   status,z
        goto    error_flush     ; return if not, flush buffer and say error!
        movf    R12,w
        movwf   R0              ; save yyy in R0

        setpage wait_10ms
        call    wait_10ms       ; ensure any writes have completed

        setpage read_eeprom
        call    read_eeprom
        setpage cmd1b
cmd1b

        btfss   status,c
        goto    eng_fail

        call    calc_hex        ; get hex characters for R8 in R0, R1

        movf    R0,w
        call    send_char       ; send upper nibble
        movf    R1,w
        call    send_char       ; send lower nibble
        call    send_crlf       ; send crlf
        call    initiate        ; ensure is still going
        goto    eng_done


;----- Command 2        WR xxx yyy zzz
;
; Writes zzz to xxx yyy (decimal bytes)

cmd2
        call    readnum         ; xxx
        movf    R11,w
        sublw   2               ; expect to find a space after number
        btfss   status,z
        goto    error_flush     ; return if not, flush buffer and say error!
        movf    R12,w
        movwf   R1              ; save xxx in R1

        call    readnum         ; yy
        movf    R11,w
        sublw   2               ; expect to find a space after number
        btfss   status,z
        goto    error_flush     ; return if not, flush buffer and say error!
        movf    R12,w
        movwf   R0              ; save yy in R0

        call    readnum         ; zz
        movf    R11,w
        sublw   3               ; Expect error 3 (LF found)
        btfss   status,z
        goto    error_flush     ; return if not, flush buffer and say error!
        movf    R12,w
        movwf   R8              ; save zz in R8

        setpage wait_10ms
        call    wait_10ms       ; ensure any writes have completed

        setpage write_eeprom
        call    write_eeprom    ; Write R8 to address R1R0
        setpage cmd2b
cmd2b

        btfss   status,c
        goto    eng_failw
        goto    eng_done



;----- Command 3        SEND xx yy
;
; Sends trace yy from barograph store xx to terminal.
; Takes a while so will screw up timing!

cmd3

        call    readnum         ; xx
        movf    R11,w
        sublw   2               ; expect to find a space after number
        btfss   status,z
        goto    error_flush     ; return if not, flush buffer and say error!
        movf    R12,w
        movwf   R1              ; save xx in R1

        call    readnum         ; yy
        movf    R11,w
        sublw   3               ; Expect error 3 (LF found)
        btfss   status,z
        goto    error_flush     ; return if not, flush buffer and say error!
        movf    R12,w           ;
        movwf   R7              ; save yy in R7

; If R1 is 1, then read bytes from EE_BARO1_H/L to EE_BARO1_ENDH/L
; If R1 is 2, then read bytes from EE_BARO2_H/L to EE_BARO2_ENDH/L
; Otherwise error.

        movf    R1,w
        btfsc   status,z
        goto    eng_fail        ; fail if asked for barograph 0

        addlw   d'256'-1
        btfss   status,z
        goto    cmd3a           ; branch if barograph 2
        movlw   EE_BARO1_H
        movwf   R1
        movlw   EE_BARO1_L
        movwf   R0
        goto    cmd3b           ; location of barograph 1 loaded

cmd3a
        addlw   d'256'-1
        btfss   status,z
        goto    eng_fail        ; fail if not barograph 2!
        movlw   EE_BARO2_H
        movwf   R1
        movlw   EE_BARO2_L
        movwf   R0


cmd3b

; Now search through EEPROM until find trace yy requested.  If reach
; end and still nothing then error!

        movf    R7,w
        btfsc   status,z
        goto    eng_fail        ; fail if asked for trace 0

; Pointer is held in R1R0
; End of buffer is held in BARO_END_H and BARO_END_L

search_for_trace
        decf    R7,f
        btfsc   status,z
        goto    send_trace

scan_next
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        clrwdt
        movlw   d'4'
        movwf   R6              ; looking for 4 bytes of 0x FF FF FF FF (then barorate)

scan_ff
        setpage read_eeprom
        call    read_eeprom     ; Read byte from R1R0 into R8 (takes 0.5ms)
        setpage scan_byte

scan_byte
        incf    R0,f
        btfsc   status,z
        incf    R1,f            ; Increment pointer

        movf    R8,w
        sublw   0xff
        btfss   status,z
        goto    scan_fail       ; not an end of buffer marker

        decfsz  R6,f              ; decrement number of bytes left to check
        goto    scan_ff           ; keep checking for all 4 bytes

        incf    R0,f              ; found end marker and pointing to barorate:
        btfsc   status,z          ;    0x FF FF FF FF (barorate)
        incf    R1,f              ; so now increment past this and jump to
        goto    search_for_trace  ; see if at required trace.


scan_fail
        movf    R6,w
        decf    R6,f
        addwf   R0,f
        btfsc   status,c
        incf    R1,f            ; move to next 5 byte location

        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R1,w
        subwf   BARO_END_H,w
        btfss   status,z
        goto    scan_next       ; not end of buffer, try next location
        movf    R0,w
        subwf   BARO_END_L,w
        btfss   status,z
        goto    scan_next       ; not end of buffer, try next location
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        goto    eng_fail        ; reached end if R1R0 = BARO_END_H,BARO_END_L



; Now can start reading from EEPROM and sending out to UART until reach
; next four bytes of 0xff.  Output in comma separated format.
;
; Barograph format is QNH_MSB, QNH_LSB, VARIO, TEMPERATURE, TAS

send_trace
        clrwdt
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R1,w
        subwf   BARO_END_H,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        btfss   status,z
        goto    read_barograph_entry
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        movf    R0,w
        subwf   BARO_END_L,w
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        btfsc   status,z
        goto    eng_done        ; end of trace so exit, echoing "Done." to terminal


read_barograph_entry
        setpage read_eeprom_16  ; alters R2-R5, R17
        call    read_eeprom_16  ; reads word into R8R9
        movf    R8,w
        movwf   R10
        movf    R9,w
        movwf   R11             ; QNH_MSB and QNH_LSB in R10R11
        movlw   2
        addwf   R0,f
        btfsc   status,c
        incf    R1,f            ; increment R1R0 a word

        call    read_eeprom     ; reads byte into R8; alters R2-R5, R17
        movf    R8,w
        movwf   R12             ; VARIO in R12
        incf    R0,f
        btfsc   status,z
        incf    R1,f            ; increment R1R0 a byte

        call    read_eeprom
        movf    R8,w
        movwf   R13             ; TEMPERATURE in R13
        incf    R0,f
        btfsc   status,z
        incf    R1,f

        call    read_eeprom
        movf    R8,w
        movwf   R14             ; TAS in R14
        incf    R0,f
        btfsc   status,z
        incf    R1,f

        setpage check_eot

check_eot
        movf    R10,w
        sublw   0xff
        btfss   status,z
        goto    send_trace_entry
        movf    R11,w
        sublw   0xff
        btfss   status,z
        goto    send_trace_entry
        movf    R12,w
        sublw   0xff
        btfss   status,z
        goto    send_trace_entry
        movf    R13,w
        sublw   0xff
        btfss   status,z
        goto    send_trace_entry


; We have reached 0xffffffffxx so end of trace!  Send the barorate which is stored
; as the last byte of the end marker.

send_barorate
        movf    R14,w
        movwf   R0
        clrf    R1
        call    send_word
        call    send_crlf       ; alters R0
        goto    eng_done        ; exit, echoing "Done." to terminal


send_trace_entry

; Save R1R0 trace position
        movf    R1,w
        movwf   R15
        movf    R0,w
        movwf   R16             ; Copy R1R0 into R15R16


; Send QNH from R10R11
        btfss   R10,7
        goto    send_qnh
        movlw   '-'
        call    send_char
        comf    R10,f
        comf    R11,f
        incf    R11,f
        btfsc   status,z        ; 2s complement on R10R11 to
        incf    R10,f           ; make positive
send_qnh
        movf    R10,w
        movwf   R1
        movf    R11,w
        movwf   R0
        call    send_word       ; alters R0-R6,R8
        movlw   ','
        call    send_char       ; alters nothing
        movlw   ' '
        call    send_char       ; alters nothing


; Send VARIO from R12
        btfss   R12,7           ; make positive if negative
        goto    done_vario_sign
        comf    R12,f
        incf    R12,f
        movlw   '-'
        call    send_char
done_vario_sign
        clrf    R1
        bcf     status,c
        rlf     R12,w
        movwf   R0
        btfsc   status,c
        incf    R1,f            ; double so instantly get in format required!
        pagesel digitise
        call    digitise
        pagesel got_vario_digits
got_vario_digits
        movf    R4,w
        btfsc   status,z
        goto    done_vario_tens
        addlw   '0'
        call    send_char
done_vario_tens
        movf    R3,w
        addlw   '0'
        call    send_char
        movlw   '.'
        call    send_char
        movf    R2,w
        addlw   '0'
        call    send_char
        movlw   ','
        call    send_char
        movlw   ' '
        call    send_char


; Now send buffer (maximum 16 byte queue)
        call    initiate        ; alters nothing.  ensure is still going
wait_for_send_1
        clrwdt                  ; ensure don't timeout
        movf    TX_END,w
        subwf   TX_PTR,w
        btfss   status,z
        goto    wait_for_send_1 ; wait till sent


; Send TEMPERATURE from R13

        movf    R13,w
        addlw   d'256'-d'50'            ; subtract 10degreesC (50 decimal)
        btfsc   status,c
        goto    temp_positive           ; carry set if positive
        movlw   '-'
        call    send_char
        movf    R13,w                   ; wreg 0 = -10, 5 = -9 ...
        sublw   d'50'                   ; wreg = 50 - wreg so 50 = -10, 45 = -9 ...

temp_positive
        movwf   R0                      ; copy R13 into R0
        clrf    R1                      ; clear R1
        bcf     status,c
        rlf     R0,f
        rlf     R1,f                    ; double R1R0 so is now temperature x 10
        pagesel digitise
        call    digitise                ; get 100s, 10s and 1s in R4 R3 R2
        pagesel got_temp_digits
got_temp_digits
        movf    R4,w
        btfsc   status,z
        goto    done_temp_tens
        addlw   '0'
        call    send_char
done_temp_tens
        movf    R3,w
        addlw   '0'
        call    send_char
        movlw   '.'
        call    send_char
        movf    R2,w
        addlw   '0'
        call    send_char
        movlw   ','
        call    send_char
        movlw   ' '
        call    send_char


; Send TAS from R14
        movf    R14,w
        movwf   R0
        clrf    R1
        call    send_word
        call    send_crlf       ; alters R0


; Now send remainder
        call    initiate        ; alters nothing.  ensure is still going
wait_for_send_2
        clrwdt                  ; ensure don't timeout
        movf    TX_END,w
        subwf   TX_PTR,w
        btfss   status,z
        goto    wait_for_send_2 ; wait till sent


; Restore R1R0 trace position
        movf    R15,w
        movwf   R1
        movf    R16,w
        movwf   R0              ; Copy R15R16 into R1R0

        goto    send_trace      ; go back for more



;----- Command 4        DUMP        Dumps RAM contents to engineering
;
; Takes 19200 baud * 512 bytes to complete so will screw up any
; timing in progress.  Only intended for a one-shot so not bothered
; however.

cmd4
        call    readdigit       ; strip spaces and get character
        movf    R11,w           ;
        sublw   2               ; should read LF so get error 2
        btfss   status,z
        goto    error_flush     ; otherwise error

; Send banks 0/1

        clrf    R3              ; reset pointer to start
dump01
        movf    R3,w            ; restore R3
        movwf   fsr             ; point to byte to read
        movf    indf,w          ; read byte into wreg
        movwf   R8              ; store in R8 for ...
        call    dumpchar        ; send it, R3 gets incremented in here
        movf    R3,w
        btfss   status,z        ; fsr zero when finished page else
        goto    dump01          ; continue until done all of page


; Send banks 2/3

        clrf    R3              ; reset pointer to start
dump23
        movf    R3,w            ; restore R3
        movwf   fsr             ; point to byte to read
        bsf     status,irp      ; ... to bank 2/3 for indirect    <--- **** NOTE ****
        movf    indf,w          ; read byte into wreg
        bcf     status,irp      ; ... to bank 0/1 for indirect    <--- **** NOTE ****
        movwf   R8              ; store in R8 for ...
        call    dumpchar        ; send it, R3 gets incremented in here
        movf    R3,w
        btfss   status,z        ; fsr zero when finished page else
        goto    dump23          ; continue until done all of page

; Finished

        goto    eng_done


; Dump character routine called from above

dumpchar
        call    calc_hex        ; get hex character in R0, R1
        movf    R0,w
        call    send_char       ; send upper nibble
        movf    R1,w
        call    send_char       ; send lower nibble
        movlw   ' '             ;
        call    send_char       ; send space
        call    initiate        ; ensure is still going
waitdump
        clrwdt                  ; ensure don't timeout
        movf    TX_END,w
        subwf   TX_PTR,w
        btfss   status,z
        goto    waitdump        ; wait till sent
        incf    R3,f            ; increment fsr
        movf    R3,w            ; copy into wreg
        andlw   b'00001111'     ; see if on even boundary
        btfss   status,z        ; skip if is
        retlw   0
        call    send_crlf       ; send crlf
        call    initiate        ; ensure is still going
        retlw   0               ;



; Byte supplied in R8, returns characters in R0R1 pair
; Corrupts R8

calc_hex
        movf    R8,w            ; do lower nibble first
        andlw   b'00001111'     ; mask out upper nibble
        addlw   d'256'-d'10'    ; subtract 10
        btfsc   status,c        ; carry set if >= 10
        addlw   d'55'-d'48'     ; in which case count from "A"-10
        addlw   d'10'+d'48'     ; restore, and add "0" for 0 ...
        movwf   R1              ; into R1

        rrf     R8,f            ; shift character for upper nibble
        rrf     R8,f            ; shift again
        rrf     R8,f            ; shift and again
        rrf     R8,w            ; shift yet again, into wreg
        andlw   b'00001111'     ; and get rid of nasty bits
        addlw   d'256'-d'10'    ; subtract 10
        btfsc   status,c        ; carry set if >= 10
        addlw   d'55'-d'48'     ; in which case count from "A"-10
        addlw   d'10'+d'48'     ; restore, and add "0" for 0 ...
        movwf   R0              ; into R0

        retlw   0



;----- Command 5        RESET      Resets FFC
;
; Just waits for watchdog timeout!

cmd5
        call    readdigit       ; strip spaces and get character
        movf    R11,w           ;
        sublw   2               ; should read LF so get error 2
        btfss   status,z
        goto    error_flush     ; otherwise error
reset
        goto    reset           ; wait for watchdog to go off



;----- Command 6        ADC x y    Read ADC
;
; If x = 0 then reads pic ADC channel y else reads external adc channel y

cmd6
        call    readnum         ; x
        movf    R11,w
        sublw   2               ; expect to find a space after number
        btfss   status,z
        goto    error_flush     ; return if not, flush buffer and say error!
        movf    R12,w
        movwf   R1              ; save adc select in R1

        call    readnum         ; y
        movf    R11,w
        sublw   3               ; Expect error 3 (LF found)
        btfss   status,z
        goto    error_flush     ; return if not, flush buffer and say error!
        movf    R12,w
        movwf   R2              ; save channel in R2

        clrwdt
        movf    R1,w            ; 0 = internal adc, else external adc
        btfss   status,z
        goto    cmd6_external_adc


cmd6_internal_adc

        pagesel single_adc
        movf    R2,w
        call    single_adc      ; result in adres, screws up R0
        pagesel cmd6a
cmd6a
        movf    adres,w
cmd6b
        movwf   R8
        call    calc_hex        ; get hex characters for R8 in R0, R1

        movf    R0,w
        call    send_char       ; send upper nibble
        movf    R1,w
        call    send_char       ; send lower nibble
        call    send_crlf       ; send crlf
        call    initiate        ; ensure is still going
        goto    eng_done


cmd6_external_adc

; Need to do the following:
;
;
;       Select channel and gain for conversion
;
;       Wait for nDRDY
;       Ignore data returned
;
;       Wait for nDRDY
;       Select channel 0 for usual pressure adc conversion
;       Useful data for channel above returned in R2R3

        movlw   TEMP_GAIN
        movwf   R3              ; set gain to default of unity

        movf    R2,w
        subwf   PRESSURE_ADC,w
        btfss   status,z
        goto    have_gain
        movlw   PRESSURE_GAIN
        movwf   R3

have_gain
        clrwdt
        pagesel set_ext_adc     ; channel number in R2, gain in R3
        call    set_ext_adc     ; select it
        call    read_ext_adc    ; wait for first nDRDY, ignore data so settles
        movlw   PRESSURE_ADC
        movwf   R2
        movlw   PRESSURE_GAIN
        movwf   R3              ; select default adc setting to setup after next read
        call    read_ext_adc    ; wait for next one for a proper read
        pagesel cmd6c           ; returns in R2R3, alters R0, R1
cmd6c

        movf    R2,w
        movwf   R8
        call    calc_hex        ; get hex characters for R8 in R0, R1
        movf    R0,w
        call    send_char       ; send upper nibble
        movf    R1,w
        call    send_char       ; send lower nibble
        call    initiate        ; ensure is still going

        movf    R3,w
        goto    cmd6b           ; steal code from above for lower byte




; =========================================================================
;                          UART SUPPORT ROUTINES
; =========================================================================

; =========================================================================
; ----- dec2word
;
; Converts R1 R2 into hex word.
; eg.  10 23 would be converted into 03FF
;     163 84 would be converted into 4000
;
; Inputs:       R1 R2
; Outputs:      R1R0
; Alters:        R2-R6

dec2word
        movf    R1,w
        movwf   R0
        clrf    R1              ; R1R0 = original R1 dibit

        bcf     status,c
        rlf     R0,f
        rlf     R1,f            ; x 2

        bcf     status,c
        rlf     R0,f
        rlf     R1,f            ; x 4

        movf    R1,w
        movwf   R4
        movf    R0,w
        movwf   R3              ; store in R4R3 for later use

        bcf     status,c
        rlf     R0,f
        rlf     R1,f            ; x 8

        bcf     status,c
        rlf     R0,f
        rlf     R1,f            ; x 16

        bcf     status,c
        rlf     R0,f
        rlf     R1,f            ; x 32

        movf    R1,w
        movwf   R6
        movf    R0,w
        movwf   R5              ; store in R6R5 for later use

        bcf     status,c
        rlf     R0,f
        rlf     R1,f            ; x 64

        bcf     status,c
        movf    R5,w
        addwf   R0,f
        btfsc   status,c
        incf    R1,f
        movf    R6,w
        addwf   R1,f            ; R1R0 = x96

        bcf     status,c
        movf    R3,w
        addwf   R0,f
        btfsc   status,c
        incf    R1,f
        movf    R4,w
        addwf   R1,f            ; R1R0 = hundreds x100

        bcf     status,c
        movf    R2,w
        addwf   R0,f
        btfsc   status,c
        incf    R1,f            ; R1R0 = 16 bit number of that entered

        retlw   0


; =========================================================================
; ----- readnum
;
; Strips spaces and read up to 3 digit (255) number.
;
; Returns R11 = 1 on unexpected number or decoding error
;
; Decoded number stored in R12 and will return one of the following:
;
; Returns R11 = 2 on space  (ie for information)
; Returns R11 = 3 on LF     (ie for information)
;
; Corrupts R3

readnum
        movlw   0ff
        movwf   R11             ; Init error to non-value
        clrf    R12             ; Clear decoded number

; ----- Strip leading spaces

rnum_strip
        movf    RX_PTR,w        ; RX_PTR points to first char to check
        subwf   RX_END,w        ; reached end of characters?
        btfsc   status,z
        goto    rnum_error      ; branch if yes

        movf    RX_PTR,w
        movwf   fsr             ; set fsr to char
        movf    indf,w          ; get it
        sublw   ' '
        btfss   status,z        ; skip if is space
        goto    rnum_decode     ; stripped spaces

        incf    RX_PTR,w        ; increment RX_PTR to wreg
        sublw   RX_BUFFER+d'15' ; see if past end of buffer
        btfss   status,c        ; carry clear if is
        movlw   d'15'           ; set so wraps
        sublw   RX_BUFFER+d'15' ; restore check
        movwf   RX_PTR          ; circularly incremented
        movwf   fsr             ; and copy to fsr for next check

        goto    rnum_strip      ; loop for more


; ----- Decode and check for valid digits

rnum_decode
        movf    RX_PTR,w        ; set up pointer and circularly inc RX_PTR
        movwf   fsr             ; for next time

        incf    RX_PTR,w        ; increment RX_PTR to wreg
        sublw   RX_BUFFER+d'15' ; see if past end of buffer
        btfss   status,c        ; carry clear if is
        movlw   d'15'           ; set so wraps
        sublw   RX_BUFFER+d'15' ; restore check
        movwf   RX_PTR          ; circularly incremented

        movf    indf,w          ; get char

        addlw   d'256'-'0'      ; subtract '0', if overflows (ie >= '0')
        btfss   status,c        ; carry will be set
        goto    rnum_done

        addlw   d'256'-d'10'    ; subtract 9, if overflows (ie >= 10)
        btfsc   status,c        ; carry will be set
        goto    rnum_done       ; if non-digit then could be error or end

        addlw   d'10'           ; restore last check
        movwf   R3              ; Use common register R3

; ----- Do R12 = R12 * 10 + digit

        clrf    R11             ; set as now decoding something
        bcf     status,c        ; clear carry ready for rotates ...

        rlf     R12,f           ; number * 2
        btfsc   status,c
        goto    rnum_error      ; abort on overflow
        movf    R12,w           ; get copy of R12*2 in wreg
        rlf     R12,f           ; R12 * 4
        btfsc   status,c
        goto    rnum_error      ; abort on overflow
        rlf     R12,f           ; R12 * 8
        btfsc   status,c
        goto    rnum_error      ; abort on overflow
        addwf   R12,w           ; wreg = R12 * 10
        btfsc   status,c
        goto    rnum_error      ; abort on overflow
        addwf   R3,w            ; wreg = R12 * 10 + digit
        btfsc   status,c
        goto    rnum_error      ; abort on overflow
        movwf   R12             ; R12 = R12 * 10 + digit

        goto    rnum_decode     ; done - go back for more

; ----- Exit points from routine

rnum_done
        movf    indf,w
        addlw   d'256'-0d       ; see if LF
        btfss   status,z
        goto    rnum_space      ; see if space if not

        movf    R11,w
        sublw   0ff
        btfsc   status,z
        goto    rnum_error      ; if never did any decoding - then error
        movlw   3
        movwf   R11             ; else set LF found after decoding
        goto    rnum_exit

rnum_space
        addlw   d'256'-' '+0d   ; see if space
        btfss   status,z
        goto    rnum_error      ; definitely error if not
        movlw   2
        movwf   R11             ; else set space found after decoding
        goto    rnum_exit

rnum_error
        movlw   1
        movwf   R11             ; set error

rnum_exit
        movf    RX_PTR,w        ;
        movwf   RX_TMP          ; update RX_TMP
        retlw   0               ; exit


; =========================================================================
; ----- readdigit
;
; Strips spaces and reads a single digit.
;
; Returns R11 = 1 on unexpected number
; Returns R11 = 2 on LF found and not digit
; On valid digit, numerical value stored in R12 and R11 = 0
;
; No need to check if end of buffer as know have got LF at end (and if call
; with this will return error)R

readdigit

; ----- Strip leading spaces

rdig_strip
        movf    RX_PTR,w        ; RX_PTR points to char to check
        movwf   fsr             ; set fsr to char

        incf    RX_PTR,w        ; increment RX_PTR to wreg
        sublw   RX_BUFFER+d'15' ; see if past end of buffer
        btfss   status,c        ; carry clear if is
        movlw   d'15'           ; set so wraps
        sublw   RX_BUFFER+d'15' ; restore check
        movwf   RX_PTR          ; circularly incremented for next time

        movf    indf,w          ; get char
        addlw   d'256'-' '
        btfsc   status,z        ; skip if not space
        goto    rdig_strip      ; strip those spaces


; ----- Now check if digit, LF, or other ...

        addlw   ' '             ; restore char
        addlw   d'256'-0d       ; check if is LF
        btfsc   status,z
        goto    digit_err2      ; LF - so return error

        addlw   d'256'+0d-'0'   ; subtract '0', if overflows (ie >= '0')
        btfss   status,c        ; carry will be set
        goto    digit_err1

        addlw   d'256'-d'10'    ; subtract 9, if overflows (ie >= 10)
        btfsc   status,c        ; carry will be set
        goto    digit_err1      ; if non-digit then could be error or end

        addlw   d'10'           ; restore last check
        movwf   R12
        clrf    R11             ; success
        goto    rdig_return

digit_err1
        movlw   1
        goto    digit_err
digit_err2
        movlw   2
digit_err
        movwf   R11
        clrf    R12

rdig_return
        movf    RX_PTR,w
        movwf   RX_TMP          ; update RX_TMP
        retlw   0               ; failure (or end of command ...)


; =========================================================================
; ----- Error requiring buffer flush

error_flush
        movf    RX_END,w
        movwf   RX_PTR
        movwf   RX_TMP          ; flush buffer then ...


; =========================================================================
; ----- Echo "Error." to engineering port, then exit

eng_error
        movlw   'E'             ; Send "Error."
        call    send_char
        movlw   'r'
        call    send_char
        movlw   'r'
        call    send_char
        movlw   'o'
        call    send_char
        movlw   'r'
        call    send_char
        movlw   '.'
        call    send_char
        call    send_crlf       ; Send CR LF
        call    initiate        ; set going
        retlw   0               ; Exit


; =========================================================================
; ----- Echo "Fail." to engineering port, then exit


eng_fail
        movlw   'F'             ; Send "Fail."
        call    send_char
        movlw   'a'
        call    send_char
        movlw   'i'
        call    send_char
        movlw   'l'
        call    send_char
        movlw   '.'
        call    send_char
        call    send_crlf       ; Send CR LF
        call    initiate        ; set going
        retlw   0               ; Exit


; =========================================================================
; ----- Echo "FailW." to engineering port, then exit


eng_failw
        movlw   'F'             ; Send "FailW."
        call    send_char
        movlw   'a'
        call    send_char
        movlw   'i'
        call    send_char
        movlw   'l'
        call    send_char
        movlw   'W'
        call    send_char
        movlw   '.'
        call    send_char
        call    send_crlf       ; Send CR LF
        call    initiate        ; set going
        retlw   0               ; Exit


; =========================================================================
; ----- Echo "Done." to engineering port, then exit


eng_done
        movlw   'D'             ; Send "Done."
        call    send_char
        movlw   'o'
        call    send_char
        movlw   'n'
        call    send_char
        movlw   'e'
        call    send_char
        movlw   '.'
        call    send_char
        call    send_crlf       ; Send CR LF
        call    initiate        ; set going
        retlw   0               ; Exit


; =========================================================================
; ----- Echo "OK." to engineering port, then exit


eng_ok
        movlw   'O'             ; Send "OK."
        call    send_char
        movlw   'K'
        call    send_char
        movlw   '.'
        call    send_char
        call    send_crlf       ; Send CR LF
        call    initiate        ; set going
        retlw   0               ; Exit


; =========================================================================
; ---- Sends CRLF out of port

send_crlf
        movlw   0d
        call    send_char
        movlw   0a
        call    send_char
        retlw   0


; =========================================================================
; ----- Routine to break down unsigned number in R1R0 and output to
;       engineering buffer.  Uses digitise routine from lcd.asm.
;
; Alters R0-R6,R8

send_word
        pagesel digitise
        call    digitise        ; break down unsigned number in R1R0 into
        pagesel send_digits     ; R6 R5 R4 R3 R2
send_digits

        movf    R6,w
        movwf   R8
        btfsc   status,z
        goto    done_tenthou
        movf    R6,w
        addlw   '0'
        call    send_char       ; send digit if non-zero
done_tenthou

        movf    R5,w            ;
        iorwf   R8,w            ; check if have had any non-zero yet
        movwf   R8              ;
        btfsc   status,z
        goto    done_thou
        movf    R5,w
        addlw   '0'
        call    send_char       ; send digit if non-zero
done_thou

        movf    R4,w
        iorwf   R8,w
        movwf   R8
        btfsc   status,z
        goto    done_hund
        movf    R4,w
        addlw   '0'
        call    send_char
done_hund

        movf    R3,w
        iorwf   R8,w
        movwf   R8
        btfsc   status,z
        goto    done_ten
        movf    R3,w
        addlw   '0'
        call    send_char
done_ten

        movf    R2,w
        addlw   '0'
        call    send_char

        retlw   0               ; all done


; =========================================================================
; ----- Send character out of engineering port, and increment buffer
;       Character provided in wreg, alters R0.

send_char
        movwf   R0              ; copy wreg into R0
        movf    TX_END,w
        movwf   fsr             ; point towards next tx entry position
        movf    R0,w
        movwf   indf            ; store character

        incf    TX_END,w        ; increment TX_END to wreg
        sublw   TX_BUFFER+d'15' ; see if past end of buffer
        btfss   status,c        ; carry clear if is
        movlw   d'15'           ; set so wraps
        sublw   TX_BUFFER+d'15' ; restore check
        movwf   TX_END          ; circularly incremented it

        retlw   0


; =========================================================================
; ----- Commence a transmission if necessary.
;
; Called when need to start off, or ensure, a transmission.  When interrupt
; routine runs out of characters it will clear txie.  The power down bit
; TXEN will be cleared by a 1 second countdown after this.
;
; This routine sets both to 1, thus starting off (or keeping going) a
; transmission.


initiate
        bsf     status,rp0      ; ... to bank 1   <--- **** NOTE ****
        bsf     txsta,txen      ; enable tx, also sets interrupt flag
        bsf     pie1,txie       ; enable interrupt (if not already going)
        bcf     status,rp0      ; ... to bank 0   <--- **** NOTE ****
        retlw   0


; =========================================================================
;                       ENGINEERING ROUTINE TABLES
; =========================================================================

;
; Jump table in own area so know in 256 byte boundary

_engtbl code

cmd_jmp
        addwf   pcl,f           ; add offset to go to jump vector
        goto    cmd0
        goto    cmd1
        goto    cmd2
        goto    cmd3
        goto    cmd4
        goto    cmd5
        goto    cmd6


; =========================================================================


; Following table contains a list of the commands.  Pointer given in wreg
; and returns character from that position.  Commands separated by zero
; bytes.
;
;       0  = tbd
;       1  = rd
;       2  = wr
;       3  = send
;       4  = dump
;       5  = reset
;       6  = adc

cmd_table
        addwf   pcl,f
        retlw   't'          ; cannot start with a zero!
        retlw   'b'
        retlw   'd'
        retlw   0
        retlw   'r'
        retlw   'd'
        retlw   0
        retlw   'w'
        retlw   'r'
        retlw   0
        retlw   's'
        retlw   'e'
        retlw   'n'
        retlw   'd'
        retlw   0
        retlw   'd'
        retlw   'u'
        retlw   'm'
        retlw   'p'
        retlw   0
        retlw   'r'
        retlw   'e'
        retlw   's'
        retlw   'e'
        retlw   't'
        retlw   0
        retlw   'a'
        retlw   'd'
        retlw   'c'
        retlw   0       ; need this last zero, even though ...
        retlw   0ff     ; end of table


; ==========================================================================


        global  service_uart, initiate


        end
